package fi.helsinki.cs.seco.onki.config;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

public class Configuration implements IOnkiConfiguration {

	protected Resource configResource = null;
	protected OnkiConfig onkiConfig = null;
	protected Configuration parentConfig = null;

	public void setConfigResource(Resource r) {
		this.configResource = r;
	}
	
	public Resource getConfigResource() {
		return this.configResource;
	}
	
	public void setOnkiConfig(OnkiConfig oc) {
		this.onkiConfig = oc;
	}
	
	public void setParent(Configuration conf) {
		this.parentConfig = conf;
	}
	
	protected String getPropertyAsString(String localname) {
		RDFNode node = this.getProperty(localname);
		if (node == null)
			return null;
		else
			return node.toString();
	}

	protected Resource getPropertyAsResource(String localname) {
		RDFNode node = this.getProperty(localname);
		if (node.canAs(Resource.class))
			return (Resource)node;
		else
			return null;
	}

	protected RDFNode getProperty(String localname) {
		return this.getProperty(this.onkiConfig.getOnkiConfProperty(localname));
	}
	
	protected RDFNode getProperty(Property p) {
		Statement stmt = this.configResource.getProperty(p);
		
		if (stmt == null) {
//			System.err.println("WARNING: No such property "+localname+" in config "+this.configresource.getLocalName());
			return null;
		}
		return stmt.getObject();
	}
	
	protected Map<String,String> getMultilingualLiterals(String p) {
		return this.getMultilingualLiterals(this.onkiConfig.getOnkiConfProperty(p));
	}
	
	protected Map<String,String> getMultilingualLiterals(Property p) {
		Map<String,String> resmap = new HashMap<String,String>();
		StmtIterator stmtI = this.configResource.listProperties(p);
		while (stmtI.hasNext()) {
			RDFNode node = stmtI.nextStatement().getObject();
			if (node.isLiteral()) {
				Literal lit = (Literal)node.as(Literal.class);
				resmap.put(lit.getLanguage(), lit.getString());
			}
		}
		return resmap;
	}
	
	protected boolean hasParentConfiguration() {
		return this.parentConfig != null;
	}
	
	protected boolean hasParentConfiguration(Class cls) {
		return this.parentConfig.getClass().isAssignableFrom(cls);
	}
	
	protected String addSlashToDir(String dir) {
		if (dir != null && !dir.endsWith("/"))
			dir += "/";
		return dir;
	}
	
	protected String getDir() {
		String dir = this.addSlashToDir(this.getPropertyAsString("dir"));
		if (dir == null)
			return null;
		if (!dir.contains(":") && this.hasParentConfiguration(OnkiServerConfiguration.class)) {
			OnkiServerConfiguration osc = (OnkiServerConfiguration)this.parentConfig;
			dir = osc.getHomeDir() + dir;
		}
		return dir;
	}
	
	protected String getFilePath(String id, boolean addDir) {
		String file = this.getPropertyAsString(id);
		if (file != null && addDir && this.getDir() != null)
			file = this.getDir() + file;
		return file;
	}
	
	
	
	public String toString() {
		String str = "**** instance of "+this.getClass().getName()+" ****\n";
		Method[] methods = this.getClass().getMethods();
		for (Method m: methods) {
			String methodname = m.getName();
			if (methodname.startsWith("get") && methodname != "getConfigResource" && methodname != "getClass") {
				try {
					str += m.getName()+" => "+m.invoke(this, new Object[] {})+"\n";
				} catch (IllegalArgumentException e) {
				} catch (IllegalAccessException e) {
				} catch (InvocationTargetException e) {
				}
			}
		}
		return str;
	}
}
