package fi.helsinki.cs.seco.onki.index;

/**
 * Represents an item to be stored in the alphabetical index.
 * 
 * @author oisuomin
 */

public class IndexItem {
	private String uriNamespace;
	private String uriNamespacePrefix;
	private String uriLocalname;
	private String language;
	private String prefLabel;
	private Iterable<String> altLabels;
	private Iterable<String> hiddenLabels;	
	private Iterable<String> types;
	private Iterable<String> superClasses;
	private Iterable<String> groups;
	private boolean isGroup;
	private Iterable<String> queryExpansions;

	public IndexItem(String uriNamespace, String uriNamespacePrefix, String uriLocalname, String language, String prefLabel, Iterable<String> altLabels, Iterable<String> hiddenLabels, Iterable<String> types, Iterable<String> superClasses, Iterable<String> groups, boolean isGroup, Iterable<String> queryExpansions) {
		this.uriNamespace = uriNamespace;
		this.uriNamespacePrefix = uriNamespacePrefix;
		this.uriLocalname = uriLocalname;
		this.language = language;
		this.prefLabel = prefLabel;
		this.altLabels = altLabels;
		this.hiddenLabels = hiddenLabels;
		this.types = types;
		this.superClasses = superClasses;
		this.groups = groups;
		this.isGroup = isGroup;
		this.queryExpansions = queryExpansions;
	}
	
    /**
	 * Getters and setters.
	 */
	
	public String getUri() {
		return this.uriNamespace+this.uriLocalname;
	}
	
	public String getUriNamespace() {
		return this.uriNamespace;
	}

	public String getUriNamespacePrefix() {
		return this.uriNamespacePrefix;
	}
	
	public String getUriLocalname() {
		return this.uriLocalname;
	}
	
	public String getLanguage() {
		return this.language;
	}
	
	public String getPrefLabel() {
		return this.prefLabel;
	}
	
	public Iterable<String> getAltLabels() {
		return this.altLabels;
	}

	public Iterable<String> getHiddenLabels() {
		return this.hiddenLabels;
	}
	
	public Iterable<String> getSuperClasses() {
		return this.superClasses;
	}
	
	public Iterable<String> getTypes() {
		return this.types;
	}
	
	public Iterable<String> getGroups() {
		return this.groups;
	}
	
	public boolean isGroup() {
		return this.isGroup;
	}
	
	public Iterable<String> getQueryExpansions() {
		return this.queryExpansions;
	}
	
	public String toString() {
		String s = "IndexItem(\"" + this.getUri() + "\", \"" + language + "\", \"" + prefLabel + "\", ";
		s+= this.iterableToString(getAltLabels()) +", ";
		s+= this.iterableToString(getSuperClasses()) +", ";
		s+= this.iterableToString(getTypes()) +")";
		return s;
	}
	
	private String iterableToString(Iterable<String> iter) {
		String s = "[";
		int idx = 0;
		for (String str: iter) {
			if (idx != 0) s += ",";
			s += "\"" + str + "\"";
			++idx;
		}
		s += "]";
		return s;
	}
}