package fi.helsinki.cs.seco.onki.logic;

import java.math.BigDecimal;
import java.math.MathContext;

/**
 * Represents a weighted mapping between two concepts.
 */

public class OnkiConceptMapping {

		private OnkiResource target;
		private double overlaps;
		private double overlappedBy;
		
		public OnkiConceptMapping(OnkiResource target) {
			this(target, -1, -1);
		}
		
		public OnkiConceptMapping(OnkiResource target, double overlaps, double overlappedBy) {
			this.target = target;
			this.overlaps = overlaps;
			this.overlappedBy = overlappedBy;
		}
		
	    /**
		 * Getters and setters.
		 */
		
		public OnkiResource getTarget() {
			return this.target;
		}
		
		public double getOverlaps() {
			return this.overlaps;
		}
		
		public double getOverlappedBy() {
			return this.overlappedBy;
		}
		
		public BigDecimal getOverlapsRounded() {
			return new BigDecimal(this.overlaps).round(new MathContext(3));
		}
		
		public BigDecimal getOverlappedByRounded() {
			return new BigDecimal(this.overlappedBy).round(new MathContext(3));
		}
		
		public double getConfidence() {
			return this.overlaps * this.overlappedBy;
		}
		
		public BigDecimal getConfidenceRounded() {
			return new BigDecimal(this.overlaps * this.overlappedBy).round(new MathContext(3));
		}
}