package fi.helsinki.cs.seco.onki.logic;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.ConfigurationFactory;

/**
 * Onki configuration utility.
 * This is merely a wrapper for Apache Commons configuration library.
 *
 * @author vpkomula
 */
public class OnkiConfigurator {

    //////////////////////////////////////////////////////////
    //	Static Constants
    //////////////////////////////////////////////////////////
	
    public static final String SETTINGS_FILE_LOGGER      = "WEB-INF/classes/log4j.xml";
    public static final String SETTINGS_FILE_APP         = "WEB-INF/app/conf/onki-root-conf.xml";

    //////////////////////////////////////////////////////////
    //	Configuration File Element keys
    //////////////////////////////////////////////////////////

    //	Common element attributes
    public static final String IMPL                                 = "[@impl]";
    public static final String ENABLED                              = "[@enabled]";
    public static final String DEFAULT                              = "[@default]";
    public static final String TYPE                              	= "[@type]";

    public static final String ELEM_ROOT 		                    = "";

    //	Class and property keys
    public static final String ELEM_CLASSES                         = ELEM_ROOT + ".classes";
    public static final String ELEM_CLASSES_LABELS                  = ELEM_CLASSES + ".labels";
    public static final String ELEM_CLASSES_LABEL                   = ELEM_CLASSES_LABELS + ".label";
    public static final String ELEM_CLASSES_FOLLOW                  = ELEM_CLASSES + ".follow";

    public static final String ELEM_CLASSES_ALPHA_INDEX             = ELEM_CLASSES + ".alpha-index";
    // apache commons configuration does not support elements with comma separated list of values and attributes at the same time, Jouni 7.7.2009
    //public static final String ELEM_CLASSES_ALPHA_INDEX_DEFAULT     = ELEM_CLASSES_ALPHA_INDEX + DEFAULT;
    public static final String ELEM_CLASSES_ALPHA_INDEX_DEFAULT     = ELEM_CLASSES_ALPHA_INDEX + "-default";
    public static final String ELEM_CLASSES_ONTOLOGY_LABEL			= ELEM_CLASSES + ".ontology-label";
    public static final String ELEM_CLASSES_ONTOLOGY_LABEL_LANG		= ELEM_CLASSES_ONTOLOGY_LABEL + "[@xml:lang]";
    public static final String ELEM_CLASSES_ONTOLOGY_SUBLABEL		= ELEM_CLASSES + ".ontology-sublabel";
    public static final String ELEM_CLASSES_ONTOLOGY_SUBLABEL_LANG	= ELEM_CLASSES_ONTOLOGY_SUBLABEL + "[@xml:lang]";
    public static final String ELEM_CLASSES_ADDITIONALS             = ELEM_CLASSES + ".additionals";
    public static final String ELEM_CLASSES_ADDITIONAL              = ELEM_CLASSES_ADDITIONALS + ".additional";
    public static final String ELEM_CLASSES_HIDDEN_PROPERTIES		= ELEM_CLASSES + ".hidden-properties";
    public static final String ELEM_CLASSES_HIDDEN_PROPERTY			= ELEM_CLASSES_HIDDEN_PROPERTIES + ".property";
    
    public static final String ELEM_CLASSES_REST_API				= ELEM_CLASSES + ".rest-api";
    public static final String ELEM_CLASSES_REST_API_GET_PROPERTIES	= ELEM_CLASSES_REST_API + ".get-properties";
    public static final String ELEM_CLASSES_REST_API_PROPERTY_ORDER = ELEM_CLASSES_REST_API_GET_PROPERTIES + ".property-order";
    public static final String ELEM_CLASSES_REST_API_PROPERTY_ORDER_PROPERTY	= ELEM_CLASSES_REST_API_PROPERTY_ORDER + ".property";
    public static final String ELEM_CLASSES_REST_API_HIDDEN_PROPERTIES	= ELEM_CLASSES_REST_API_GET_PROPERTIES + ".hidden-properties";
    public static final String ELEM_CLASSES_REST_API_HIDDEN_PROPERTY	= ELEM_CLASSES_REST_API_HIDDEN_PROPERTIES + ".property";
    public static final String ELEM_CLASSES_REST_API_GET_FULL_PRESENTATION	= ELEM_CLASSES_REST_API + ".get-full-presentation";
    public static final String ELEM_CLASSES_REST_API_HIERARCHY_SIZE = ELEM_CLASSES_REST_API_GET_FULL_PRESENTATION + ".hierarchy-size";
    public static final String ELEM_CLASSES_REST_API_HIERARCHY_SIZE_PARENTS = ELEM_CLASSES_REST_API_HIERARCHY_SIZE + ".parents";
    public static final String ELEM_CLASSES_REST_API_HIERARCHY_SIZE_CHILDREN = ELEM_CLASSES_REST_API_HIERARCHY_SIZE + ".children";
    public static final String ELEM_CLASSES_REST_API_HIERARCHY_SIZE_SIBLINGS = ELEM_CLASSES_REST_API_HIERARCHY_SIZE + ".siblings";
    public static final String ELEM_CLASSES_REST_API_EMPTY_METHODS = ELEM_CLASSES_REST_API_GET_FULL_PRESENTATION + ".empty-methods";
    public static final String ELEM_CLASSES_REST_API_EMPTY_METHOD = ELEM_CLASSES_REST_API_EMPTY_METHODS + ".method";
    
    public static final String ELEM_CLASSES_INDEX_NAMESPACES		= ELEM_CLASSES + ".index-namespaces";
    public static final String ELEM_CLASSES_INDEX_NAMESPACES_LANG	= ELEM_CLASSES_INDEX_NAMESPACES + "[@lang-spec]";
    public static final String ELEM_CLASSES_INDEX_NAMESPACE			= ELEM_CLASSES_INDEX_NAMESPACES + ".namespace";
    public static final String ELEM_CLASSES_INDEX_ALT_LABELS		= ELEM_CLASSES + ".index-alt-labels";
    public static final String ELEM_CLASSES_INDEX_ALT_LABEL			= ELEM_CLASSES_INDEX_ALT_LABELS + ".label";
    
	public static final String ELEM_CLASSES_INDEX_HIDDEN_LABELS		= ELEM_CLASSES + ".index-hidden-labels";
	public static final String ELEM_CLASSES_INDEX_HIDDEN_LABEL			= ELEM_CLASSES_INDEX_HIDDEN_LABELS + ".label";
	public static final String ELEM_CLASSES_INDEX_LABELS		= ELEM_CLASSES + ".index-labels";
	public static final String ELEM_CLASSES_INDEX_LABEL			= ELEM_CLASSES_INDEX_LABELS + ".label";

    public static final String ELEM_CLASSES_PROPERTY_FIDDLERS		= ELEM_CLASSES + ".property-fiddlers";
    public static final String ELEM_CLASSES_PROPERTY_FIDDLERS_FIDDLER = ELEM_CLASSES_PROPERTY_FIDDLERS + ".fiddler";
    public static final String ELEM_CLASSES_MAPPING_FIDDLERS		= ELEM_CLASSES + ".mapping-fiddlers";
    public static final String ELEM_CLASSES_MAPPING_FIDDLERS_FIDDLER = ELEM_CLASSES_MAPPING_FIDDLERS + ".fiddler";
    public static final String ELEM_CLASSES_GROUP					= ELEM_CLASSES + ".group-search";
    public static final String ELEM_CLASSES_GROUP_ENABLED			= ELEM_CLASSES_GROUP + ENABLED;
    public static final String ELEM_CLASSES_GROUP_SEARCHABLE			= ELEM_CLASSES_GROUP + "[@searchable]";
    public static final String ELEM_CLASSES_GROUP_DEFAULT_QUERY			= ELEM_CLASSES_GROUP + "[@default_query]";
    public static final String ELEM_CLASSES_GROUP_IS_GROUP_PROPERTY	= ELEM_CLASSES_GROUP + ".isGroupProperty";
    public static final String ELEM_CLASSES_GROUP_IS_GROUP_PROPERTY_TYPE = ELEM_CLASSES_GROUP_IS_GROUP_PROPERTY + TYPE;
    public static final String ELEM_CLASSES_GROUP_IS_GROUP_VALUE = ELEM_CLASSES_GROUP + ".isGroupValue";
    public static final String ELEM_CLASSES_GROUP_IS_GROUP_VALUE_TYPE = ELEM_CLASSES_GROUP_IS_GROUP_VALUE + TYPE;
    public static final String ELEM_CLASSES_GROUP_BELONGS_TO_GROUP_PROPERTY	= ELEM_CLASSES_GROUP + ".belongsToGroupProperty";
    public static final String ELEM_CLASSES_GROUP_BELONGS_TO_GROUP_PROPERTY_TYPE	= ELEM_CLASSES_GROUP_BELONGS_TO_GROUP_PROPERTY + TYPE;
    public static final String ELEM_CLASSES_TYPE_RESTRICTION					= ELEM_CLASSES + ".type-restriction";
    public static final String ELEM_CLASSES_TYPE_RESTRICTION_ENABLED					= ELEM_CLASSES_TYPE_RESTRICTION + ENABLED;
    public static final String ELEM_CLASSES_TYPE_RESTRICTION_INCLUSIVE					= ELEM_CLASSES_TYPE_RESTRICTION + "[@inclusive]";
    public static final String ELEM_CLASSES_TYPE_RESTRICTION_TYPE					= ELEM_CLASSES_TYPE_RESTRICTION + ".type";
    
    public static final String ELEM_CLASSES_QUERY_EXPANSION					= ELEM_CLASSES + ".query-expansion";
    public static final String ELEM_CLASSES_QUERY_EXPANSION_ENABLED			= ELEM_CLASSES_QUERY_EXPANSION + ENABLED;
    public static final String ELEM_CLASSES_QUERY_EXPANSION_PROPERTY			= ELEM_CLASSES_QUERY_EXPANSION + ".property";
    public static final String ELEM_CLASSES_QUERY_EXPANSION_PROPERTY_INVERSE			= ELEM_CLASSES_QUERY_EXPANSION_PROPERTY + "[@inverse]";
    public static final String ELEM_CLASSES_QUERY_EXPANSION_PROPERTY_SYMMETRIC			= ELEM_CLASSES_QUERY_EXPANSION_PROPERTY + "[@symmetric]";
    public static final String ELEM_CLASSES_QUERY_EXPANSION_PROPERTY_TRANSITIVE			= ELEM_CLASSES_QUERY_EXPANSION_PROPERTY + "[@transitive]";
    public static final String ELEM_CLASSES_URL_PROPERTIES = ELEM_CLASSES + ".url-properties";
    public static final String ELEM_CLASSES_URL_PROPERTIES_PROPERTY = ELEM_CLASSES_URL_PROPERTIES + ".property";
    public static final String ELEM_CLASSES_RELATED_PROPERTIES = ELEM_CLASSES + ".related-properties";
    public static final String ELEM_CLASSES_RELATED_PROPERTIES_PROPERTY = ELEM_CLASSES_RELATED_PROPERTIES + ".property";
    public static final String ELEM_CLASSES_DESCRIPTION_PROPERTIES = ELEM_CLASSES + ".description-properties";
    public static final String ELEM_CLASSES_DESCRIPTION_PROPERTIES_PROPERTY = ELEM_CLASSES_DESCRIPTION_PROPERTIES + ".property";
    public static final String ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES = ELEM_CLASSES + ".equivalent-class-properties";
    public static final String ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES_PROPERTY = ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES + ".property";
    public static final String ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES_PROPERTY_USE_IN_HIERARCHY = ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES_PROPERTY + "[@use-in-hierarchy]";
    public static final String ELEM_CLASSES_MAP_PROPERTIES = ELEM_CLASSES + ".map-properties";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_GOOGLE_MAPS_KEY = ELEM_CLASSES_MAP_PROPERTIES + ".google-maps-key";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_POLYGON_PROPERTY = ELEM_CLASSES_MAP_PROPERTIES + ".polygon-property";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY = ELEM_CLASSES_MAP_PROPERTIES + ".point-property";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY_RADIUS = ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY + "[@radius]";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY_LAT_PROPERTY = ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY + ".lat-property";
    public static final String ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY_LONG_PROPERTY = ELEM_CLASSES_MAP_PROPERTIES_POINT_PROPERTY + ".long-property";
    public static final String ELEM_CLASSES_SKOS_COLLECTION_AS_NODE_LABEL = ELEM_CLASSES + ".skos-collection-as-node-label";
    public static final String ELEM_CLASSES_SKOS_COLLECTION_AS_NODE_LABEL_ENABLED = ELEM_CLASSES_SKOS_COLLECTION_AS_NODE_LABEL + ENABLED;
    
    //	Visualization
    public static final String ELEM_VISUALIZATION                   = ELEM_ROOT + ".visualization";
    public static final String ELEM_VISUALIZATION_DEFAULT_LOCALE    = ELEM_VISUALIZATION + ".default-locale";

    //	Localization
    public static final String ELEM_VISUALIZATION_LOCALES           = ELEM_VISUALIZATION + ".locales";
    public static final String ELEM_VISUALIZATION_LOCALES_DEFAULT   = ELEM_VISUALIZATION_LOCALES + DEFAULT;
    public static final String ELEM_VISUALIZATION_LOCALE            = ELEM_VISUALIZATION_LOCALES + ".locale";
    public static final String ELEM_VISUALIZATION_LOCALE_ID         =  "[@id]";

    // Languages
    public static final String ELEM_LANGUAGES                       = ELEM_ROOT + ".languages";
    public static final String ELEM_LANGUAGES_PREFERENCE            = ELEM_LANGUAGES + ".preference";

    // MultiOnki
    public static final String ELEM_MULTIONKI						= ELEM_ROOT + ".multionki";
    public static final String ELEM_MULTIONKI_ENABLED				= ELEM_MULTIONKI + ENABLED;
    public static final String ELEM_MULTIONKI_EMPHASIZING			= ELEM_MULTIONKI + "[@emphasizing]";
    public static final String ELEM_MULTIONKI_NOMAPPINGSINHIERARCHY	= ELEM_MULTIONKI + "[@doNotUseMappingsInHierarchyGeneration]";

    // Default view
    public static final String ELEM_DEFAULTVIEW                     = ELEM_ROOT + ".defaultview";
    public static final String ELEM_DEFAULTVIEW_CONCEPT             = ELEM_DEFAULTVIEW + ".concept";
    public static final String ELEM_DEFAULTVIEW_TYPE                = ELEM_DEFAULTVIEW + ".type";
    
    // Index
    public static final String ELEM_INDEX							= ELEM_ROOT + ".index";
    public static final String ELEM_INDEX_TYPE						= ELEM_INDEX + "[@type]";
    public static final String ELEM_INDEX_PATH						= ELEM_INDEX + "[@path]";
    
    // Ontology info page
    public static final String ELEM_ONTOLOGY_INFO_PAGE				= ELEM_ROOT + ".ontology-info-page";
    public static final String ELEM_ONTOLOGY_INFO_PAGE_ENABLED		= ELEM_ONTOLOGY_INFO_PAGE + ENABLED;
    
    ////////////////////////////////////////////////////////
    //	Private variables
    ////////////////////////////////////////////////////////

    private static OnkiConfigurator instance;
    private Configuration config;

    ////////////////////////////////////////////////////////
    //	Constructors
    ////////////////////////////////////////////////////////

    /** 
     * prevent instantation
     */
    private OnkiConfigurator() {;}

    /**
     *
     * @param confFile
     * @throws FileNotFoundException
     * @throws IOException
     * @throws ConfigurationException
     */
    private OnkiConfigurator(String confFile) throws FileNotFoundException, IOException, ConfigurationException {
        ConfigurationFactory factory = new ConfigurationFactory();
        File cnfFile = new File(confFile);

        if (!cnfFile.exists())
			throw new FileNotFoundException("Configuration file '" + confFile + "' does not exist");
		else if (!cnfFile.canRead())
			throw new IOException("Configuration file '" + confFile + "' can't be read, permissions ok?");
		else if (cnfFile.isDirectory()) throw new IOException("Configuration file '" + confFile + "'is not a file (is a dir)");

        URL configURL = cnfFile.toURL();
        factory.setConfigurationURL(configURL);
        config = factory.getConfiguration();
    }

    ////////////////////////////////////////////////////////
    //	Public accessors
    ////////////////////////////////////////////////////////

    /**
     * Get the Apache Commons Configuration object for reading configuration.
     *
     * @param confFile
     * @return
     * @throws FileNotFoundException
     * @throws IOException
     * @throws ConfigurationException
     */
    public static Configuration getConfiguration(String confFile) throws FileNotFoundException, IOException,ConfigurationException {
        if (instance == null) instance = new OnkiConfigurator(confFile);
        return instance.config;
    }

    /**
	 * Get the "type" attribute associated to n:th property value of some configuration
	 * property, e.g. follow hierarchy property. 
	 */
    
    public static String getType(String confProperty, int number) {
    	return instance.config.getString(confProperty+"("+number+")[@type]");
    }
    
    /**
	 * Should an equivalent class property configured as n:th property value be used
	 * for concept hierarchy generation.
	 */
    
    public static boolean isUsedInHierarchy(int number) {
    	return instance.config.getBoolean(ELEM_CLASSES_EQUIVALENT_CLASS_PROPERTIES_PROPERTY+"("+number+")[@use-in-hierarchy]", false);
    }
}