package fi.helsinki.cs.seco.onki.logic.mappingfiddlers;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;

import fi.helsinki.cs.seco.onki.logic.IMappingFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiConceptMapping;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;

/**
 * Generates the weighted overlap mappings between SAPO place spaceworms.
 */

public class SapoMappingFiddler implements IMappingFiddler {

	private static final String NS_ONKI = "http://demo.seco.tkk.fi/onki/ns/property#";
	private static Property overlapsNewerRegion = ResourceFactory.createProperty(NS_ONKI + "overlapsNewerRegion");
	private static Property overlapsOlderRegion = ResourceFactory.createProperty(NS_ONKI + "overlapsOlderRegion");
	
	public Map<OnkiResource, Collection<OnkiConceptMapping>> findMappings(
			OnkiResource ores) {
		
		LinkedHashMap<OnkiResource, Collection<OnkiConceptMapping>> map = null;
		
		String SPARQLPrefixes = "PREFIX spatiotemporal:  <http://www.yso.fi/onto/mapping/geo/spatiotemporal/>"
			+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>";
		
		String uri = ores.getUri();

		Model model = ores.getResource().getModel();
		/*
		String sparql =

    		SPARQLPrefixes
    				+ "SELECT DISTINCT ?mapinstance ?overlaps ?overlappedby ?argument2 ?argument1 "
    				+ " WHERE { {?mapinstance spatiotemporal:argument1 <"
    				+ uri
    				+ "> ;"
    				+ "              spatiotemporal:argument1 ?argument1 ;"
    				+ "              spatiotemporal:argument2 ?argument2 ;"
    				+ "              spatiotemporal:overlaps ?overlaps ;"
    				+ "              spatiotemporal:overlappedBy ?overlappedby ."
    				+
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				// +
    				// " FILTER (?overlappedby >= 0.001)" +
    				"}" + "UNION " + " {?mapinstance spatiotemporal:argument2 <"
    				+ uri + "> ;"
    				+ "               spatiotemporal:argument2 ?argument2 ;"
    				+ "               spatiotemporal:argument1 ?argument1 ;"
    				+ "               spatiotemporal:overlaps ?overlaps ;"
    				+ "               spatiotemporal:overlappedBy ?overlappedby ." +
    				// " FILTER (?overlappedby >= 0.001)" +
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				// +
    				"}" + "} " + 
    				//"ORDER BY DESC(?overlaps) DESC(?overlappedby)";
    				"ORDER BY DESC(xsd:decimal(?overlaps)) DESC(xsd:decimal(?overlappedby))";
		*/
		// 1) find mappings to newer regions
		
		String sparql1 =
    		SPARQLPrefixes
    				+ "SELECT DISTINCT ?argument2 ?overlaps ?overlappedby"
    				+ " WHERE {?mapinstance spatiotemporal:argument1 <"
    				+ uri
    				+ "> ;"
    				+ "              spatiotemporal:argument2 ?argument2 ;"
    				+ "              spatiotemporal:overlaps ?overlaps ;"
    				+ "              spatiotemporal:overlappedBy ?overlappedby ."
    				+
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				"}" +
    				"ORDER BY DESC(xsd:decimal(?overlaps)) DESC(xsd:decimal(?overlappedby))";
		
		Query query = QueryFactory.create(sparql1);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		
		LinkedList<OnkiConceptMapping> mappings = new LinkedList<OnkiConceptMapping>();
		
		while (results.hasNext()) {
			QuerySolution qs = results.nextSolution();
			RDFNode targetNode = qs.get("argument2");
			RDFNode overlapsNode = qs.get("overlaps");
			RDFNode overlappedByNode = qs.get("overlappedby");
			if (targetNode.isURIResource() && overlapsNode.isLiteral() && overlappedByNode.isLiteral()) {
				Resource target = (Resource)targetNode;
				double overlaps = ((Literal)overlapsNode).getDouble();
				double overlappedBy = ((Literal)overlappedByNode).getDouble();
				mappings.add(new OnkiConceptMapping(new OnkiResource(target, ores.getLangPreferences()), overlaps, overlappedBy));
			}
		}
		if (!mappings.isEmpty()) {
			map = new LinkedHashMap<OnkiResource, Collection<OnkiConceptMapping>>();
			map.put(new OnkiResource(overlapsNewerRegion, ores.getLangPreferences()), mappings);
		}

		// 2) find mappings to older regions
		String sparql2 =
    		SPARQLPrefixes
    				+ "SELECT DISTINCT ?argument1 ?overlappedby ?overlaps"
    				+ " WHERE {?mapinstance spatiotemporal:argument2 <"
    				+ uri
    				+ "> ;"
    				+ "              spatiotemporal:argument1 ?argument1 ;"
    				+ "              spatiotemporal:overlappedBy ?overlappedby ;"
    				+ "              spatiotemporal:overlaps ?overlaps ."
    				+
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				"}" +
    				"ORDER BY DESC(xsd:decimal(?overlappedby)) DESC(xsd:decimal(?overlaps))";
		
		Query query2 = QueryFactory.create(sparql2);
		QueryExecution qe2 = QueryExecutionFactory.create(query2, model);
		ResultSet results2 = qe2.execSelect();
		
		LinkedList<OnkiConceptMapping> mappings2 = new LinkedList<OnkiConceptMapping>();
		
		while (results2.hasNext()) {
			QuerySolution qs = results2.nextSolution();
			RDFNode targetNode = qs.get("argument1");
			RDFNode overlapsNode = qs.get("overlappedby");
			RDFNode overlappedByNode = qs.get("overlaps");
			if (targetNode.isURIResource() && overlapsNode.isLiteral() && overlappedByNode.isLiteral()) {
				Resource target = (Resource)targetNode;
				double overlaps = ((Literal)overlapsNode).getDouble();
				double overlappedBy = ((Literal)overlappedByNode).getDouble();
				mappings2.add(new OnkiConceptMapping(new OnkiResource(target, ores.getLangPreferences()), overlaps, overlappedBy));
			}
		}
		if (!mappings2.isEmpty()) {
			if (map == null)
				map = new LinkedHashMap<OnkiResource, Collection<OnkiConceptMapping>>();
			map.put(new OnkiResource(overlapsOlderRegion, ores.getLangPreferences()), mappings2);
		}
		
		return map;
	}
}