package fi.helsinki.cs.seco.onki.logic.mappingfiddlers;

import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDFS;

import fi.helsinki.cs.seco.onki.logic.IMappingFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiConceptMapping;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;

/**
 * Generates the weighted co-occurence mappings between YSO concepts.
 */

public class YsoMappingFiddler implements IMappingFiddler {

	private static final String NS_ONKI = "http://demo.seco.tkk.fi/onki/ns/property#";
	private static Property coversProp = ResourceFactory.createProperty(NS_ONKI + "confidence");
	
	public Map<OnkiResource, Collection<OnkiConceptMapping>> findMappings(
			OnkiResource ores) {
		
		LinkedHashMap<OnkiResource, Collection<OnkiConceptMapping>> map = null;
		
		String SPARQLPrefixes = "PREFIX setmapping:  <http://www.yso.fi/onto/mapping/>"
			+ "PREFIX xsd: <http://www.w3.org/2001/XMLSchema#>";
		
		String uri = ores.getUri();

		Model model = ores.getResource().getModel();
		/*
		String sparql =

    		SPARQLPrefixes
    				+ "SELECT DISTINCT ?mapinstance ?overlaps ?overlappedby ?argument2 ?argument1 "
    				+ " WHERE { {?mapinstance spatiotemporal:argument1 <"
    				+ uri
    				+ "> ;"
    				+ "              spatiotemporal:argument1 ?argument1 ;"
    				+ "              spatiotemporal:argument2 ?argument2 ;"
    				+ "              spatiotemporal:overlaps ?overlaps ;"
    				+ "              spatiotemporal:overlappedBy ?overlappedby ."
    				+
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				// +
    				// " FILTER (?overlappedby >= 0.001)" +
    				"}" + "UNION " + " {?mapinstance spatiotemporal:argument2 <"
    				+ uri + "> ;"
    				+ "               spatiotemporal:argument2 ?argument2 ;"
    				+ "               spatiotemporal:argument1 ?argument1 ;"
    				+ "               spatiotemporal:overlaps ?overlaps ;"
    				+ "               spatiotemporal:overlappedBy ?overlappedby ." +
    				// " FILTER (?overlappedby >= 0.001)" +
    				// " FILTER ((xsd:decimal(?overlappedby)) > (xsd:decimal(0.5)))"
    				// +
    				"}" + "} " + 
    				//"ORDER BY DESC(?overlaps) DESC(?overlappedby)";
    				"ORDER BY DESC(xsd:decimal(?overlaps)) DESC(xsd:decimal(?overlappedby))";
		*/
		
		// find mappings
		
		String sparql =

    		SPARQLPrefixes
    				+ "SELECT DISTINCT ?argument1 ?argument2 ?covers ?coveredby "
    				+ " WHERE { {?mapinstance setmapping:argument1 <"
    				+ uri
    				+ "> ;"
    				+ "              setmapping:argument1 ?argument1 ;"
    				+ "              setmapping:argument2 ?argument2 ;"
    				+ "              setmapping:covers ?covers ;"
    				+ "              setmapping:coveredBy ?coveredby ."
    				
    				+
    				// " FILTER ((xsd:decimal(?covers) * xsd:decimal(?coveredby)) > (xsd:decimal(0.05)))"
    				// +
    				"}" + "UNION " + " {?mapinstance setmapping:argument2 <"
    				+ uri + "> ;"
    				+ "               setmapping:argument2 ?argument2 ;"
    				+ "               setmapping:argument1 ?argument1 ;"
    				+ "               setmapping:covers ?covers ;"
    				+ "               setmapping:coveredBy ?coveredby ." +
    				// " FILTER ((xsd:decimal(?covers) * xsd:decimal(?coveredby)) > (xsd:decimal(0.05)))"
    				// +
    				"}" + "} " + 
    				"ORDER BY DESC(xsd:decimal(?covers) * xsd:decimal(?coveredby))";
		
		Query query = QueryFactory.create(sparql);
		QueryExecution qe = QueryExecutionFactory.create(query, model);
		ResultSet results = qe.execSelect();
		
		LinkedList<OnkiConceptMapping> mappings = new LinkedList<OnkiConceptMapping>();
		
		LinkedList<String> uris = new LinkedList<String>();
		
		while (results.hasNext()) {
			QuerySolution qs = results.nextSolution();
			RDFNode targetNode1 = qs.get("argument1");
			RDFNode targetNode2 = qs.get("argument2");
			RDFNode coversNode = qs.get("covers");
			RDFNode coveredByNode = qs.get("coveredby");
			if (targetNode1.isURIResource() && targetNode2.isURIResource() && coversNode.isLiteral() && coveredByNode.isLiteral()) {
				Resource target1 = (Resource)targetNode1;
				Resource target2 = (Resource)targetNode2;
				Resource target;
				double covers;
				double coveredBy;
				if (target1.getURI().equals(uri)) {
					target = target2;
					covers = ((Literal)coveredByNode).getDouble();
					coveredBy = ((Literal)coversNode).getDouble();
				}
				else {
					target = target1;
					covers = ((Literal)coversNode).getDouble();
					coveredBy = ((Literal)coveredByNode).getDouble();
				}
				
				// temporary hack
				boolean redundant = false;
				Property related = ResourceFactory.createProperty("http://www.yso.fi/onto/yso-meta/2007-03-02/associativeRelation");
				
				if (model.listStatements(ores.getResource(), RDFS.subClassOf, target).hasNext())
					redundant = true;
				else if (model.listStatements(target, RDFS.subClassOf, ores.getResource()).hasNext())
					redundant = true;
				else if (model.listStatements(ores.getResource(), related, target).hasNext())
					redundant = true;
				else if (model.listStatements(target, related, ores.getResource()).hasNext())
					redundant = true;
				else {
					for (StmtIterator iter = model.listStatements(ores.getResource(), RDFS.subClassOf, (RDFNode)null); iter.hasNext(); ) {
						if (model.listStatements(target, RDFS.subClassOf, iter.nextStatement().getResource()).hasNext())
							redundant = true;
					}
				}
				
				if (!redundant && !uris.contains(target.getURI())) {
					mappings.add(new OnkiConceptMapping(new OnkiResource(target, ores.getLangPreferences()), covers, coveredBy));
					uris.add(target.getURI());
				}
			}
		}
		if (!mappings.isEmpty()) {
			map = new LinkedHashMap<OnkiResource, Collection<OnkiConceptMapping>>();
			map.put(new OnkiResource(coversProp, ores.getLangPreferences()), mappings);
		}
		
		return map;
	}
}