package fi.helsinki.cs.seco.onki.logic.propertyfiddlers;

import java.util.List;
import java.util.LinkedList;

import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.vocabulary.RDFS;

import fi.helsinki.cs.seco.onki.logic.IPropertyFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.logic.OnkiStatement;

/**
 * Generate relations to from preferred terms to nonpreferred terms in IPSV.
 */

public class IPSVAddAltLabelsPropertyFiddler implements IPropertyFiddler {
	
	private static final String NS_IPSV = "http://www.esd.org.uk/standards/ipsv/2.00/ipsv-schema#";
	private static final Property IPSV_useItem = ResourceFactory.createProperty(NS_IPSV+"useItem");
	
	private static final String NS_ONKI = "http://demo.seco.tkk.fi/onki/ns/property#";
	private static final Property ONKI_altLabel = ResourceFactory.createProperty(NS_ONKI+"altLabel");

	/** 
	 * Find backward relations to nonpreferred terms.
	 */
	
	public List<OnkiStatement> findBackward(OnkiResource ores) {
		if (!ores.getResource().isResource())
			return null;
		
		StmtIterator stmti = ores.getResource().getModel().listStatements(null, IPSV_useItem, ores.getResource());
		if (!stmti.hasNext())
			return null;

		List<OnkiStatement> reslist = new LinkedList<OnkiStatement>();
		while (stmti.hasNext()) {
			reslist.add(new OnkiStatement(ores, ONKI_altLabel, stmti.nextStatement().getSubject().getProperty(RDFS.label).getObject(), ores.getLangPreferences(), OnkiStatement.TYPE_INVERSE));
		}
		return reslist;
	}

	// not needed currently
	public List<OnkiStatement> findForward(OnkiResource ores, Statement stmt) {
		return null;
	}
}