package fi.helsinki.cs.seco.onki.logic.propertyfiddlers;

import java.util.List;
import java.util.LinkedList;

import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.vocabulary.RDF;

import fi.helsinki.cs.seco.onki.logic.IPropertyFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.logic.OnkiStatement;

/**
 * Generate relations to from nonpreferred terms to preferred terms in IPSV.
 */

public class IPSVRemoveDuplicateAltLabelsPropertyFiddler implements IPropertyFiddler {
	
	private static final String NS_IPSV = "http://www.esd.org.uk/standards/ipsv/2.00/ipsv-schema#";
	private static final Property IPSV_useItem = ResourceFactory.createProperty(NS_IPSV+"useItem");
	
	private static final String NS_ONKI_CLASSES = "http://demo.seco.tkk.fi/onki/ns/classes#";
	private static final Property ONKI_hiddenType = ResourceFactory.createProperty(NS_ONKI_CLASSES+"hidden");

	// not needed currently
	public List<OnkiStatement> findBackward(OnkiResource ores) {
		return null;
	}

	/** 
	 * Find forward relations to preferred terms.
	 */
	
	public List<OnkiStatement> findForward(OnkiResource ores, Statement stmt) {
		if (!ores.getResource().isResource())
			return null;
		if (!stmt.getPredicate().equals(IPSV_useItem))
			return null;
		
		List<OnkiStatement> reslist = new LinkedList<OnkiStatement>();
		reslist.add(new OnkiStatement(ores, stmt.getPredicate(), stmt.getObject(), ores.getLangPreferences()));
		reslist.add(new OnkiStatement(ores, RDF.type, ONKI_hiddenType, ores.getLangPreferences(), OnkiStatement.TYPE_INDIRECT));
		
		return reslist;
	}
}