package fi.helsinki.cs.seco.onki.logic.propertyfiddlers;

import java.util.List;
import java.util.LinkedList;

import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResourceFactory;

import fi.helsinki.cs.seco.onki.logic.IPropertyFiddler;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.logic.OnkiStatement;

/**
 * Generate relations from members of a collection to the collection in SKOS vocabularies.
 */

public class SkosMemberPropertyFiddler implements IPropertyFiddler {

	private static final String NS_SKOS = "http://www.w3.org/2004/02/skos/core#";
	private static final Property SKOS_member = ResourceFactory.createProperty(NS_SKOS+"member");
	private static final String NS_ONKI = "http://demo.seco.tkk.fi/onki/ns/property#";
	private static final Property ONKI_memberOf = ResourceFactory.createProperty(NS_ONKI+"memberOf");
	
	/** 
	 * Find backward relation to SKOS Collection.
	 */
	
	public List<OnkiStatement> findBackward(OnkiResource ores) {
		if (!ores.getResource().isResource())
			return null;
		
		StmtIterator stmti = ores.getResource().getModel().listStatements(null, SKOS_member, ores.getResource());
		if (!stmti.hasNext())
			return null;

		List<OnkiStatement> reslist = new LinkedList<OnkiStatement>();
		do
			reslist.add(new OnkiStatement(ores, ONKI_memberOf, stmti.nextStatement().getSubject(), ores.getLangPreferences(), OnkiStatement.TYPE_INVERSE));
		while (stmti.hasNext());
		return reslist;
	}

	// not needed currently
	public List<OnkiStatement> findForward(OnkiResource ores, Statement stmt) {
		return null;
	}
}