package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.resource.Resource;  
import org.restlet.util.Series;

import fi.helsinki.cs.seco.onki.logic.OnkiConceptMapping;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.service.IBaseStatement;
import fi.helsinki.cs.seco.onki.service.OnkiDirectory;
import fi.helsinki.cs.seco.onki.service.OnkiHierarchyRelation;
import fi.helsinki.cs.seco.onki.service.OnkiQueryResults;

/**
 * Defines methods that REST resource classes can use from root Restlet.
 */

public abstract class BaseResource extends Resource {
	
	private long startTime;
String requestPath;

	public BaseResource(Context context, Request request, Response response) {  
		super(context, request, response);
		this.startTime = System.currentTimeMillis();
		this.requestPath = request.getOriginalRef().getPath();
	}
	
	protected Collection<IBaseStatement> getLabels(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		return ((RestApplication) getApplication()).getLabels(uri, lang, propertyLang, inGetFullPresentation);
	}
	
	protected Collection<IBaseStatement> getAltLabels(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		return ((RestApplication) getApplication()).getAltLabels(uri, lang, propertyLang, inGetFullPresentation);
	}
	
	protected Collection<LiteralStatement> getDescriptions(String uri, String lang, String propertyLang) {
		return ((RestApplication) getApplication()).getDescriptions(uri, lang, propertyLang);
	}
	
	protected Collection<ResourceStatement> getRelatedConcepts(String uri, String lang, String propertyLang) {
		return ((RestApplication) getApplication()).getRelatedConcepts(uri, lang, propertyLang);
	}
	
	protected Collection<IBaseStatement> getEquivalentConcepts(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		return ((RestApplication) getApplication()).getEquivalentConcepts(uri, lang, propertyLang, inGetFullPresentation);
	}

	protected Collection<ResourceStatement> getGroups(String uri, String lang, String propertyLang) {
		return ((RestApplication) getApplication()).getGroups(uri, lang, propertyLang);
	}

	protected Collection<IBaseStatement> getProperties(String uri, Collection<String> propertyUris, String lang, String propertyLang, boolean filter) {
		return ((RestApplication) getApplication()).getProperties(uri, propertyUris, lang, propertyLang, filter);
	}
	
	protected String getPropertiesAsRdf(String uri) {
		return ((RestApplication) getApplication()).getPropertiesAsRdf(uri);
	}

	protected Collection<ResourceStatement> getTypes(String uri, String lang, String propertyLang) {
		return ((RestApplication) getApplication()).getTypes(uri, lang, propertyLang);
	}
	
	protected Collection<OnkiHierarchyRelation> getConceptHierarchy(String uri, String lang, int parents, int children, int siblings) {
		return ((RestApplication) getApplication()).getConceptHierarchy(uri, lang, parents, children, siblings);
	}
	
	protected Collection<OnkiResource> getHierarchyRoots(String uri, String lang) {
		return ((RestApplication) getApplication()).getHierarchyRoots(uri, lang);
	}
	
	protected String getPolygon(String uri) {
		return ((RestApplication) getApplication()).getPolygon(uri);
	}
	
	protected Map <OnkiResource, Collection<OnkiConceptMapping>> getConceptMappings(String uri, String lang) {
		return ((RestApplication) getApplication()).getConceptMappings(uri, lang);
	}
	
	protected OnkiDirectory getDirectory(String type, Integer memberAmount, String lang) {
		return ((RestApplication) getApplication()).getDirectory(type, memberAmount, lang);
	}
	
	protected Collection<String> getAvailableLanguages() {
		return ((RestApplication) getApplication()).getAvailableLanguages();
	}
	
	protected Collection<Character> getAvailableAlphabets(String lang) {
		return ((RestApplication) getApplication()).getAvailableAlphabets(lang);
	}

	protected Map<String, Integer> getAvailableLabelPrefixes(String lang) {
		return ((RestApplication) getApplication()).getAvailableLabelPrefixes(lang);
	}
	
	protected Collection<String> getAvailableTypeUris() {
		return ((RestApplication) getApplication()).getAvailableTypeUris();
	}

	protected Collection<String> getAvailableConceptNamespaces() {
		return ((RestApplication) getApplication()).getAvailableConceptNamespaces();
	}
	
	protected OnkiQueryResults search(String query, String lang, Integer maxHits, Integer startFromHit, Collection<String> types, Collection<String> parents, Collection<String> groups, Collection<String> excludedTypes, Collection<String> excludedParents, Collection<String> excludedGroups, boolean autocompletion, boolean multipleHitsOfConcept) {
		return ((RestApplication) getApplication()).search(query, lang, maxHits, startFromHit, types, parents, groups, excludedTypes, excludedParents, excludedGroups, autocompletion, multipleHitsOfConcept);
	}
	
	protected HashMap<String, String> getMetadata() {
		return ((RestApplication) getApplication()).getMetadata();
	}
	
	protected Collection<String> getOntologySourceUrls() {
		return ((RestApplication) getApplication()).getOntologySourceUrls();
	}
	
	protected Date getLastModified() {
		return ((RestApplication) getApplication()).getLastModified();
	}
	
	protected Date getExpires() {
		return ((RestApplication) getApplication()).getExpires();
	}
	
	protected int getHierarchySize(int part) {
		return ((RestApplication) getApplication()).getHierarchySize(part);
	}
	
	protected boolean containsResource(String uri) {
		return ((RestApplication) getApplication()).containsResource(uri);
	}
	
	protected JSONObject getFullPresentation(String uri, String lang, String propertyLang) {
		return ((RestApplication) getApplication()).getFullPresentation(uri, lang, propertyLang);
	}
	
	protected void finalize() {
		Series<Parameter> headers = new Form();
		headers.add("X-Onki-Processing-Time", String.valueOf(System.currentTimeMillis() - this.startTime));
headers.add("X-Onki-Request-Path", this.requestPath);
		this.getResponse().getAttributes().put("org.restlet.http.headers", headers);
	}
}