package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;
import java.util.HashSet;

import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;  
import org.restlet.data.Form;
import org.restlet.data.MediaType;  
import org.restlet.data.Parameter;
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;  

import fi.helsinki.cs.seco.onki.service.OnkiQueryResult;

/**
 * Represents the full presentation of all the concepts found in the search index.
 * Wraps together methods used to get relevant information about concepts
 * by the ONKI2 browser, reducing the amount of HTTP requests.
 */
public class GetAllFullPresentationsResource extends BaseResource {
	
	private Collection<String> langs = new HashSet<String>();
	private Collection<String> propertyLangs = new HashSet<String>();
	
	private JSONObject obj = new JSONObject();
	
	public GetAllFullPresentationsResource(Context context, Request request, Response response) {  
		super(context, request, response);
		
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("l"))
				this.langs.add(param.getValue());
			else if (param.getName().equals("pl"))
				this.propertyLangs.add(param.getValue());
		}

		/*int counter = 0;
		long startTime = System.currentTimeMillis();
		long time = 0;
		int limit = 1000;*/
		
		for (OnkiQueryResult resource : this.search("*", null, -1, -1, null, null, null, null, null, null, true, false).getResults()) {
			/*if (time == 0)
				time = System.currentTimeMillis();*/
			JSONObject resourceObj = new JSONObject();
			try {
				int langCounter = 0;
				for (String propertyLang : this.propertyLangs) {
					resourceObj.put(propertyLang, this.getFullPresentation(resource.getUri(), null, propertyLang));
					langCounter++;
				}
				obj.put(resource.getUri(), resourceObj);

				/*counter++;
				System.out.println(counter+" ("+langCounter+"): "+resource.getUri()+" - "+(System.currentTimeMillis()-time)+"ms");
				time = System.currentTimeMillis();
				if (counter >= limit) {
					System.out.println("avg process time / concept: "+((time-startTime)/limit)+"ms");
					break;
				}*/
			}
			catch (JSONException e) {
				e.printStackTrace();
			}
		}
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */
	@Override  
	public Representation represent(Variant variant) throws ResourceException {		
		JsonRepresentation representation = new JsonRepresentation(obj);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}