package fi.helsinki.cs.seco.onki.servlet;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Context;
import org.restlet.data.Form;
import org.restlet.data.MediaType;
import org.restlet.data.Parameter;
import org.restlet.data.Request;
import org.restlet.data.Response;
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;
import org.restlet.resource.ResourceException;
import org.restlet.resource.Variant;

import fi.helsinki.cs.seco.onki.service.OnkiDirectory;
import fi.helsinki.cs.seco.onki.service.OnkiDirectoryElement;
import fi.helsinki.cs.seco.onki.service.OnkiResourceSerializable;

/**
 * Represents  a concept directory.
 */

public class GetDirectoryResource extends BaseResource {
	String type;
	String lang;
	Integer memberAmount;
	OnkiDirectory directory;
	
	public GetDirectoryResource(Context context, Request request, Response response) {  
		super(context, request, response);  
			   
		Form query = getRequest().getResourceRef().getQueryAsForm();
		 
		for (Parameter param : query) {
			if (param.getName().equals("t"))
				this.type = param.getValue();
			else if (param.getName().equals("l"))
				this.lang = param.getValue();
			else if (param.getName().equals("m"))
				this.memberAmount = new Integer(param.getValue());
		}
		 
		this.directory = getDirectory(this.type, this.memberAmount, this.lang);
		   
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {
		JSONObject object = new JSONObject();
		if (this.directory != null) {
			try {
				object.put("type", directory.getType());
				object.put("lang", directory.getLang());
				object.put("size", directory.getSize());
				JSONArray array = new JSONArray();
				for (OnkiDirectoryElement elem : this.directory.getElements()) {
					JSONObject directoryObject = new JSONObject();
					if (elem.getUri() != null)
						directoryObject.put("uri", elem.getUri());
					directoryObject.put("label", elem.getLabel());
					directoryObject.put("size", elem.getSize());
					JSONArray members = new JSONArray();
					for (OnkiResourceSerializable member : elem.getMembers()) {
						JSONObject memberObject = new JSONObject();
						memberObject.put("uri", member.getUri());
						memberObject.put("label", member.getLabel());
						members.put(memberObject);
					}
					directoryObject.put("members", members);
					array.put(directoryObject);
				}
				object.put("elements", array);
			}
			catch (JSONException e) {
				;
			}
		}

		JsonRepresentation representation = new JsonRepresentation(object);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}