package fi.helsinki.cs.seco.onki.servlet;

import java.util.Collection;

import org.json.JSONArray;
import org.restlet.Context;  
import org.restlet.data.MediaType;  
import org.restlet.data.Request;  
import org.restlet.data.Response;  
import org.restlet.ext.json.JsonRepresentation;
import org.restlet.resource.Representation;  
import org.restlet.resource.ResourceException;  
import org.restlet.resource.Variant;

/**
 * Represents source URLs of an ontology.
 */

public class GetOntologySourceUrlsResource extends BaseResource {
	Collection<String> sourceUrls;
	
	public GetOntologySourceUrlsResource(Context context, Request request, Response response) {  
		super(context, request, response);
		this.sourceUrls = getOntologySourceUrls();
		
		getVariants().add(new Variant(MediaType.APPLICATION_JSON));
	}
	
	/** 
	 * Returns a full representation for a given variant. 
	 */  
	@Override  
	public Representation represent(Variant variant) throws ResourceException {  
		JSONArray array = new JSONArray();
		if (this.sourceUrls != null) {
			for (String s : this.sourceUrls) {
				// internal visualization rdf file is not interesting
				if (s.endsWith("onki-services-visualization.rdf"))
					continue;
				// don't let the world see internal file system paths
				if (s.startsWith("file:")) {
					String[] parts = s.split("/");
					if (parts.length > 1)
						s = parts[parts.length-2] + "/" + parts[parts.length-1];
				}
				array.put(s);
			}
		}

		JsonRepresentation representation = new JsonRepresentation(array);
		representation.setModificationDate(getLastModified());
		representation.setExpirationDate(getExpires());
		this.finalize();
	    return representation;
	}
}