package fi.helsinki.cs.seco.onki.servlet;

import java.io.UnsupportedEncodingException;
import java.util.Map;
import java.util.Collection;
import java.net.URLEncoder;

import fi.helsinki.cs.seco.onki.utils.Encoder;

/**
 * Represents URL of the ONKI SKOS browser.
 * For reading the current GET parameters and also for creating links.
 */

public class OnkiURL {

	private OnkiURLGenerator urlgenerator;
	private Map<String,String[]> params;
	
	public OnkiURL(Map<String,String[]> params, OnkiURLGenerator urlgenerator) {
		this.params = params;
		this.urlgenerator = urlgenerator;
	}
	
	public String getBaseURLString() {
		return "?";
	}
	
    /**
	 * Getters for specific GET parameters.
	 */
	
	public String getConcept() {
		return this.getParam(OnkiURLGenerator.PARAM_CONCEPT); 
	}
	
	public String getLang() {
		return this.getParam(OnkiURLGenerator.PARAM_LANG);
	}

	public String getPage() {
		return this.getParam(OnkiURLGenerator.PARAM_PAGE);
	}
	
	public String getQuery() {
		return this.getParam(OnkiURLGenerator.PARAM_QUERY);
	}

	public String[] getQueryType() {
		return this.getParams(OnkiURLGenerator.PARAM_QUERY_TYPE);
	}

	public String[] getQueryParent() {
		return this.getParams(OnkiURLGenerator.PARAM_QUERY_PARENT);
	}
	
	public String[] getQueryGroup() {
		return this.getParams(OnkiURLGenerator.PARAM_QUERY_GROUP);
	}
	
	public String getGroupQuery() {
		return this.getParam(OnkiURLGenerator.PARAM_GROUP_QUERY);
	}
	
	public String getReferer() {
		return this.getParam(OnkiURLGenerator.PARAM_REFERER);
	}
	
	public String getFetchConceptMode() {
		return this.getParam(OnkiURLGenerator.PARAM_FETCH_CONCEPT_MODE);
	}
	
	public String[] getParams(String key) {
		return this.params.get(key);
	}
	
	public String getParam(String key) {
		if (this.params.get(key) != null && this.params.get(key).length>0)
			return this.params.get(key)[0];
		return null;
	}

    /**
	 * When writing parameter values to XHTML template, they should be escaped
	 * for preventing XSS code injection.
	 */
	
	public String[] getParamsEscaped(String key) {		
		String[] params = new String[this.getParams(key).length];
		for (int i=0; i<this.getParams(key).length; i++)
			params[i] = Encoder.escapeXml(this.getParams(key)[i]);
		return params;
	}
	
    /**
	 * Get all the GET parameters.
	 */
	
	public Collection<String> getParamKeys() {
		return this.params.keySet();
	}
	
	public Collection<String[]> getParamValues() {
		return this.params.values();
	}
	
	public String toString() {
		return this.getURLstring();
	}
	
	public String getURLstring() {
		String url = this.getBaseURLString();	 
		String query = "";

		for(String key: this.params.keySet()) {
			try {
				String key2 = URLEncoder.encode(key, this.urlgenerator.encoding());
				for (String value : this.params.get(key)) {
					String value2 = URLEncoder.encode(value, this.urlgenerator.encoding());
					if (query.length() > 0)
						query += "&amp;";
					query += key2+"="+value2;
				}
			} catch (UnsupportedEncodingException ex) {
				System.out.println("Error: in urlencoding "+ex);
			}
		}
		return url + query	;
	}
}