package fi.helsinki.cs.seco.onki.servlet;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.restlet.Application;
import org.restlet.Restlet;
import org.restlet.Router;

import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.ResourceFactory;
import com.hp.hpl.jena.vocabulary.RDF;

import fi.helsinki.cs.seco.onki.index.IndexResult;
import fi.helsinki.cs.seco.onki.index.IndexResults;
import fi.helsinki.cs.seco.onki.index.IndexResultsMetadata;
import fi.helsinki.cs.seco.onki.logic.OnkiConceptMapping;
import fi.helsinki.cs.seco.onki.logic.OnkiConfigurator;
import fi.helsinki.cs.seco.onki.logic.OnkiResource;
import fi.helsinki.cs.seco.onki.logic.OnkiStatement;
import fi.helsinki.cs.seco.onki.logic.OntologyProcessor;
import fi.helsinki.cs.seco.onki.service.IBaseStatement;
import fi.helsinki.cs.seco.onki.service.OnkiDirectory;
import fi.helsinki.cs.seco.onki.service.OnkiHierarchyRelation;
import fi.helsinki.cs.seco.onki.service.OnkiQueryResult;
import fi.helsinki.cs.seco.onki.service.OnkiQueryResults;
import fi.helsinki.cs.seco.onki.service.OnkiQueryResultsMetadata;
import fi.helsinki.cs.seco.onki.stats.OnkiModelStatistics;

/**
 * Handles REST API calls.
 */

public class RestApplication extends Application {

	private OntologyProcessor ontologyProcessor = OntologyProcessor.getInstance();
	
	// values for cache headers
	private Date lastModified;
	private Date expires;
	
	private static final int PARENTS = 0;
	private static final int CHILDREN = 1;
	private static final int SIBLINGS = 2;

	/** 
	 * Creates a root Restlet that will receive all incoming calls. 
	 */  
	
	@Override  
	public synchronized Restlet createRoot() {
		Router router = new Router(getContext());  

		//router.attach("/conceptLabel/{conceptUri}", ConceptLabelResource.class);
		//router.attach("/getLabel?{query}", ConceptLabelResource.class);
		//router.attach("/getLabel?u={uri}&l={lang}", ConceptLabelResource.class);
		router.attach("/getLabels", GetLabelsResource.class);
		router.attach("/getAltLabels", GetAltLabelsResource.class);
		router.attach("/getDescriptions", GetDescriptionsResource.class);

		router.attach("/getRelatedConcepts", GetRelatedConceptsResource.class);
		router.attach("/getEquivalentConcepts", GetEquivalentConceptsResource.class);
		router.attach("/getGroups", GetGroupsResource.class);
		
		router.attach("/getProperties", GetPropertiesResource.class);
		
		// redundant?
		router.attach("/getPropertiesAsRdf", GetPropertiesAsRdfResource.class);
		
		router.attach("/getTypes", GetTypesResource.class);
		
		router.attach("/getConceptHierarchy", GetConceptHierarchyResource.class);
		router.attach("/getHierarchyRoots", GetHierarchyRootsResource.class);
		
		router.attach("/getFullPresentation", GetFullPresentationResource.class);
		router.attach("/getAllFullPresentations", GetAllFullPresentationsResource.class);
		
		router.attach("/getMetadata", GetMetadataResource.class);
		router.attach("/getAvailableLanguages", GetAvailableLanguagesResource.class);
		router.attach("/getAvailableTypeUris", GetAvailableTypeUrisResource.class);
		router.attach("/getAvailableConceptNamespaces", GetAvailableConceptNamespacesResource.class);
		router.attach("/getAvailableLabelPrefixes", GetAvailableLabelPrefixesResource.class);
		router.attach("/getOntologySourceUrls", GetOntologySourceUrlsResource.class);
		
		router.attach("/getDirectory", GetDirectoryResource.class);
		
		router.attach("/search", SearchResource.class);
				
		router.attach("/ping", PingResource.class);
		
		this.lastModified = ontologyProcessor.getStartTime();
		this.expires = ontologyProcessor.getCacheExpireTime();

		return router;
	}

    /**
	 * Get the label of a concept, redundant method.
	 */

	public String getLabel(String uri, String lang) {
		if (this.isOntologyProcessorSet()) {
			ArrayList<String> langs = new ArrayList<String>();
			langs.add(lang);
			langs.add(""); // if there is no label for the desired language, get the label with no language code
			if (this.ontologyProcessor.createOnkiResource(uri, langs) != null) {
				return this.ontologyProcessor.createOnkiResource(uri, langs).getLabel();
			}
		}
		return null;
	}

	//TODO: reuse methods (see methods ending "2" which are not yet in use)
	
	public Collection<IBaseStatement> getLabels2(String uri, String lang, String propertyLang) {
		return this.getProperties2(uri, this.ontologyProcessor.getPropsAsList(OnkiResource.LABEL), lang, propertyLang);
	}
	public Collection<IBaseStatement> getAltLabels2(String uri, String lang, String propertyLang) {
		return this.getProperties2(uri, this.ontologyProcessor.getPropsAsList(OnkiResource.ALT_LABEL), lang, propertyLang);
	}
	
	public Collection<IBaseStatement> getDescriptions2(String uri, String lang, String propertyLang) {
		return this.getProperties2(uri, this.ontologyProcessor.getPropsAsList(OnkiResource.DESCRIPTION), lang, propertyLang);
	}
	
	public Collection<IBaseStatement> getRelatedConcepts2(String uri, String lang) {
		return this.getProperties2(uri, this.ontologyProcessor.getPropsAsList(OnkiResource.RELATED_CONCEPT), null, lang);
	}
	
	public Collection<IBaseStatement> getEquivalentConcepts2(String uri, String lang) {
		return this.getProperties2(uri, this.ontologyProcessor.getPropsAsList(OnkiResource.EQUIVALENT_CONCEPT), null, lang);
	}
	
	public Collection<IBaseStatement> getProperties2(String uri, Collection<String> propertyUris, String lang, String propertyLang) {
		if (this.isOntologyProcessorSet()) {
			if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
				LinkedList<IBaseStatement> labels = new LinkedList<IBaseStatement>();
				if (lang != null && lang.equals("*"))
					lang = null;
				// TODO: generalize to all properties
				for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getLabels(lang, propertyLang))
					labels.add(new LiteralStatement(stmt.getObjectAsString(), stmt.getLang(), stmt.getPredicate().getUri(),	stmt.getPredicate().getLabel()));
				return labels;
			}
		}
		return null;
	}
	
	//TODO: combine functionalities of methods
	
    /**
	 * Get labels of a concept.
	 */
	
	public Collection<IBaseStatement> getLabels(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		if (this.isOntologyProcessorSet()) {
			if (!inGetFullPresentation || !isMethodEmptyInGetFullPresentation("getLabels")) {
				if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
					LinkedList<IBaseStatement> labels = new LinkedList<IBaseStatement>();
					if (lang != null && lang.equals("*"))
						lang = null;
					for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getLabels(lang, propertyLang))
						labels.add(new LiteralStatement(stmt.getObjectAsString(), stmt.getLang(), stmt.getPredicate().getUri(),	stmt.getPredicate().getLabel()));
					return labels;
				}
			}
		}
		return null;
	}
	
    /**
	 * Get alternative labels of a concept.
	 */
	
	public Collection<IBaseStatement> getAltLabels(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		if (this.isOntologyProcessorSet()) {
			if (!inGetFullPresentation || !isMethodEmptyInGetFullPresentation("getAltLabels")) {
				if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
					LinkedList<IBaseStatement> altLabels = new LinkedList<IBaseStatement>();
					if (lang != null && lang.equals("*"))
						lang = null;
					for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getAltLabels(lang, propertyLang))
						altLabels.add(new LiteralStatement(stmt.getObjectAsString(), stmt.getLang(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
					return altLabels;
				}
			}
		}
		return null;
	}
	
    /**
	 * Get textual desriptions of a concept.
	 */
	
	public Collection<LiteralStatement> getDescriptions(String uri, String lang, String propertyLang) {
		if (this.isOntologyProcessorSet()) {
			if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
				LinkedList<LiteralStatement> descriptions = new LinkedList<LiteralStatement>();
				if (lang != null && lang.equals("*"))
					lang = null;
				for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getDescriptions(lang, propertyLang))
					descriptions.add(new LiteralStatement(stmt.getObjectAsString(), stmt.getLang(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
				return descriptions;
			}
		}
		return null;
	}
	
    /**
	 * Get concepts related to a concept.
	 */
	
	public Collection<ResourceStatement> getRelatedConcepts(String uri, String lang, String propertyLang) {
		if (this.isOntologyProcessorSet()) {
			if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
				LinkedList<ResourceStatement> relatedConcepts = new LinkedList<ResourceStatement>();
				
				if (propertyLang == null && lang != null)
					propertyLang = lang;
				else if (lang == null && propertyLang != null)
					lang = propertyLang;
				
				for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getRelatedConcepts(propertyLang))
					relatedConcepts.add(new ResourceStatement(stmt.getObjectAsURI(), this.ontologyProcessor.createOnkiResource(stmt.getObjectAsURI(), this.getLangPreferences(lang)).getLabel(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
				return relatedConcepts;
			}
		}
		return null;
	}
	
    /**
	 * Get the equivalent concepts of a concept.
	 */
	
	public Collection<IBaseStatement> getEquivalentConcepts(String uri, String lang, String propertyLang, boolean inGetFullPresentation) {
		// TODO: add support for symmetry and transitivity
		if (this.isOntologyProcessorSet()) {
			if (!inGetFullPresentation || !isMethodEmptyInGetFullPresentation("getEquivalentConcepts")) {
				if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
					LinkedList<IBaseStatement> equivalentConcepts = new LinkedList<IBaseStatement>();
					
					if (propertyLang == null && lang != null)
						propertyLang = lang;
					else if (lang == null && propertyLang != null)
						lang = propertyLang;
					
					for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(null)).getEquivalentConcepts(propertyLang))
						equivalentConcepts.add(new ResourceStatement(stmt.getObjectAsURI(), this.ontologyProcessor.createOnkiResource(stmt.getObjectAsURI(), this.getLangPreferences(lang)).getLabel(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
					return equivalentConcepts;
				}
			}
		}
		return null;
	}
	
    /**
	 * Get the groups a concept belongs to.
	 */
	
	public Collection<ResourceStatement> getGroups(String uri, String lang, String propertyLang) {
		if (this.isOntologyProcessorSet()) {
			if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
				LinkedList<ResourceStatement> groups = new LinkedList<ResourceStatement>();
				
				if (propertyLang == null && lang != null)
					propertyLang = lang;
				else if (lang == null && propertyLang != null)
					lang = propertyLang;
				
				// TODO: support multiple group properties
				OnkiResource prop = this.ontologyProcessor.createOnkiResource(this.ontologyProcessor.getConfig().getString(OnkiConfigurator.ELEM_CLASSES_GROUP_BELONGS_TO_GROUP_PROPERTY), this.getLangPreferences(propertyLang));
				for (OnkiResource res : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(lang)).getGroups())
					groups.add(new ResourceStatement(res.getUri(), res.getLabel(), prop.getUri(), prop.getLabel()));
				return groups;
			}
		}
		return null;
	}
	
    /**
	 * Get the properties of a concept.
	 */
	
	public Collection<IBaseStatement> getProperties(String uri, Collection<String> propertyUris, String lang, String propertyLang, boolean filter) {
		if (this.isOntologyProcessorSet()) {
			if (this.ontologyProcessor.createOnkiResource(uri, null) != null) {
				LinkedList<IBaseStatement> stmts = new LinkedList<IBaseStatement>();
				HashSet<Property> props = null;
				if (propertyUris != null) {
					props = new HashSet<Property>();
					for (String propertyUri : propertyUris)
						props.add(ResourceFactory.createProperty(propertyUri));
				}
				
				if (propertyLang == null && lang != null)
					propertyLang = lang;
				else if (lang == null && propertyLang != null)
					lang = propertyLang;

				for (OnkiStatement stmt : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(propertyLang)).getProperties(true, props, true, filter)) {
					if (stmt.getObject().isURIResource())
						stmts.add(new ResourceStatement(stmt.getObjectAsURI(), this.ontologyProcessor.createOnkiResource(stmt.getObjectAsURI(), this.getLangPreferences(lang)).getLabel(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
					else if (stmt.getObject().isLiteral())
						stmts.add(new LiteralStatement(stmt.getObjectAsString(), stmt.getLang(), stmt.getPredicate().getUri(), stmt.getPredicate().getLabel()));
					// TODO: anonymous nodes?
				}
				return stmts;
			}
		}
		return null;
	}

    /**
	 * Get the properties of a concept as RDF.
	 */
	
	public String getPropertiesAsRdf(String uri) {
		if (this.isOntologyProcessorSet()) {
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			this.ontologyProcessor.getResourceAsRDF(uri, out);
			try {
				return out.toString("UTF-8");
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
		return null;
	}
	
    /**
	 * Get the types of a concept.
	 */
	
	public Collection<ResourceStatement> getTypes(String uri, String lang, String propertyLang) {
		if (this.isOntologyProcessorSet()) {
			LinkedList<ResourceStatement> stmts = new LinkedList<ResourceStatement>();

			if (propertyLang == null && lang != null)
				propertyLang = lang;
			else if (lang == null && propertyLang != null)
				lang = propertyLang;
			
			String propName = this.ontologyProcessor.createOnkiResource(RDF.type.getURI(), this.getLangPreferences(propertyLang)).getLabel();
			for (OnkiResource ores : this.ontologyProcessor.createOnkiResource(uri, this.getLangPreferences(lang)).getTypes())
				stmts.add(new ResourceStatement(ores.getUri(), ores.getLabel(), RDF.type.getURI(), propName));
			return stmts;
		}
		return null;
	}
	
    /**
	 * Get the concept hierarchy of a concept.
	 */
	
	public Collection<OnkiHierarchyRelation> getConceptHierarchy(String uri, String lang, int parents, int children, int siblings) {
		// TODO: implement parents == n, children == n, children == *, "full tree" (uri == null)
		if (this.isOntologyProcessorSet()) {
			List langPreferences = this.getLangPreferences(lang);
			return this.ontologyProcessor.getConceptTree(uri, langPreferences, parents, children, siblings);
		}
		return null;
	}
	
    /**
	 * Get the hierarchy roots of a concept.
	 */
	
	public Collection<OnkiResource> getHierarchyRoots(String uri, String lang) {
		if (this.isOntologyProcessorSet()) {
			return this.ontologyProcessor.getHierarchyRoots(uri, this.getLangPreferences(lang));
		}
		return null;
	}
	
    /**
	 * Get the geographical polygon a concept.
	 */
	
	public String getPolygon(String uri) {
		if (this.isOntologyProcessorSet()) {
			OnkiResource ores = this.ontologyProcessor.createOnkiResource(uri, null);
			if (ores != null)
				return ores.getPolygon();
		}
		return null;
	}
	
    /**
	 * Get the weighted mappings of a concept.
	 */
	
	public Map <OnkiResource, Collection<OnkiConceptMapping>> getConceptMappings(String uri, String lang) {
		if (this.isOntologyProcessorSet()) {
			List langPreferences = this.getLangPreferences(lang);
			OnkiResource ores = this.ontologyProcessor.createOnkiResource(uri, langPreferences);
			if (ores != null)
				return ores.getMappings();
		}
		return null;
	}
	
    /**
	 * Get a concept directory.
	 */
	
	public OnkiDirectory getDirectory(String type, Integer memberAmount, String lang) {
		if (this.isOntologyProcessorSet()) {
			if (memberAmount == null)
				memberAmount = 5;
			return this.ontologyProcessor.getDirectory(type, this.getLangPreferences(lang), memberAmount);
		}
		return null;
	}
	
    /**
	 * Get the languages of labels the concept index contains.
	 */
	
	public Collection<String> getAvailableLanguages() {
		if (this.isOntologyProcessorSet())
			return this.ontologyProcessor.getAvailableLanguages();
		return null;
	}
	
    /**
	 * Get the types of concepts the concept index contains.
	 */
	
	public Collection<String> getAvailableTypeUris() {
		if (this.isOntologyProcessorSet())
			return this.ontologyProcessor.getAvailableTypeUris();
		return null;
	}
	
	/**
	 * Get all the alphabets of the concept labels in the index.
	 */
	
	public Collection<Character> getAvailableAlphabets(String lang) {
		if (this.isOntologyProcessorSet())
			return this.ontologyProcessor.getAvailableAlphabets(lang);
		return null;
	}
	
	/**
	 * Get all the label prefixes of the concepts in the index.
	 */
	
	public Map<String, Integer> getAvailableLabelPrefixes(String lang) {
		if (this.isOntologyProcessorSet())
			return this.ontologyProcessor.getAvailableLabelPrefixes(lang);
		return null;
	}
	
    /**
	 * Get the namespaces of concepts the concept index contains.
	 */
	
	public Collection<String> getAvailableConceptNamespaces() {
		if (this.isOntologyProcessorSet())
			return this.ontologyProcessor.getAvailableConceptNamespaces();
		return null;
	}
	
    /**
	 * Search the concept index.
	 */
	
	public OnkiQueryResults search(String query, String lang, Integer maxHits, Integer startFromHit, Collection<String> types, Collection<String> parents, Collection<String> groups, Collection<String> excludedTypes, Collection<String> excludedParents, Collection<String> excludedGroups, boolean autocompletion, boolean multipleHitsOfConcept) {
		// TODO: same code as in OnkiQuery.java, should be reused
		if (this.isOntologyProcessorSet()) {
			if (query == null || query.length() == 0)
				query = "*";
			
			query = query.trim();
			
			if (maxHits == null)
				maxHits = 50;
			
			if (startFromHit == null)
				//startFromHit = -1;
				startFromHit = 1;

			if ((types == null || types.size() == 0) && ontologyProcessor.getDefaultViewTypes().size() > 0) {
				types = ontologyProcessor.getDefaultViewTypes();
			}
			
			if (types != null && types.size() > 0) {
				HashSet<String> tempTypes = new HashSet<String>();
				for (String type : types)
					if (type != null && type.length()>0)
						tempTypes.add(type);
				types = tempTypes;
			}

			IndexResults results = this.ontologyProcessor.searchConcept(query, lang, types, parents, groups, false, false, maxHits, startFromHit, excludedTypes, excludedParents, excludedGroups, multipleHitsOfConcept);
			IndexResultsMetadata indexMetadata = results.getMetadata();
			OnkiQueryResultsMetadata metadata = new OnkiQueryResultsMetadata(indexMetadata.getTotalHitsAmount(), indexMetadata.getContainingHitsAmount());
			
			List<OnkiQueryResult> responses = new ArrayList<OnkiQueryResult>();

			for (IndexResult indRes : results.getResults()) {
				// autocompletion=off does not work with wildcards (should be implemented in AlphabeticalIndex)
				// TODO: labels should be normalized for alternative comparison
				//       (i.e. now concepts including diacritics are not returned as hits, if the query string does not contain those diacritics)
				// TODO: would string size comparison be enough?
				if (!autocompletion && ((indRes.getAltLabel() != null && !indRes.getAltLabel().toLowerCase().equals(query.toLowerCase())) ||
						(indRes.getPrefLabel() != null && !indRes.getPrefLabel().toLowerCase().equals(query.toLowerCase()))))
					break;
				OnkiQueryResult qresult = new OnkiQueryResult();
				qresult.setUri(indRes.getUri());
				qresult.setNamespacePrefix(indRes.getUriNamespacePrefix());
				qresult.setTitle(indRes.getPrefLabel());
				qresult.setAltLabel(indRes.getAltLabel());
				if (indRes.getUriNamespacePrefix() != null && indRes.getUriNamespacePrefix().length() > 0)
					qresult.setSerkki(indRes.getUriNamespacePrefix()+":"+indRes.getPrefLabel());
				else
					qresult.setSerkki(indRes.getPrefLabel());
				responses.add(qresult);
			}
			// if ac=false (responses does not contain all the hits), update the amount of the hits
			// TODO: the totalHits amount may not be correct, if all the matching hits do not fit into the result set (maxHits < matching hits)
			if (responses.size() != metadata.getContainingHitsAmount())
				metadata = new OnkiQueryResultsMetadata(responses.size(), responses.size());
			return new OnkiQueryResults(metadata, responses);
		}
		System.out.println("No ontology processor set!");
		return null;
	}

    /**
	 * Get metadata about the ontology.
	 */
	
	public HashMap<String, String> getMetadata() {
		HashMap<String, String> metadata = new HashMap<String, String>();
		if (this.isOntologyProcessorSet()) {
			// last modified timestamp of the ontology
			Date date = new Date(this.ontologyProcessor.getFilesLastModified());
			DateFormat iso8601DateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ssZ");
			String dateString = iso8601DateFormat.format(date);
			dateString = dateString.substring (0, 22) + ":" + dateString.substring (22);
			metadata.put("ontologyLastModified", dateString);
			
			// statistics
			OnkiModelStatistics stats = this.ontologyProcessor.getOntologyStatistics();
			// concept amount
			int concepts = stats.getTotalClassCount();
			if (concepts == 0)
				concepts = stats.getTotalIndividualCount();
			metadata.put("conceptAmount", String.valueOf(concepts));
			// property amount
			int properties = stats.getTotalPropertyCount();
			metadata.put("propertyAmount", String.valueOf(properties));
			// triple amount
			int statements = stats.getTotalStatementCount();
			metadata.put("statementAmount", String.valueOf(statements));
			
			// link to information page
			if (this.ontologyProcessor.getConfig().getBoolean(OnkiConfigurator.ELEM_ONTOLOGY_INFO_PAGE_ENABLED))
				metadata.put("infoUrl", this.ontologyProcessor.getConfig().getString(OnkiConfigurator.ELEM_ONTOLOGY_INFO_PAGE));
			return metadata;
		}
		System.out.println("No ontology processor set!");
		return null;
	}
	
    /**
	 * Get the source URLs of the ontology.
	 */
	
	public Collection<String> getOntologySourceUrls() {
		if (this.isOntologyProcessorSet()) {
			return this.ontologyProcessor.getOntologySourceUrls();
		}
		System.out.println("No ontology processor set!");
		return null;
	}
	
    /**
	 * Get information for cache headers.
	 */
	
	public Date getLastModified() {
		return this.lastModified;
	}
	
	public Date getExpires() {
		return this.expires;
	}
	
    /**
	 * Get default values for the concept hierarchy size.
	 */
	
	public int getHierarchySize(int part) {
		switch (part) {
			case (PARENTS):
				if (this.ontologyProcessor.getConfig().containsKey(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_PARENTS))
					return new Integer(this.ontologyProcessor.getConfig().getString(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_PARENTS));
				break;
			case (CHILDREN):
				if (this.ontologyProcessor.getConfig().containsKey(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_CHILDREN))
					return new Integer(this.ontologyProcessor.getConfig().getString(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_CHILDREN));
				break;
			case (SIBLINGS):
				if (this.ontologyProcessor.getConfig().containsKey(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_SIBLINGS))
					return new Integer(this.ontologyProcessor.getConfig().getString(OnkiConfigurator.ELEM_CLASSES_REST_API_HIERARCHY_SIZE_SIBLINGS));
				break;
		}
		return 0;
	}
	
	/**
	 * Does the ontology contain the given resource (as subject, predicate or object in any statement).
	 */
	
	public boolean containsResource(String uri) {
		if (this.isOntologyProcessorSet()) {
			return this.ontologyProcessor.containsResource(uri);
		}
		return false;
	}
	
	/**
	 * Get the full presentation of a concept as a JSON object.
	 */
	
	public JSONObject getFullPresentation(String uri, String lang, String propertyLang) {
		if (uri != null && this.containsResource(uri)) {
			// get the data
			boolean hasGrandChildren;
			Collection<IBaseStatement> labels;
			Collection<IBaseStatement> altLabels;
			Collection<IBaseStatement> equivalentConcepts;
			Collection<IBaseStatement> properties;
			Collection<OnkiHierarchyRelation> parentTree;
			Collection<OnkiHierarchyRelation> childTree;
			Collection<OnkiHierarchyRelation> siblingTree;
			String polygon;
			Map <OnkiResource, Collection<OnkiConceptMapping>> mappings;
			
			if (propertyLang == null && lang != null)
				propertyLang = lang;
			else if (lang == null && propertyLang != null)
				lang = propertyLang;
			
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	System.out.println("------ rest request starts here ------");
	this.ontologyProcessor.printRunTime("", "", false, true);
}
long requestStartTime = System.nanoTime();
long totalHierarchyTime = 0;
			// children & hasGrandChildren
if (this.isOntologyProcessorSet()) {
	this.ontologyProcessor.parentHierarchyCounter = 0;
	this.ontologyProcessor.parentHierarchyTime = 0;
	this.ontologyProcessor.childHierarchyCounter = 0;
	this.ontologyProcessor.childHierarchyTime = 0;
}
			childTree = getConceptHierarchy(uri, lang, 0, 1, 0);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getChildren (1)", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	System.out.println("parentHierarchyCounter: "+this.ontologyProcessor.parentHierarchyCounter);
	System.out.println("parentHierarchyTime: "+this.ontologyProcessor.parentHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.parentHierarchyCounter > 0)
		System.out.println("parentHierarchy average: "+this.ontologyProcessor.parentHierarchyTime / this.ontologyProcessor.parentHierarchyCounter / 1000000.0);
	System.out.println("childHierarchyCounter: "+this.ontologyProcessor.childHierarchyCounter);
	System.out.println("childHierarchyTime: "+this.ontologyProcessor.childHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.childHierarchyCounter > 0)
		System.out.println("childHierarchy average: "+this.ontologyProcessor.childHierarchyTime / this.ontologyProcessor.childHierarchyCounter / 1000000.0);
	totalHierarchyTime += this.ontologyProcessor.parentHierarchyTime + this.ontologyProcessor.childHierarchyTime;
}
if (this.isOntologyProcessorSet()) {
	this.ontologyProcessor.parentHierarchyCounter = 0;
	this.ontologyProcessor.parentHierarchyTime = 0;
	this.ontologyProcessor.childHierarchyCounter = 0;
	this.ontologyProcessor.childHierarchyTime = 0;
}
			hasGrandChildren = childTree.size() != getConceptHierarchy(uri, null, 0, 2, 0).size();
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("hasGrandChildren", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	System.out.println("parentHierarchyCounter: "+this.ontologyProcessor.parentHierarchyCounter);
	System.out.println("parentHierarchyTime: "+this.ontologyProcessor.parentHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.parentHierarchyCounter > 0)
		System.out.println("parentHierarchy average: "+this.ontologyProcessor.parentHierarchyTime / this.ontologyProcessor.parentHierarchyCounter / 1000000.0);
	System.out.println("childHierarchyCounter: "+this.ontologyProcessor.childHierarchyCounter);
	System.out.println("childHierarchyTime: "+this.ontologyProcessor.childHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.childHierarchyCounter > 0)
		System.out.println("childHierarchy average: "+this.ontologyProcessor.childHierarchyTime / this.ontologyProcessor.childHierarchyCounter / 1000000.0);
	totalHierarchyTime += this.ontologyProcessor.parentHierarchyTime + this.ontologyProcessor.childHierarchyTime;
}
			//getLabels in all languages
			labels = getLabels(uri, null, propertyLang, true);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getLabels", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
}
			//getAltLabels in all languages
			altLabels = getAltLabels(uri, null, propertyLang, true);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getAltLabels", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
}
			//getEquivalentConcepts
			equivalentConcepts = getEquivalentConcepts(uri, lang, propertyLang, true);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getEquivalentConcepts", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
}
			// properties sorted and filtered
			properties = getProperties(uri, null, lang, propertyLang, true);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getProperties", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
}
			// parents
if (this.isOntologyProcessorSet()) {
	this.ontologyProcessor.parentHierarchyCounter = 0;
	this.ontologyProcessor.parentHierarchyTime = 0;
	this.ontologyProcessor.childHierarchyCounter = 0;
	this.ontologyProcessor.childHierarchyTime = 0;
}
			parentTree = getConceptHierarchy(uri, lang, getHierarchySize(PARENTS), 0, 0);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getParents", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	System.out.println("parentHierarchyCounter: "+this.ontologyProcessor.parentHierarchyCounter);
	System.out.println("parentHierarchyTime: "+this.ontologyProcessor.parentHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.parentHierarchyCounter > 0)
		System.out.println("parentHierarchy average: "+this.ontologyProcessor.parentHierarchyTime / this.ontologyProcessor.parentHierarchyCounter / 1000000.0);
	totalHierarchyTime += this.ontologyProcessor.parentHierarchyTime + this.ontologyProcessor.childHierarchyTime;
}
			// siblings
if (this.isOntologyProcessorSet()) {
	this.ontologyProcessor.parentHierarchyCounter = 0;
	this.ontologyProcessor.parentHierarchyTime = 0;
	this.ontologyProcessor.childHierarchyCounter = 0;
	this.ontologyProcessor.childHierarchyTime = 0;
}
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.logRunTime(true);
	this.ontologyProcessor.printRunTime("siblingTime", "", false, true);
}
			siblingTree = getConceptHierarchy(uri, lang, 0, 0, getHierarchySize(SIBLINGS));
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.logRunTime(false);
	this.ontologyProcessor.printRunTime("getSiblings", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	System.out.println("parentHierarchyCounter: "+this.ontologyProcessor.parentHierarchyCounter);
	System.out.println("parentHierarchyTime: "+this.ontologyProcessor.parentHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.parentHierarchyCounter > 0)
		System.out.println("parentHierarchy average: "+this.ontologyProcessor.parentHierarchyTime / this.ontologyProcessor.parentHierarchyCounter / 1000000.0);
	System.out.println("childHierarchyCounter: "+this.ontologyProcessor.childHierarchyCounter);
	System.out.println("childHierarchyTime: "+this.ontologyProcessor.childHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.childHierarchyCounter > 0)
		System.out.println("childHierarchy average: "+this.ontologyProcessor.childHierarchyTime / this.ontologyProcessor.childHierarchyCounter / 1000000.0);
	totalHierarchyTime += this.ontologyProcessor.parentHierarchyTime + this.ontologyProcessor.childHierarchyTime;
}
			if (getHierarchySize(CHILDREN) != 1) {
if (this.isOntologyProcessorSet()) {
	this.ontologyProcessor.parentHierarchyCounter = 0;
	this.ontologyProcessor.parentHierarchyTime = 0;
	this.ontologyProcessor.childHierarchyCounter = 0;
	this.ontologyProcessor.childHierarchyTime = 0;
}
				childTree = getConceptHierarchy(uri, lang, 0, getHierarchySize(CHILDREN), 0);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getChildren", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	System.out.println("parentHierarchyCounter: "+this.ontologyProcessor.parentHierarchyCounter);
	System.out.println("parentHierarchyTime: "+this.ontologyProcessor.parentHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.parentHierarchyCounter > 0)
		System.out.println("parentHierarchy average: "+this.ontologyProcessor.parentHierarchyTime / this.ontologyProcessor.parentHierarchyCounter / 1000000.0);
	System.out.println("childHierarchyCounter: "+this.ontologyProcessor.childHierarchyCounter);
	System.out.println("childHierarchyTime: "+this.ontologyProcessor.childHierarchyTime / 1000000.0);
	if (this.ontologyProcessor.childHierarchyCounter > 0)
		System.out.println("childHierarchy average: "+this.ontologyProcessor.childHierarchyTime / this.ontologyProcessor.childHierarchyCounter / 1000000.0);
	totalHierarchyTime += this.ontologyProcessor.parentHierarchyTime + this.ontologyProcessor.childHierarchyTime;
}
			}
			polygon = getPolygon(uri);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getPolygon", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
}
			mappings = getConceptMappings(uri, propertyLang);
if (this.isOntologyProcessorSet() && this.ontologyProcessor.debugLogEnabled) {
	this.ontologyProcessor.printRunTime("getMappings", "", true, false);
	this.ontologyProcessor.printRunTime("", "", false, true);
	this.ontologyProcessor.printRunTime("total", "", (long)((System.nanoTime()-requestStartTime) / 1000000.0), true, false);
	this.ontologyProcessor.logRunTime(false);
	System.out.println(" "+totalHierarchyTime / 1000000.0+" - total hierarchy time");
	
	System.out.println("cache hits: "+this.ontologyProcessor.cacheHits);
	System.out.println("cache misses: "+this.ontologyProcessor.cacheMisses);
	System.out.println("cache hit percent: "+this.ontologyProcessor.cacheHits / (1.0*this.ontologyProcessor.cacheHits+this.ontologyProcessor.cacheMisses) * 100);
	//this.ontologyProcessor.emptyUriToLabelCache();
	this.ontologyProcessor.printUriToLabelCacheStats();
}
			// construct the JSON object
			JSONObject obj = new JSONObject();
			
			JSONArray propertyArray = new JSONArray();
			if (properties != null) {
				try {
					JSONObject prop = new JSONObject();
					JSONArray objects = new JSONArray();
					for (IBaseStatement stmt : properties) {
						JSONObject object = new JSONObject();
						if (stmt instanceof ResourceStatement) {
							object.put("uri", ((ResourceStatement) stmt).getUri());
							object.put("label", ((ResourceStatement) stmt).getLabel());
						}
						else if (stmt instanceof LiteralStatement) {
							object.put("value", ((LiteralStatement) stmt).getValue());
							object.put("lang", ((LiteralStatement) stmt).getLang());
						}
						if (!prop.has("propertyUri")) { // first statement
							prop.put("propertyUri", stmt.getPredicateUri());
							prop.put("propertyLabel", stmt.getPredicateLabel());
						}
						if (prop.getString("propertyUri").equals(stmt.getPredicateUri())) { // same predicate as in previous statement of first statement
							objects.put(object);
						}
						else { // some other predicate
							prop.put("objects", objects);
							propertyArray.put(prop);
							prop = new JSONObject();
							prop.put("propertyUri", stmt.getPredicateUri());
							prop.put("propertyLabel", stmt.getPredicateLabel());
							objects = new JSONArray();
							objects.put(object);
						}
					}
					// last statement
					if (objects.length() > 0) {
						prop.put("objects", objects);
						propertyArray.put(prop);
					}
					
					obj.put("hasGrandChildren", hasGrandChildren);
					obj.put("getLabels", this.statementsIntoArray(labels));
					obj.put("getAltLabels", this.statementsIntoArray(altLabels));
					obj.put("getEquivalents", this.statementsIntoArray(equivalentConcepts));
					obj.put("getProperties", propertyArray);
					obj.put("getParents", this.treeIntoArray(parentTree));
					obj.put("getChildren", this.treeIntoArray(childTree));
					obj.put("getSiblings", this.treeIntoArray(siblingTree));
					
					if (polygon == null)
						obj.put("getPolygon", JSONObject.NULL);
					else
						obj.put("getPolygon", polygon);

					JSONArray mappingsArr = new JSONArray();
					if (mappings != null) {
						for (Entry<OnkiResource, Collection<OnkiConceptMapping>> mapping : mappings.entrySet()) {
							JSONObject mappingObj = new JSONObject();
							mappingObj.put("propertyUri", mapping.getKey().getUri());
							mappingObj.put("propertyLabel", mapping.getKey().getLabel());
							JSONArray values = new JSONArray();
							for (OnkiConceptMapping value : mapping.getValue()) {
								JSONObject valueObj = new JSONObject();
								valueObj.put("targetUri", value.getTarget().getUri());
								valueObj.put("targetLabel", value.getTarget().getLabel());
								valueObj.put("overlaps", value.getOverlapsRounded());
								valueObj.put("overlappedBy", value.getOverlappedByRounded());
								values.put(valueObj);
							}
							mappingObj.put("targets", values);
							mappingsArr.put(mappingObj);
						}
					}
					obj.put("getMappings", mappingsArr);
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
			}
			return obj;
		}
		return null;
	}
	
	// Private helper methods
	
	/**
	 * Helper to make a JSON array from collection of statements.
	 */
	
	private JSONArray statementsIntoArray(Collection<IBaseStatement> stmts) {
		JSONArray array = new JSONArray();
		if (stmts != null) {
			for (IBaseStatement stmt : stmts) {
				JSONObject object = new JSONObject();
				try {
					if (stmt instanceof LiteralStatement) {
						object.put("label", ((LiteralStatement)stmt).getValue());
						object.put("lang", ((LiteralStatement)stmt).getLang());
					}
					else if (stmt instanceof ResourceStatement) {
						object.put("label", ((ResourceStatement)stmt).getLabel());
						object.put("uri", ((ResourceStatement)stmt).getUri());
					}
					object.put("propertyUri", stmt.getPredicateUri());
					object.put("propertyLabel", stmt.getPredicateLabel());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}
		return array;
	}
	
	/**
	 * Helper to make a JSON array from concept hierarchy tree.
	 */
	
	private JSONArray treeIntoArray(Collection<OnkiHierarchyRelation> tree) {
		JSONArray array = new JSONArray();
		if (tree != null) {
			for (OnkiHierarchyRelation relation : tree) {
				JSONObject object = new JSONObject();
				try {
					object.put("conceptUri", relation.getConceptUri());
					object.put("label", relation.getLabel());
					object.put("parentUri", relation.getParentUri());
					object.put("indent", relation.getIndent());
				}
				catch (JSONException e) {
					e.printStackTrace();
				}
				array.put(object);
			}
		}
		return array;
	}
	
    /**
	 * Is a method configured to return empty response if "get full presentation" method.
	 */
	
	private boolean isMethodEmptyInGetFullPresentation(String methodName) {
		List<String> emptyMethods = Arrays.asList(this.ontologyProcessor.getConfig().getStringArray(OnkiConfigurator.ELEM_CLASSES_REST_API_EMPTY_METHOD));
		return emptyMethods.contains(methodName);
	}
	
    /**
	 * Get the preferred languages.
	 */
	
	private List getLangPreferences(String lang) {
		return this.ontologyProcessor.getLangPreferences(lang);
	}

    /**
	 * Is the OntologyProcessor set.
	 */
	
 	private boolean isOntologyProcessorSet() {
		if (this.ontologyProcessor == null)
			this.ontologyProcessor = OntologyProcessor.getInstance();
		if (this.ontologyProcessor != null)
			return true;
		return false;
	}
}