dojo.provide("semweb.graph.CanvasDrawer");

dojo.require("prototype.prototype");
dojo.require("MochiKit.MochiKit");
dojo.require("semweb.canvas.drawString");
dojo.require("semweb.graph.Graph");

semweb.graph.CanvasDrawer = Class.create();

semweb.graph.CanvasDrawer.prototype = {
  
  setOptions: function(options) {
    this.options = {
      radius: 20,
      arrowHeadLength: null,
      arrowAngle: Math.PI/10,
      mecd: 10, // minimum edge-click distance
      font: tahoma8,
      edgeColor: 'blue'
    }
    
    Object.extend(this.options, options || {});
  },

  initialize: function(element, graph, options) {
    this.element = element;
    this.graph = graph;
	this.setOptions(options);
    this.ctx = element.getContext("2d");
  },
  
  rotate: function(point, length, angle) {
    var dx = length * Math.cos(angle);
    var dy = length * Math.sin(angle);
    return [point[0]+dx, point[1]+dy];
  },

  draw: function() {
    this.ctx.clearRect(0, 0, this.element.width, this.element.height);
    var nodes = this.graph.nodes();
    for (var i = 0; i < nodes.length; i++) {
      this.drawNode(nodes[i]);
    }
  },
  
  animate: function(iterator, count, timeout) {
    iterator.layout(10);
    this.draw();
    if (count>0) semweb.callLater(timeout,this.animate.bind(this),iterator,count-1,timeout);
  },
  
  drawNode: function(node) {
    node.radius = this.options.radius;
    // Draw the text (or if the value is an element, move it)
    if(typeof node.value == 'string') {
      // hack to make the text appear more centered.
	  this.ctx.drawString(node.value, this.options.font, node.x-node.width/2, node.y-node.height/2);
    } else {
      node.value.style.position = 'absolute';
      var tmp = node.value.getDimensions();
      if (tmp.width>tmp.height) node.radius = tmp.width;
      else node.radius=tmp.height;
      var pos = MochiKit.DOM.elementPosition(this.element);
      node.value.style.left     = pos.x + node.x - tmp.width/2 + 'px';
      node.value.style.top      = pos.y + node.y - tmp.height/2 + 'px';    
    }
    this.ctx.strokeStyle = 'black'
    this.ctx.beginPath();
    this.ctx.arc(node.x, node.y, node.radius, 0, Math.PI*2, true);
    this.ctx.closePath();
    this.ctx.stroke();
    var nodes = node.getEdgesOut();
    for (var i = 0; i < nodes.length; i++) {
      this.drawEdge(node,nodes[i]);
    }
  },
  
  drawEdge: function(node1,node2) {
    // find the angle of the edge
    var tan = (node2.y - node1.y) / (node2.x - node1.x);
    var theta = Math.atan(tan);
    if(node1.x <= node2.x) {theta += Math.PI}
    var source = this.rotate([node1.x, node1.y], -node1.radius, theta);
    var target = this.rotate([node2.x, node2.y], node2.radius, theta);

    // draw the edge
    var color = this.options.edgeColor;
    this.ctx.strokeStyle = color;
    this.ctx.fillStyle = color;
    this.ctx.lineWidth = 1.0;
    this.ctx.beginPath();
    this.ctx.moveTo(source[0], source[1]);
    this.ctx.lineTo(target[0], target[1]);
    this.ctx.stroke();
    
    this.drawArrowHead(theta, source[0], source[1], target[0], target[1]);
  },

  drawArrowHead: function(theta, startx, starty, endx, endy) {
	var length = this.options.arrowHeadLength ? this.options.arrowHeadLength : this.options.radius;
    var top = this.rotate([endx, endy], length, theta + this.options.arrowAngle);
    var bottom = this.rotate([endx, endy], length, theta - this.options.arrowAngle);
    this.ctx.beginPath();
    this.ctx.moveTo(endx, endy);
    this.ctx.lineTo(top[0], top[1]);
    this.ctx.lineTo(bottom[0], bottom[1]);
    this.ctx.fill();
  },

  elementAt: function(point) {
	var node = undefined;
	var edge = undefined;
	var minnd = Infinity;
	var mined = Infinity;
	var nodes = this.graph.nodes();
	for (var i = 0; i < nodes.length; i++) {
      var n = nodes[i];
      var dx = point[0] - n.x;
      var dy = point[1] - n.y;
      var d = Math.sqrt(dx * dx + dy * dy);
      if(d < minnd && d <= n.radius) {
        minnd = d;
        node = n;
      }
      for (var j = 0; j< nodes.length; j++) {
        var n2 = nodes[j];
        var x0 = point[0];
        var y0 = point[1];

        var num = Math.abs((n2.x-n.x)*(n1.y-y0)-(n.x-x0)*(n2.y-n1.y));
        var den = Math.sqrt((n2.x-n.x)*(n2.x-n.x)+(n2.y-n1.y)*(n2.y-n1.y));
        var d = num/den;
        if(d < mined && d <= this.options.mecd) {
          mined = d;
          edge = [n1, n2];
        }
      }
    }
    return [node, edge];
  }
};
