dojo.provide("semweb.graph.EventHandler");
dojo.require("prototype.prototype");

semweb.graph.EventHandler = Class.create();
semweb.graph.EventHandler.prototype = {
  setOptions: function(options) {
    this.options = {
      nodeClick:      Prototype.emptyFunction,
      initNodeDrag:   Prototype.emptyFunction,
      updateNodeDrag: Prototype.emptyFunction,
      endNodeDrag:    Prototype.emptyFunction,
      initEdgeDrag:   Prototype.emptyFunction,
      updateEdgeDrag: Prototype.emptyFunction,
      endEdgeDrag:    Prototype.emptyFunction,
      moveNodeOnDrag: true
    }

    Object.extend(this.options, options || {});
  },

  initialize: function(renderer, options) {
    this.renderer = renderer;
    this.setOptions(options);
    
    this.dragged=false;

    this.eventMouseDown = this.initDrag.bindAsEventListener(this);
    this.eventMouseMove = this.updateDrag.bindAsEventListener(this);
    this.eventMouseUp   = this.endDrag.bindAsEventListener(this);
    this.eventClick = this.eventClick.bindAsEventListener(this);
    Event.observe(document, "click", this.eventClick)
    Event.observe(document, "mousedown", this.eventMouseDown);
    Event.observe(document, "mousemove", this.eventMouseMove);
    Event.observe(document, "mouseup", this.eventMouseUp);
  },

  offset: function(event) {
    var pointer = [Event.pointerX(event), Event.pointerY(event)];
    var pos     = Position.cumulativeOffset(this.renderer.element);
    return [0,1].map( function(i) { return (pointer[i] - pos[i]) });
  },
  
  eventClick: function(event) {
    if(Event.isLeftClick(event)) {
      var node = this.renderer.elementAt(this.offset(event))[0];
      this.options.nodeClick(node);
    }
  },

  initDrag: function(event) {
    this.dragged=false;
    if(Event.isLeftClick(event)) {
      var tmp = this.renderer.elementAt(this.offset(event));
      this.activeNode = tmp[0];
      if(this.activeNode != null) {
        this.options.initNodeDrag(this.activeNode);	
      } else {
        this.activeEdge = tmp[1];
        if(this.activeEdge != null) {
          this.options.initEdgeDrag(this.activeEdge);	
        }
      }
      Event.stop(event);
    }
  },

  updateDrag: function(event) {
    this.dragged=true;	
    if(this.activeNode) {
//	  if(this.options.moveNodeOnDrag) {
//        this.activeNode.center = this.offset(event);
//      }
      this.options.updateNodeDrag(this.activeNode,this.offset(event));
    } else if(this.activeEdge) {
      this.options.updateEdgeDrag(this.activeEdge);
    }

    // TODO: delegate to a pluggable method

	// TODO: this belongs in the 'reorg' event handler, but we'll keep it
	// here for now, while we're experimenting....

    // TODO: Fix default renderer so that draw() departs from visual coords
    // instead of recomputing them from the virtual layout ones. -Or perhaps
    // better: rid
  },

  endDrag: function(event) {	
    if(this.activeNode) {
      this.options.endNodeDrag(this.activeNode);
      this.activeNode = null;
    } else if(this.activeEdge) {
      this.options.endEdgeDrag(this.activeEdge);
      this.activeEdge = null;
    }
    // TODO: delegate to a pluggable method:
    // edgeAdded(source, target)
    // nodeDropped(source)
    // edgeRemoved(edge) - when dragged outside the canvas
    // edgeBent(edge, point) - when dragged inside the canvas. New spline point (must be removable too)
  }
};

