dojo.provide("semweb.graph.SpringLayout");

dojo.require("prototype.prototype");
dojo.require("semweb.graph.Graph");

semweb.graph.SpringLayout = Class.create();
semweb.graph.SpringLayout.prototype = {
  initialize: function(graph,minx,miny,maxx,maxy) {
    this.maxx=maxx;
    this.maxy=maxy;
    this.minx=minx;
    this.miny=miny;
    this.sscale = maxx-minx;
    if (maxy-miny<this.sscale) this.sscale = maxy-miny;
    this.graph = graph;
    this.iterations = 500;
    this.maxRepulsiveForceDistance = 200;
//    this.maxRepulsiveForceDistance = this.sscale / 10;
    this.k = this.sscale / 5; // edge 
    this.c = 0.01; // damping factor (0.01)
//    this.maxVertexMovement = 0.5;
    this.maxVertexMovement = 5;
  },
  
  randomize: function() {
    var nodes = this.graph.nodes();
    for (var i= 0; i< nodes.length;i++) {
      var node = nodes[i];
      node.x=this.minx+Math.random()*(this.maxx-this.minx);
      node.y=this.miny+Math.random()*(this.maxy-this.miny);
    }
  },
  
  layout: function(count) {
    if (count == undefined) count = this.iterations;
    for (var i = 0; i < count; i++) {
      this.iterate();
    }
  },
  
  iterate: function() {
    var nodes = this.graph.nodes();
    for (var i = 0; i < nodes.length; i++) {
      var node = nodes[i];
      for (var j = i + 1; j < nodes.length; j++) {
        var node2 = nodes[j];
        this.layoutRepulsive(node, node2);
      }
      var nodes2 = node.getEdgesOut();
      for (var j = 0; j< nodes2.length; j++) {
        var node2 = nodes2[j];
        this.layoutAttractive(node,node2);
      }
    }
    
    // Move by the given force
    for (var i = 0; i < nodes.length; i++) {
      var node = nodes[i];
      
      var xmove = this.c * node.xdelta;
      var ymove = this.c * node.ydelta;

      var max = this.maxVertexMovement;
      if(xmove > max) xmove = max;
      if(xmove < -max) xmove = -max;
      if(ymove > max) ymove = max;
      if(ymove < -max) ymove = -max;
      
      node.x += xmove;
      if (node.x>this.maxx) node.x=this.maxx;
      if (node.x<this.minx) node.x=this.minx;
      node.y += ymove;
      if (node.y>this.maxy) node.y=this.maxy;
      if (node.y<this.miny) node.y=this.miny;
      node.xdelta = 0;
      node.ydelta = 0;
    }
  },

  layoutRepulsive: function(node1, node2) {
    var dx = node2.x - node1.x
    var dy = node2.y - node1.y;
    var d2 = dx * dx + dy * dy;
    if(d2 < 1) {
      dx = 10 * Math.random() + 10;
      dy = 10 * Math.random() + 10;
      var d2 = dx * dx + dy * dy;
    }
    var d = Math.sqrt(d2);
    if(d < this.maxRepulsiveForceDistance) {
      var repulsiveForce = this.k * this.k / d;
      node2.xdelta += repulsiveForce * dx / d;
      node2.ydelta += repulsiveForce * dy / d;
      node1.xdelta -= repulsiveForce * dx / d;
      node1.ydelta -= repulsiveForce * dy / d;
    }
  },

  layoutAttractive: function(node1, node2) {
    var dx = node2.x - node1.x;
    var dy = node2.y - node1.y;
    var d2 = dx * dx + dy * dy;
    if(d2 < 1) {
      dx = 10 * Math.random() + 10;
      dy = 10 * Math.random() + 10;
      var d2 = dx * dx + dy * dy;
    }
    var d = Math.sqrt(d2);
    if(d > this.maxRepulsiveForceDistance) {
      d = this.maxRepulsiveForceDistance;
      d2 = d * d;
    }
    var attractiveForce = (d2 - this.k * this.k) / this.k;
//    if(edge.weight == undefined || edge.weight < 1) edge.weight = 1;
//    attractiveForce *= Math.log(edge.weight) * 0.5 + 1;
    node2.xdelta -= attractiveForce * dx / d;
    node2.ydelta -= attractiveForce * dy / d;
    node1.xdelta += attractiveForce * dx / d;
    node1.ydelta += attractiveForce * dy / d;
  }
};
