package fi.seco.semweb.util.sql;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class NonExceptingResultSet implements ResultSet {

	private ResultSet rs;
	private static Log log = LogFactory.getLog(NonExceptingResultSet.class);

	public NonExceptingResultSet(ResultSet rs) {
		this.rs=rs;
	}

	public boolean absolute(int row) {
		try {
			return rs.absolute(row);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void afterLast() {
		try {
			rs.afterLast();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void beforeFirst() {
		try {
			rs.beforeFirst();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void cancelRowUpdates() {
		try {
			rs.cancelRowUpdates();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void clearWarnings() {
		try {
			rs.clearWarnings();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void close() {
		try {
			rs.close();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void deleteRow() {
		try {
			rs.deleteRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public int findColumn(String columnName) {
		try {
			return rs.findColumn(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public boolean first() {
		try {
			return rs.first();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public Array getArray(int i) {
		try {
			return rs.getArray(i);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Array getArray(String colName) {
		try {
			return rs.getArray(colName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public InputStream getAsciiStream(int columnIndex) {
		try {
			return rs.getAsciiStream(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public InputStream getAsciiStream(String columnName) {
		try {
			return rs.getAsciiStream(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	@Deprecated
	public BigDecimal getBigDecimal(int columnIndex, int scale) {
		try {
			return rs.getBigDecimal(columnIndex, scale);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public BigDecimal getBigDecimal(int columnIndex) {
		try {
			return rs.getBigDecimal(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	@Deprecated
	public BigDecimal getBigDecimal(String columnName, int scale) {
		try {
			return rs.getBigDecimal(columnName, scale);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public BigDecimal getBigDecimal(String columnName) {
		try {
			return rs.getBigDecimal(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public InputStream getBinaryStream(int columnIndex) {
		try {
			return rs.getBinaryStream(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public InputStream getBinaryStream(String columnName) {
		try {
			return rs.getBinaryStream(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Blob getBlob(int i) {
		try {
			return rs.getBlob(i);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Blob getBlob(String colName) {
		try {
			return rs.getBlob(colName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public boolean getBoolean(int columnIndex) {
		try {
			return rs.getBoolean(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean getBoolean(String columnName) {
		try {
			return rs.getBoolean(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public byte getByte(int columnIndex) {
		try {
			return rs.getByte(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}

	}

	public byte getByte(String columnName) {
		try {
			return rs.getByte(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public byte[] getBytes(int columnIndex) {
		try {
			return rs.getBytes(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public byte[] getBytes(String columnName) {
		try {
			return rs.getBytes(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Reader getCharacterStream(int columnIndex) {
		try {
			return rs.getCharacterStream(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Reader getCharacterStream(String columnName) {
		try {
			return rs.getCharacterStream(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Clob getClob(int i) {
		try {
			return rs.getClob(i);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Clob getClob(String colName) {
		try {
			return rs.getClob(colName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getConcurrency() {
		try {
			return rs.getConcurrency();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public String getCursorName() {
		try {
			return rs.getCursorName();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Date getDate(int columnIndex, Calendar cal) {
		try {
			return rs.getDate(columnIndex, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Date getDate(int columnIndex) {
		try {
			return rs.getDate(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Date getDate(String columnName, Calendar cal) {
		try {
			return rs.getDate(columnName, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Date getDate(String columnName) {
		try {
			return rs.getDate(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public double getDouble(int columnIndex) {
		try {
			return rs.getDouble(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public double getDouble(String columnName) {
		try {
			return rs.getDouble(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getFetchDirection() {
		try {
			return rs.getFetchDirection();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getFetchSize() {
		try {
			return rs.getFetchSize();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public float getFloat(int columnIndex) {
		try {
			return rs.getFloat(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public float getFloat(String columnName) {
		try {
			return rs.getFloat(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getInt(int columnIndex) {
		try {
			return rs.getInt(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public int getInt(String columnName) {
		try {
			return rs.getInt(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public long getLong(int columnIndex) {
		try {
			return rs.getLong(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public long getLong(String columnName) {
		try {
			return rs.getLong(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public ResultSetMetaData getMetaData() {
		try {
			return rs.getMetaData();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	public Object getObject(int arg0, Map arg1) {
		try {
			return rs.getObject(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Object getObject(int columnIndex) {
		try {
			return rs.getObject(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	public Object getObject(String arg0, Map arg1) {
		try {
			return rs.getObject(arg0, arg1);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Object getObject(String columnName) {
		try {
			return rs.getObject(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Ref getRef(int i) {
		try {
			return rs.getRef(i);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Ref getRef(String colName) {
		try {
			return rs.getRef(colName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getRow() {
		try {
			return rs.getRow();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public short getShort(int columnIndex) {
		try {
			return rs.getShort(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public short getShort(String columnName) {
		try {
			return rs.getShort(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	public Statement getStatement() {
		try {
			return rs.getStatement();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getString(int columnIndex) {
		try {
			return rs.getString(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public String getString(String columnName) {
		try {
			return rs.getString(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Time getTime(int columnIndex, Calendar cal) {
		try {
			return rs.getTime(columnIndex, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Time getTime(int columnIndex) {
		try {
			return rs.getTime(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Time getTime(String columnName, Calendar cal) {
		try {
			return rs.getTime(columnName, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Time getTime(String columnName) {
		try {
			return rs.getTime(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Timestamp getTimestamp(int columnIndex, Calendar cal) {
		try {
			return rs.getTimestamp(columnIndex, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Timestamp getTimestamp(int columnIndex) {
		try {
			return rs.getTimestamp(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Timestamp getTimestamp(String columnName, Calendar cal) {
		try {
			return rs.getTimestamp(columnName, cal);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public Timestamp getTimestamp(String columnName) {
		try {
			return rs.getTimestamp(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public int getType() {
		try {
			return rs.getType();
		} catch (SQLException e) {
			log.error("",e);
			return -1;
		}
	}

	@Deprecated
	public InputStream getUnicodeStream(int columnIndex) {
		try {
			return rs.getUnicodeStream(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	@Deprecated
	public InputStream getUnicodeStream(String columnName) {
		try {
			return rs.getUnicodeStream(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public URL getURL(int columnIndex) {
		try {
			return rs.getURL(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public URL getURL(String columnName) {
		try {
			return rs.getURL(columnName);
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public SQLWarning getWarnings() {
		try {
			return rs.getWarnings();
		} catch (SQLException e) {
			log.error("",e);
			return null;
		}
	}

	public void insertRow() {
		try {
			rs.insertRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean isAfterLast() {
		try {
			return rs.isAfterLast();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean isBeforeFirst() {
		try {
			return rs.isBeforeFirst();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean isFirst() {
		try {
			return rs.isFirst();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean isLast() {
		try {
			return rs.isLast();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean last() {
		try {
			return rs.last();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void moveToCurrentRow() {
		try {
			rs.moveToCurrentRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void moveToInsertRow() {
		try {
			rs.moveToInsertRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean next() {
		try {
			return rs.next();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean previous() {
		try {
			return rs.previous();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void refreshRow() {
		try {
			rs.refreshRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean relative(int rows) {
		try {
			return rs.relative(rows);
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean rowDeleted() {
		try {
			return rs.rowDeleted();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean rowInserted() {
		try {
			return rs.rowInserted();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public boolean rowUpdated() {
		try {
			return rs.rowUpdated();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public void setFetchDirection(int direction) {
		try {
			rs.setFetchDirection(direction);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void setFetchSize(int rows) {
		try {
			rs.setFetchSize(rows);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateArray(int columnIndex, Array x) {
		try {
			rs.updateArray(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateArray(String columnName, Array x) {
		try {
			rs.updateArray(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateAsciiStream(int columnIndex, InputStream x, int length) {
		try {
			rs.updateAsciiStream(columnIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateAsciiStream(String columnName, InputStream x, int length) {
		try {
			rs.updateAsciiStream(columnName, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBigDecimal(int columnIndex, BigDecimal x) {
		try {
			rs.updateBigDecimal(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBigDecimal(String columnName, BigDecimal x) {
		try {
			rs.updateBigDecimal(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBinaryStream(int columnIndex, InputStream x, int length) {
		try {
			rs.updateBinaryStream(columnIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBinaryStream(String columnName, InputStream x, int length) {
		try {
			rs.updateBinaryStream(columnName, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBlob(int columnIndex, Blob x) {
		try {
			rs.updateBlob(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBlob(String columnName, Blob x) {
		try {
			rs.updateBlob(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBoolean(int columnIndex, boolean x) {
		try {
			rs.updateBoolean(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBoolean(String columnName, boolean x) {
		try {
			rs.updateBoolean(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateByte(int columnIndex, byte x) {
		try {
			rs.updateByte(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateByte(String columnName, byte x) {
		try {
			rs.updateByte(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBytes(int columnIndex, byte[] x) {
		try {
			rs.updateBytes(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateBytes(String columnName, byte[] x) {
		try {
			rs.updateBytes(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateCharacterStream(int columnIndex, Reader x, int length) {
		try {
			rs.updateCharacterStream(columnIndex, x, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateCharacterStream(String columnName, Reader reader, int length) {
		try {
			rs.updateCharacterStream(columnName, reader, length);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateClob(int columnIndex, Clob x) {
		try {
			rs.updateClob(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateClob(String columnName, Clob x) {
		try {
			rs.updateClob(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateDate(int columnIndex, Date x) {
		try {
			rs.updateDate(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateDate(String columnName, Date x) {
		try {
			rs.updateDate(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateDouble(int columnIndex, double x) {
		try {
			rs.updateDouble(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateDouble(String columnName, double x) {
		try {
			rs.updateDouble(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateFloat(int columnIndex, float x) {
		try {
			rs.updateFloat(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateFloat(String columnName, float x) {
		try {
			rs.updateFloat(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateInt(int columnIndex, int x) {
		try {
			rs.updateInt(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateInt(String columnName, int x) {
		try {
			rs.updateInt(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateLong(int columnIndex, long x) {
		try {
			rs.updateLong(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateLong(String columnName, long x) {
		try {
			rs.updateLong(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateNull(int columnIndex) {
		try {
			rs.updateNull(columnIndex);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateNull(String columnName) {
		try {
			rs.updateNull(columnName);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateObject(int columnIndex, Object x, int scale) {
		try {
			rs.updateObject(columnIndex, x, scale);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateObject(int columnIndex, Object x) {
		try {
			rs.updateObject(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateObject(String columnName, Object x, int scale) {
		try {
			rs.updateObject(columnName, x, scale);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateObject(String columnName, Object x) {
		try {
			rs.updateObject(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateRef(int columnIndex, Ref x) {
		try {
			rs.updateRef(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateRef(String columnName, Ref x) {
		try {
			rs.updateRef(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateRow() {
		try {
			rs.updateRow();
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateShort(int columnIndex, short x) {
		try {
			rs.updateShort(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateShort(String columnName, short x) {
		try {
			rs.updateShort(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateString(int columnIndex, String x) {
		try {
			rs.updateString(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateString(String columnName, String x) {
		try {
			rs.updateString(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateTime(int columnIndex, Time x) {
		try {
			rs.updateTime(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateTime(String columnName, Time x) {
		try {
			rs.updateTime(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateTimestamp(int columnIndex, Timestamp x) {
		try {
			rs.updateTimestamp(columnIndex, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public void updateTimestamp(String columnName, Timestamp x) {
		try {
			rs.updateTimestamp(columnName, x);
		} catch (SQLException e) {
			log.error("",e);
		}
	}

	public boolean wasNull() {
		try {
			return rs.wasNull();
		} catch (SQLException e) {
			log.error("",e);
			return false;
		}
	}

	public int getHoldability() throws SQLException {
		return rs.getHoldability();
	}

	public Reader getNCharacterStream(int arg0) throws SQLException {
		return rs.getNCharacterStream(arg0);
	}

	public Reader getNCharacterStream(String arg0) throws SQLException {
		return rs.getNCharacterStream(arg0);
	}

	public NClob getNClob(int arg0) throws SQLException {
		return rs.getNClob(arg0);
	}

	public NClob getNClob(String arg0) throws SQLException {
		return rs.getNClob(arg0);
	}

	public String getNString(int arg0) throws SQLException {
		return rs.getNString(arg0);
	}

	public String getNString(String arg0) throws SQLException {
		return rs.getNString(arg0);
	}

	public RowId getRowId(int arg0) throws SQLException {
		return rs.getRowId(arg0);
	}

	public RowId getRowId(String arg0) throws SQLException {
		return rs.getRowId(arg0);
	}

	public SQLXML getSQLXML(int arg0) throws SQLException {
		return rs.getSQLXML(arg0);
	}

	public SQLXML getSQLXML(String arg0) throws SQLException {
		return rs.getSQLXML(arg0);
	}

	public boolean isClosed() throws SQLException {
		return rs.isClosed();
	}

	@SuppressWarnings("unchecked")
	public boolean isWrapperFor(Class arg0) throws SQLException {
		return rs.isWrapperFor(arg0);
	}

	public <T> T unwrap(Class<T> arg0) throws SQLException {
		return rs.unwrap(arg0);
	}

	public void updateAsciiStream(int arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateAsciiStream(arg0, arg1, arg2);
	}

	public void updateAsciiStream(int arg0, InputStream arg1) throws SQLException {
		rs.updateAsciiStream(arg0, arg1);
	}

	public void updateAsciiStream(String arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateAsciiStream(arg0, arg1, arg2);
	}

	public void updateAsciiStream(String arg0, InputStream arg1) throws SQLException {
		rs.updateAsciiStream(arg0, arg1);
	}

	public void updateBinaryStream(int arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateBinaryStream(arg0, arg1, arg2);
	}

	public void updateBinaryStream(int arg0, InputStream arg1) throws SQLException {
		rs.updateBinaryStream(arg0, arg1);
	}

	public void updateBinaryStream(String arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateBinaryStream(arg0, arg1, arg2);
	}

	public void updateBinaryStream(String arg0, InputStream arg1) throws SQLException {
		rs.updateBinaryStream(arg0, arg1);
	}

	public void updateBlob(int arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateBlob(arg0, arg1, arg2);
	}

	public void updateBlob(int arg0, InputStream arg1) throws SQLException {
		rs.updateBlob(arg0, arg1);
	}

	public void updateBlob(String arg0, InputStream arg1, long arg2) throws SQLException {
		rs.updateBlob(arg0, arg1, arg2);
	}

	public void updateBlob(String arg0, InputStream arg1) throws SQLException {
		rs.updateBlob(arg0, arg1);
	}

	public void updateCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateCharacterStream(arg0, arg1, arg2);
	}

	public void updateCharacterStream(int arg0, Reader arg1) throws SQLException {
		rs.updateCharacterStream(arg0, arg1);
	}

	public void updateCharacterStream(String arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateCharacterStream(arg0, arg1, arg2);
	}

	public void updateCharacterStream(String arg0, Reader arg1) throws SQLException {
		rs.updateCharacterStream(arg0, arg1);
	}

	public void updateClob(int arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateClob(arg0, arg1, arg2);
	}

	public void updateClob(int arg0, Reader arg1) throws SQLException {
		rs.updateClob(arg0, arg1);
	}

	public void updateClob(String arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateClob(arg0, arg1, arg2);
	}

	public void updateClob(String arg0, Reader arg1) throws SQLException {
		rs.updateClob(arg0, arg1);
	}

	public void updateNCharacterStream(int arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateNCharacterStream(arg0, arg1, arg2);
	}

	public void updateNCharacterStream(int arg0, Reader arg1) throws SQLException {
		rs.updateNCharacterStream(arg0, arg1);
	}

	public void updateNCharacterStream(String arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateNCharacterStream(arg0, arg1, arg2);
	}

	public void updateNCharacterStream(String arg0, Reader arg1) throws SQLException {
		rs.updateNCharacterStream(arg0, arg1);
	}

	public void updateNClob(int arg0, NClob arg1) throws SQLException {
		rs.updateNClob(arg0, arg1);
	}

	public void updateNClob(int arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateNClob(arg0, arg1, arg2);
	}

	public void updateNClob(int arg0, Reader arg1) throws SQLException {
		rs.updateNClob(arg0, arg1);
	}

	public void updateNClob(String arg0, NClob arg1) throws SQLException {
		rs.updateNClob(arg0, arg1);
	}

	public void updateNClob(String arg0, Reader arg1, long arg2) throws SQLException {
		rs.updateNClob(arg0, arg1, arg2);
	}

	public void updateNClob(String arg0, Reader arg1) throws SQLException {
		rs.updateNClob(arg0, arg1);
	}

	public void updateNString(int arg0, String arg1) throws SQLException {
		rs.updateNString(arg0, arg1);
	}

	public void updateNString(String arg0, String arg1) throws SQLException {
		rs.updateNString(arg0, arg1);
	}

	public void updateRowId(int arg0, RowId arg1) throws SQLException {
		rs.updateRowId(arg0, arg1);
	}

	public void updateRowId(String arg0, RowId arg1) throws SQLException {
		rs.updateRowId(arg0, arg1);
	}

	public void updateSQLXML(int arg0, SQLXML arg1) throws SQLException {
		rs.updateSQLXML(arg0, arg1);
	}

	public void updateSQLXML(String arg0, SQLXML arg1) throws SQLException {
		rs.updateSQLXML(arg0, arg1);
	}

}
