package fi.seco.semweb.model;

import java.io.ObjectStreamException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

import fi.seco.semweb.util.JenaHelper;
import fi.seco.semweb.util.LString;
import fi.seco.semweb.util.iterator.IIterableIterator;
import fi.seco.semweb.util.iterator.IteratorToIIterableIterator;

public class ResourceHoldingObject extends ModelHoldingObject implements IResourceHoldingObject {

	@SuppressWarnings("unused")
	private Log log = LogFactory.getLog(ResourceHoldingObject.class);
	private static final long serialVersionUID = 1L;

	protected Resource resource;

	public ResourceHoldingObject(Resource resource) {
		super(resource.getModel());
		this.resource=resource;
	}

	public Resource getResource() {
		return resource;
	}

	public void setResource(Resource resource) {
		this.resource = resource;
	}

	public Statement getProperty(String p) {
	   return resource.getProperty(p(p));
	}

	public RDFNode getPropertyAsObject(String p) {
	   Statement tmp=getProperty(p);
	   return tmp!=null ? tmp.getObject() : null;
	}

	public RDFNode getPropertyAsObject(Property p) {
		   Statement tmp=resource.getProperty(p);
		   return tmp!=null ? tmp.getObject() : null;
	}

	@SuppressWarnings("unchecked")
	public IIterableIterator<Statement> listProperties(String p) {
		return new IteratorToIIterableIterator<Statement>(resource.listProperties(p(p)));
	}

	@SuppressWarnings("unchecked")
	public IIterableIterator<Statement> listInverseProperties(String p) {
		return new IteratorToIIterableIterator<Statement>(model.listStatements(null,p(p),resource));
	}

	@SuppressWarnings("unchecked")
	public IIterableIterator<RDFNode> getListItems(String p) {
		return getListItems(p(p));
	}

	@SuppressWarnings("unchecked")
	public IIterableIterator<RDFNode> getListItems(Property p) {
		return new IteratorToIIterableIterator<RDFNode>(JenaHelper.getListItems(resource,p));
	}

	@Override
	public String toString() {
		return resource.toString();
	}

	public StmtIterator listProperties(Property arg0) {
		return resource.listProperties(arg0);
	}

	public StmtIterator listProperties() {
		return resource.listProperties();
	}

	public StmtIterator listInverseProperties(Property arg0) {
		return model.listStatements(null,arg0,resource);
	}

	public StmtIterator listInverseProperties() {
		return model.listStatements(null,null,resource);
	}

	public LString getLabels() {
		return new LString(resource);
	}

	private String serializedForm;
	private static int count=0;

	private static final Property anonymousP = JenaHelper.p("http://www.seco.tkk.fi/onto/model/serialized#anonP");

	@Override
	protected Object writeReplace() throws ObjectStreamException {
		super.writeReplace();
		if (resource.isAnon()) {
			Resource tmp = JenaHelper.r("http://www.seco.tkk.fi/onto/model/serialized#resource"+count++);
			tmp.addProperty(anonymousP,resource);
			serializedForm = tmp.toString();
		} else serializedForm = resource.toString();
		return this;
	}

	@Override
	protected Object readResolve() throws ObjectStreamException {
		super.readResolve();
		resource = model.createResource(serializedForm);
		if (resource.hasProperty(anonymousP)) resource = resource.getProperty(anonymousP).getResource();
		return this;
	}

}
