package fi.seco.semweb.model;

import com.hp.hpl.jena.rdf.model.Resource;

public class WeightedResourceObject extends ResourceObject {
	private static final long serialVersionUID = 1L;

	private float weight;

	/**
	 * Create a new WeightedResourceObject
	 * @param resource the resource
	 * @param weight the weight (must be between 0.0f and 1.0f, inclusive)
	 */

	public WeightedResourceObject(Resource resource, float weight) {
		super(resource);
		if (weight < 0.0f || weight > 1.0f) throw new IllegalArgumentException("Weight " + weight + " out of range");
		this.weight = weight;
	}

	/**
	 * Create a new WeightedResourceObject with default weight 1.0
	 * @param resource the resource
	 */

	public WeightedResourceObject(Resource resource) {
		this(resource, 1.0f);
	}

	/**
	 * Get the weight of this resource
	 * @return the weight
	 */

	public float getWeight() {
		return weight;
	}

	/**
	 * Compare this with another object
	 */

	@Override
	public boolean equals(Object other) {
		if (!(other instanceof WeightedResourceObject))
			return false;
		WeightedResourceObject ob = (WeightedResourceObject) other;
		return (weight == ob.weight && getResource().equals(ob.getResource()));
	}

}
