package fi.seco.semweb.service.indexing.tree.model;

import java.util.ArrayList;
import java.util.List;

public class MemoryNode<T> implements IInputNode<T> {

	private String id;
	private T info;
	private List<MemoryNode<T>> subNodes = new ArrayList<MemoryNode<T>>();
	private MemoryNode<T> parentNode;

	public MemoryNode() { }

	public MemoryNode(String id) {
		this.id=id;
	}

	public MemoryNode(String id, T info) {
		this(id);
		this.info=info;
	}

	public void addSubNode(MemoryNode<T> node) {
		subNodes.add(node);
		node.setParentNode(this);
	}

	public List<? extends MemoryNode<T>> getSubNodes() {
		return subNodes;
	}

	public boolean hasSubNodes() {
		return !subNodes.isEmpty();
	}

	public int getSubNodesCount() {
		return subNodes.size();
	}

	public MemoryNode<T> getParentNode() {
		return parentNode;
	}

	private void setParentNode(MemoryNode<T> parentNode) {
		this.parentNode=parentNode;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	@SuppressWarnings("unchecked")
	public T getInfo() {
		return info;
	}

	public void setInfo(T info) {
		this.info=info;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		toString(sb,"");
		return sb.toString();
	}

	public void toString(StringBuilder sb, String indent) {
		sb.append(indent);
		sb.append("* ");
		sb.append(id);
		sb.append('\n');
		for (MemoryNode<T> c : getSubNodes()) c.toString(sb,indent+" ");
	}

}
