/**
 * 
 * Vanha eristimen peruskäynnistysluokka. Nykyään toiminnallisuudet
 * siirretty GeneralExtractoriin. Voidaan joskus tulevaisuudessa
 * heittää roskiin.
 */

package fi.seco.semweb.service.poka;

import java.io.File;
import java.io.PrintWriter;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fi.seco.semweb.service.poka.name.NameExtractor;
import fi.seco.semweb.service.poka.opas.ConceptMatch;
import fi.seco.semweb.service.poka.util.DocToHTML;

public class PokaConcept {
	private static final Log log = LogFactory.getLog(PokaConcept.class);
	
	//adjective, base, up
	private static final String [] DEFAULTNODES= {"AD","BS","UP"};

	//numeral
	private static final String [] DEFAULTREGEXNODES = {"NM"};
//	public PokaDom doc;
	public GeneralExtractor doc;
	private String pokadoc;

	public PokaConcept() {
		doc = new GeneralExtractor();
	}
	public String getDoc() {
		return doc.getDocument();
	}
	
	public GeneralExtractor getPokaModel() {
		return this.doc;
	}

	public void setDoc(String file) {	
//		pokadoc = file;
		log.debug("file on: '"+file+"'");
		this.doc.setDocument(file);		
	}
	
	public void setInputString(String input) {
		//pokadoc = new PokaDoctor(input, false).getResult();
		this.doc.setInputString(input);		
	}
	
/**
 * Ladataan doc uudelleen
 */
	public void refresh() {
		this.doc.setDocument(pokadoc);
	}

/**
 * Seuraavat 2 metodia lisäävät käsitesolmut dokkariin,
 * eri korostuksin addConcepts heivataan pois joskus,
 * addCU säästetään.
 */
	public void addConcepts() {

		System.out.print("Lisätään käsitteet...");
		doc.addYsoConcepts(GeneralExtractor.CONCEPTNODES, true, false);
		
		log.info(this.doc.getYSOUris());
		System.out.println("...käsitteet ok");

	}
	public void addCU() {
		System.out.print("Lisätään käsitteet...");
		doc.addYsoConcepts(GeneralExtractor.CONCEPTNODES, true, true);
		log.info(this.doc.getYSOUris());
		System.out.println("...käsitteet ok");
		
	}
/**
 * metodi regexien lisäämiseen
 *
 */

	public void addRegex(String regex, String uri) {
		doc.addRegex(regex,uri);
	}

/**
 * metodi regexien poistamiseen
 *
 */
	public void removeRegex(String regex) {
		doc.removeRegex(regex);
	}
/**
 * metodi regexien merkitsemiseksi dokkariin
 * (huom! tällä hetkellä lisätään regexit ainoastaan
 * numeraali(NM)-tageihin, voisi laajentaa ainakin 
 * BS-solmuihin (ja myös AD)
 */
	public void addRegexConcepts() {
		System.out.print("Lisätään regex-käsitteet...");

		doc.addRegexConcepts(DEFAULTREGEXNODES, true);
		System.out.println("...regex ok!");	
	}

	public void addVerbs() {
		log.info("Lisätään verbit...");
		doc.addVerbs(false, true);
		log.info("...verbit ok!");
	}
	
	public String getHTML() {
		String result;
		log.info("Luodaan HTML...");
		result =  DocToHTML.convertHTML(doc.getDocument(),"/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/stylesheets/pokadoc.xsl");
		log.info("...HTML ok!");
		return result;
	}
	
	//TODO: korjaa xsl, pukkaa ainoastaan tagattua ulos
	public String getText() {
		String result;
		log.info("Luodaan teksti...");
		result =  DocToHTML.convertHTML(doc.getDocument(),"text.xsl");
		log.info("...ok!");
		return result;
	}
	//TODO: korjaa xsl, pukkaa ainoastaan tagattua ulos
	public String getTextConcepts() {
		String result;
		log.info("Luodaan teksti (käsitteet)...");
		result =  DocToHTML.convertHTML(doc.getDocument(),"textConcepts.xsl");
		log.info("...ok!");
		return result;
	}
	

	public String getConceptNodes() {
		return this.doc.getConceptNodes(null); //null = oletussolmut
	}
	
	public String getPlain() {
		return DocToHTML.convertHTML(doc.getDocument(),"/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/stylesheets/plain.xsl");
	}

	public String getURIs() {
		return this.doc.getYSOUris();
	}
	
	public Set<String> getURISet() {
		return this.doc.getYSOURISet();
	}
	
	
	public void addNames() {
		log.info("Lisätään nimet...");
		this.doc.addNames();
		log.info("...nimien lisäys valmis!");
	}
	

	public boolean writeToFile(String filename, String text) throws Exception {
		log.info("Kirjoitetaan tiedostoon '"+filename+"'...");
		File outputFile = new File(filename);
		PrintWriter pw = new PrintWriter(outputFile, "ISO-8859-1");
		pw.write(text);
		pw.flush();
		log.info("...tiedosto valmis!");
		return true;
	}
	
	public static void main(String [] args) {

		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka");
		hep.setLevel(Level.DEBUG);
		
		String filename;
		PokaConcept my = null;
		try {
//			filename = args[0];
			filename= "/home/alm/aaparser/text/KalevalaRuno1.txt";
			long now = System.currentTimeMillis();
			my = new PokaConcept();
			long after = System.currentTimeMillis();
			System.out.println("Meni " + (after - now));
//			my.setInputString("Pirkko Määttä on kiva tyyppi. Antti Vehviläisen äät?");
			my.setDoc(filename);
		} catch(Exception e) {
			System.out.println(e);
			System.out.println("Usage: PokaConcept <Document to parse>");
			System.exit(0);
		}
		
		my.addNames();

		my.addRegex("[1-2][8-9|0][0-9][0-9]", "http://yso.fi/YSO#vuosi");
		my.addRegex("\\d(\\d)?\\.\\d(\\d)?\\.\\d\\d\\d\\d","http://yso.fi/YSO#päivämäärät");

//		my.addRegexConcepts();

//		my.addLabel("http://yso.fi/YSO#julkisuuden_henkilöt", "tähti");

//		my.removeLabel( "http://www.cs.helsinki.fi/group/seco/ns/2004/04/YSO#p_SCANDE4iv_SCANDE4m_SCANDE4_SCANDE4r_SCANDE4t", "vaikuttaja");
//		my.removeRegex("1[8-9][0-9][0-9]");

//		my.refresh();
		
		my.addVerbs();
		my.addCU();
		my.addConcepts();
	
		
		
		//Olli: laiska lisäys: muutin PokaDomin julkiseksi,
		//muutetaan takaisin jos järkevää; alla esimerkki mäppäyksestä
		// <löydetty label> --> <uri + yläkäsite x n> 

/*		
		ArrayList <ConceptMatch> cm = my.doc.getConcepts();
		
		for(ConceptMatch c: cm) {
			System.out.println("Text : "+c.getText());
			System.out.println("Lemma: "+c.getLemma());
			System.out.println("TreeMap: "+c.getUri());
		}
*/		

/*		
		ArrayList <String> names = my.doc.getNames();
		System.out.println("------\n-nimet:");
		for(String s: names) {
			System.out.println(s);
		}	
*/
		
//		System.out.println(my.getDoc());
		
//--> tulostaa (termejä vastaavat) urit
//		System.out.println("Uris: " + my.getURISet());

/*
		try {
			my.writeToFile("/home/alm/termo/penkki/outputDok.dok", my.getDoc());				
			
		} catch(Exception e) {
			System.out.println("Virhe kirjoitettaessa tiedostoon: " + e);
		}
*/
		try {
			my.writeToFile("/home/alm/termo/penkki/output.html", my.getHTML());							
		} catch(Exception e) {
			System.out.println("Virhe kirjoitettaessa tiedostoon: " + e);
		}


	}

}