/**
 * Yleinen kehyseristin, hieman vastaava kuin OpasExtractor,
 * tarkennettu yleisest eristimest.
 */


package fi.seco.semweb.service.poka.frame;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.hp.hpl.jena.ontology.OntClass;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.ontology.OntProperty;
import com.hp.hpl.jena.ontology.OntResource;
import com.hp.hpl.jena.ontology.UnionClass;
import com.hp.hpl.jena.rdf.model.Container;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Seq;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.util.FileManager;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

import fi.seco.semweb.service.poka.Configuration;
import fi.seco.semweb.service.poka.DocumentExtractor;
import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.name.NameExtractor;
import fi.seco.semweb.service.poka.util.DocToHTML;
import fi.seco.semweb.service.poka.util.PlaceQuery;
import fi.seco.semweb.service.poka.util.FoundResources;
import fi.seco.semweb.service.poka.util.TrieIndex;
import fi.seco.semweb.service.poka.util.VerbQuery;
import fi.seco.semweb.service.poka.util.YsoQuery;

public class FrameExtractor extends DocumentExtractor {
	private static final Log log = LogFactory.getLog(FrameExtractor.class);
	
	private OntModel m; //malli: YSO, verbit, paikat
	
	public static final String FRAMEPATH = "/home/alm/ontologies/ontologies/poka/frame/FrameModelFull.owl"; 
	public static final String COOKFRAMEPATH = "/home/alm/ontologies/ontologies/poka/frame/CookFull.owl";
	
//	public static final String VERBPATH = "/home/alm/ontologies/ontologies/poka/frame/Term-verbs.rdf"; 
	public static final String VERBPATH = "/home/alm/ontologies/ontologies/poka/frame/Term-verbs-full.rdf";
	public static final String VERBPATHBE = "/home/alm/ontologies/ontologies/poka/frame/Term-verbs-full-with-be.rdf";
//objektityypisten verbiominaisuuksien (=frame roolit) nimet:
	public static final String BASE  = FrameOntologyBuilder.DEFAULTNAMESPACE;
	
//-------------------------------------------
	
	public static final String CONCEPTNODE = "C"; //CONCEPT -- ksitesolmu
	public static final String REGEXNODE = "R"; //REGEX -- regexill lydetty ksite
	public static final String NAMENODE = "NA"; //NAME -- tulee myhemmin nimien tunnistukseen
	public static final String VERBNODE = "V";
	public static final String PLACENODE = "PL";
	public static final String ANIMALNODE = "AN";
	public static final String RELATIVEPLACENODE = "RP";
	public static final String RELATIVETIMENODE = "RT";
	public static final String PRONOUNNODE = "PN";

	//regexin kohde TODO: pivmr menee <OT>-solmuksi eik sisll dependenssej!
	//TODO: luvuilla nytt olevan nihke suhde toimintaan FDG:ss!
	//--> Listty regexiin OT! (14-9-06)
	public static final String [] NUMERALNODES = {"NM","OT"}; //=regex
//---------------------------------------------

	//verbiHash: uri --> Verb
	private HashMap<String, Verb> uriToVerb;	
	
	//regex --> Uri
	private HashMap<String, String> regexToC;
	
	//kyselyrajapinnat
	private YsoQuery yso;
	private FrameVerbQuery verbs;
	private PlaceQuery places;

	private NameExtractor ne;
	private TrieIndex termExist;
	private TrieIndex verbExist; //TODO: korvataan uudella framemallilla
	private TrieIndex placeExist;
	
	private TrieIndex faunaExist; //elukat
	private TrieIndex relativePlaceExist; //suhteelliset paikat 
	
	//HUOM! seuraava kyseenalainen: osa termeist ilman ksitevastaavuutta, kokeilu!
	private TrieIndex relativeTimeExist; //suhteelliset ajat
	
	private FoundResources all;
	
	public FrameExtractor() {
		super();
		
		this.regexToC = new HashMap<String, String>();
		
//		m = ModelFactory.createDefaultModel();
        //TODO: mik paras / nopein OntModel? getVerbInfo hidasteli oletuksella.
		m = ModelFactory.createOntologyModel(OntModelSpec.OWL_DL_MEM);
//		m = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM_MICRO_RULE_INF);

		ne = new NameExtractor();
		
		log.info("listn Framemalli"); //TODO: UTF-8 kun Cook tulloo
		this.addOntology(FrameExtractor.FRAMEPATH, "ISO-8859-1");
		log.info("Frame ok");				
		this.places = new PlaceQuery(m);
		this.yso = new YsoQuery(m);
		this.verbs = new FrameVerbQuery(m);
		
		log.info("Listn termitiedostot");
/*		this.termExist = new TrieIndex(TrieIndex.getNeulemmas(),
				 					   "ISO-8859-1",
				 					   TrieIndex.DEFAULTNAMESPACE);*/
		//yso ilman elukoita ja paikkojaAll-animals-bugs-paikka.rdf
		this.termExist = new TrieIndex("/home/alm/ontologies/ontologies/poka/frame/All-animals-bugs-paikka.rdf",
									   "UTF-8", Configuration.DEFAULTFRAMENAMESPACE);
		
		log.debug("Yso ok");

		//HUOM! asuminen hoidettu termitiedostolla!
		//uudet verbit 01-08-06! (olla on mukana)
		this.verbExist = new TrieIndex(FrameExtractor.VERBPATH,
				 					   "ISO-8859-1",
				 					   Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Verbist ok");
		this.placeExist = new TrieIndex(Configuration.getPlaceLemmas(),
										"ISO-8859-1",
										Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Paikat ok");
		
		this.faunaExist = new TrieIndex("/home/alm/ontologies/ontologies/poka/frame/Terms-animal-bug.rdf",
										"ISO-8859-1", Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Elit ok");
		
		this.relativePlaceExist = new TrieIndex("/home/alm/ontologies/ontologies/poka/frame/Terms-place-relativeFIXED.rdf",
										   "ISO-8859-1", Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Suhteelliset paikat ok");
		
		this.relativeTimeExist = new TrieIndex("/home/alm/ontologies/ontologies/poka/frame/Terms-time-relative.rdf",
											"ISO-8859-1", Configuration.DEFAULTFRAMENAMESPACE);
		log.debug("Suhteellinen aika ok");
		
		log.info("termistt ok.");
		
		all = new FoundResources();
		
		//verbitietojen indeksointi: verbien termindeksilt otetaan
		//kaikki urit, urien tiedot haetaan framemallista metodia
		//getVerbInfo kytten
		log.info("Luodaan verbimppys");
		this.uriToVerb = this.createVerbIndex();
		log.info("verbimppys ok");
		
	}
	
/**
 * Mallien lisys tiedostoista, apinoitu TermoQueryst
 * 
 *
 */
	public boolean addOntology(String filename, String encoding) {
		Model mAttach;
		try {
			InputStream in = this.readFile(filename);
			mAttach = ModelFactory.createDefaultModel();
			mAttach.read(new InputStreamReader(in, encoding), "");
			this.m.add(mAttach);				

			return true;
		} catch(IOException e) {
			System.out.println("Error reading file '"+
	  			filename+"'.");
			//System.exit(0);
			return false;
		}
	}
	
	public void setDocument(String documentUrl) {
		log.debug("Asetetaan dokumentti '"+documentUrl+"'");
		super.setDocument(documentUrl);
		log.debug("Asetetaan nimentunnistin dokumentille");
		this.ne.setDocument(doc);
		this.clearPokaInstance();
	}
	/**
	 * Tyhjent dokumentin instanssitiedot
	 * Huom! tyhjennetn automaattisesti kun uusi dokumentti asetetaan
	 */	
		public void clearPokaInstance() {
			this.all.clear();
			this.ne.index.clear();
		}

	/**
	 * Luo ontologian perusteella verbi-indeksin: 
	 * mppys urista Verbi-olioon. Yksittinen 
	 * verbiolio sislt tiedot rooleista, joita kyseiselle
	 * verbill on.
	 * @return
	 */
		public HashMap<String, Verb> createVerbIndex() {
			//luodaan verbej varten String-->Integer hashmap, joten ei
			//tarvi tehd merkkijonovertailuja (=vertailu 'automatisoitu'):
			HashMap<String, Integer> propNameToInt = new HashMap<String,Integer>(Configuration.VERBSIZE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.AGENT_STR, Configuration.AGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.PATIENT_STR, Configuration.PATIENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.PLACE_STR, Configuration.PLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.TIME_STR, Configuration.TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.MATERIAL_STR, Configuration.MATERIAL);
			propNameToInt.put(FrameExtractor.BASE+Configuration.INSTRUMENT_STR, Configuration.INSTRUMENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RELATIVE_PLACE_STR, Configuration.RELATIVEPLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ABSOLUTE_PLACE_STR, Configuration.ABSOLUTEPLACE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ABSOLUTE_TIME_STR, Configuration.ABSOLUTE_TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.HUMAN_AGENT_STR, Configuration.HUMANAGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.ANIMAL_AGENT_STR, Configuration.ANIMALAGENT);
			propNameToInt.put(FrameExtractor.BASE+Configuration.EFFECTOR_STR, Configuration.EFFECTOR);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RESOURCE_STR, Configuration.RESOURCE);
			propNameToInt.put(FrameExtractor.BASE+Configuration.RELATIVE_TIME_STR, Configuration.RELATIVE_TIME);
			propNameToInt.put(FrameExtractor.BASE+Configuration.GOAL_STR, Configuration.GOAL);
			propNameToInt.put(FrameExtractor.BASE+Configuration.DESTINATION_STR, Configuration.DESTINATION);
						
			HashSet <String> verbUris = verbExist.getTermURIs();
			HashMap<String, Verb> uriToVerbs = new HashMap<String, Verb>();
			
			uriToVerbs = this.getVerbInfo(verbUris, propNameToInt, true);
			
			return uriToVerbs;
		}
		
		
		
/**
 * Tiedoston lukija (TermoQueryst)
 */

	private InputStream readFile(String filename)
							throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
        if (in == null) {
    		throw new IllegalArgumentException(
            	"File '" + filename + "' not found");
		}
		return in;
	}
	public TrieIndex getVerbIndex() {
		return this.verbExist;
	}
	
//------------------------------
	
	
	public void addPlaces(boolean upper) {
		log.info("Listn paikat");
 	    this.addConcepts(FrameExtractor.PLACENODE, GeneralExtractor.CONCEPTNODES, false, upper, this.placeExist, this.places, this.all, false); 	    
	}
	
	public void addVerbs() { //TODO: multiword nyt false! (kvi koulunsa)
		log.info("Listn verbit");
		this.addConcepts(FrameExtractor.VERBNODE,GeneralExtractor.VERBNODES, true, false, verbExist,
				         verbs, all, true);
	}
	
	public void addYso() {
		log.info("Listn YSO");
		this.addConcepts(FrameExtractor.CONCEPTNODE,GeneralExtractor.CONCEPTNODES, true, false, termExist,
						yso, all, true);
	}
	public void addNames() {
		log.info("Listn nimet");
		this.doc = ne.addNames();
	}
	
	public void addAnimals() {
		log.info("Listn elukat");
		this.addConcepts(FrameExtractor.ANIMALNODE, GeneralExtractor.CONCEPTNODES, true, false, this.faunaExist, this.yso, this.all, true);
	}
	
	public void addRelativePlace() {
		log.info("Listn suhteelliset paikat");
		this.addConcepts(FrameExtractor.RELATIVEPLACENODE, GeneralExtractor.CONCEPTNODES, true, false, this.relativePlaceExist, this.yso, this.all, true);
	}
	
	public void addRelativeTime() {
		log.info("Listn suhteelliset ajat");
		this.addConcepts(FrameExtractor.RELATIVETIMENODE, GeneralExtractor.CONCEPTNODES, true, false, this.relativeTimeExist, this.yso, this.all, true);
	}
	
	public String getHTML() { 
		String result;
		log.debug("Luodaan HTML...");
		result =  DocToHTML.convertHTML(this.getDocument(),"/home/alm/workspace/Trunk/fi/seco/semweb/service/poka/stylesheets/pokadoc.xsl");
		log.debug("...HTML ok!");
		return result;
	}
	
	/**
	 * Entityfinderin kytss, palauttaa tietyn lauseen ksitteet
	 * 
	 */
			
	public String getConceptsOfSentence(int sentence) {
		String expression = "/A/S[@no='"+sentence+"']/C|"+ //concept
							"/A/S[@no='"+sentence+"']/NA|"+ //name
							"/A/S[@no='"+sentence+"']/V|"+ //verb
							"/A/S[@no='"+sentence+"']/R|"+ //regex
						    "/A/S[@no='"+sentence+"']/PL";

		String result = "";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
		try {
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			for(int i = 0; i < nodes.getLength(); i++){
				Node item = nodes.item(i);
				result += item.getNodeName()+" ";
				result += item.getChildNodes().item(0).getTextContent()+" ";
				result += item.getChildNodes().item(1).getTextContent()+"\n";
			}
		 } catch (XPathExpressionException e) {
		   			System.out.println("PokaDom / Verbs: " + e);
		 }
		 return result;
	}
/**
 * Palauttaa ominaisuudet (domainit), jotka on mritetty
 * frameontologiassa kyseiselle verbille.
 * @param uri verbi-ksitteen uri
 * @return kyseltv verbi-mykky
 */
	public Verb getVerb(String uri) {
		return this.uriToVerb.get(uri);
	}
	
//-----------------------------------------------------
/**
 * Kun verbi on lytynyt, tll metodilla palautetaan
 * FrameFinderille tieto siit mit rooleja kyseinen verbi sislt
 * 
 * --> TT NYKY KYTETN KONSTRUKTORISTA LUOMAAN
 *     MPPYS URI --> VERB
 *     @param subclasses jos true, iteroidaan mahd. aliluokat ja ulotetaan domain aliluokkiin
 */	

	private HashMap<String, Verb> getVerbInfo(HashSet<String> verbUris, HashMap<String, Integer> prop, boolean subclasses) {

		HashMap <String, Verb> result = new HashMap<String, Verb>();

		//TODO: purkka, joka lis hashiin kaikki verbit (=ei tarvi iteroida
		//vaikeasti lpi verbej, joilla ei ominaisuuksia
		//
		for(String s: verbUris) {
			result.put(s, new Verb(s));
		}
		
		Set <String> propertyUris = prop.keySet();

		for(String pUri : propertyUris) {
			OntProperty op = this.m.getOntProperty(pUri);
			String property = op.getURI();
			String resourceURI;
			log.debug("PROPERTY URI: "+property);
			ExtendedIterator ei = op.listDomain();
			while(ei.hasNext()) {
				OntClass or = (OntClass)ei.next();
				if(or.isURIResource()) {
					
					resourceURI = or.getURI();
					log.debug("\tOminaisuudella yksi arvo domainissa.");
					
					result = this.verbToHash(resourceURI, property, result, prop);
					
					if(subclasses) {
						this.checkSubClasses(or, property, result, prop);							
					}
					
															
				} else if(or.isAnon()) {
					log.debug("\tOminaisuudella useita arvoja (anonyymi.)");					

					log.debug("Union? " + or.isUnionClass());

					UnionClass uc = or.asUnionClass();			
					ExtendedIterator exit = uc.listOperands();
					while(exit.hasNext()) {
						OntClass ores = (OntClass)exit.next();
						
						resourceURI = ores.getURI();
						
						result = this.verbToHash(resourceURI, property, result, prop);
					
						if(subclasses) {
							this.checkSubClasses(ores, property, result, prop);							
						}
					
					}

				}
			}
		}
		return result;
	}
/**
 * Verbin lisys parametrina annettuun hashMappiin
 * @param verbClass
 * @param property
 * @param result
 */
	private HashMap<String, Verb> verbToHash(String verbURI, String property, 
			               HashMap <String, Verb> result, HashMap<String, Integer> prop) {
		Verb temp;

		log.debug("verbi: "+verbURI);
		log.debug("ominaisuudella "+property+" saadaan hashista: "+prop.get(property));
		
		if((temp =result.get(verbURI))==null) {
			log.debug("Verbi '"+verbURI+"' ei resultsetiss, listn.");
			temp = new Verb(verbURI);
			temp.setValue(prop.get(property));
			result.put(verbURI, temp);
			log.debug("Verbi mapissa: "+result.containsKey(verbURI));
		} else {
			log.debug("Verbi lytyy, listn vain ominaisuus.");
			temp.setValue(prop.get(property));
		}
		
		return result;	
	}
/**
 * Tarkistaa onko luokalla aliluokkia. Esim jos domainiksi mritelty frame JA
 * framelta lytyy ominaisuus x, liitetn ominaisuus x kaikkiin aliluokkiin
 * eksplisiittisesti
 * @param verbResource
 * @param property
 * @param result
 * @param prop
 * @return
 */	
	private HashMap<String, Verb> checkSubClasses(OntClass verbResource, String property,
					HashMap <String, Verb> result, HashMap<String, Integer> prop) {
		ExtendedIterator sub = verbResource.listSubClasses();
		String resourceURI;
		while(sub.hasNext()) {
			log.debug("Luokalla '"+verbResource.getURI()+"' aliluokkia.");
			OntClass subclass = (OntClass)sub.next();

			//rekursio: Depth-First-Search, tarkistetaan aliluokan aliluokat!									
			result = this.checkSubClasses(subclass, property, result, prop);
			
			resourceURI = subclass.getURI();
			log.debug("Aliluokka: '"+ resourceURI+"'");	
			result = this.verbToHash(resourceURI, property, result, prop);
		}
		return result;
	}
	
	
//-----------------------------------------------------
//REGEX toisteisena tll (=lytyy mys GeneralExtractorista, jota ei kytet tss
// TODO: jos silt tuntuu, ota pois YSO-kytkent parametrisoimalla se
//-----------------------------------------------------

	public void addRegex(String regexp, String uri) {
		this.regexToC.put(regexp, uri);
	}
	public void removeRegex(String regexp) {
		this.regexToC.remove(regexp);
	}
//	TODO: yhdist muihin jos silt tuntuu	
	public void addRegexConcepts(String [] nodeNames, boolean upper) {
		Iterator <String> i = regexToC.keySet().iterator();
		String tempRegex;

		String expression = "";
		for(String s: nodeNames) {
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);
        NodeList nodes;
		String temp, currURI;
		String concepts;
		XPath xpath = XPathFactory.newInstance().newXPath();
		while(i.hasNext()) {
		
			tempRegex = i.next();

			try {
				currURI = this.regexToC.get(tempRegex);
							
				if(upper) {
					concepts = currURI.substring(	
						currURI.indexOf("#"), currURI.length())+": "+
                            yso.getUpper(currURI);			
				} else {
					concepts = currURI+" | "; //purukumia	
						
				}
					 
				nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
        		for(int ii = 0; ii < nodes.getLength(); ii++){
          			Node item = nodes.item(ii);
					temp = item.getFirstChild().getTextContent(); //tekstimuoto
					log.debug("Regex tekstimuoto: "+temp);
					if(temp.matches(tempRegex)) {
						log.debug("REGEX MTS");

						int sentence = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getTextContent());
						int token = Integer.parseInt(item.getAttributes().getNamedItem("no").getTextContent());
						this.all.addURI(currURI, sentence, token, true);
						
						this.appendConcept(FrameExtractor.REGEXNODE, item, concepts, false);
					}
				}
			} catch (XPathExpressionException e) {
	    		System.out.println(e);
		    }
		}

	}


//	-----------------------------------------------------


	
	public static void main (String [] args) 
							throws IOException {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka");
		hep.setLevel(Level.INFO);
		
		FrameExtractor fe = new FrameExtractor();
		fe.setDocument("/home/alm/termo/FRAME/Asumistesti.txt");
		fe.addNames();
		fe.addPlaces(true);
		fe.addVerbs();
//		fe.addYso(); 
//      addYso korvattu seuraavalla kolmikolla
		fe.addAnimals();
		fe.addRelativePlace();
		fe.addYso();
		
		
//		System.out.println(fe.getDocument());
		fe.writeToFile("/home/alm/termo/penkki/output.html",fe.getDocument());
		
	}
	
	
}
