package fi.seco.semweb.service.poka.frame;

/**
 * Uutta verbiontologiaa varten luotu kopio FrameFinderist
 */


import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.util.debug.DomDebugger;

public class FrameFinderCook {
	private static final Log log = LogFactory.getLog(FrameFinderCook.class);
	
	private HashMap <Integer, Sentence> sentences;
	public FrameExtractorCook frameExtractor;
	
	private FrameResolverCook resolver;
	
	public FrameFinderCook() {
		this.frameExtractor = new FrameExtractorCook();
		this.sentences = new HashMap<Integer, Sentence>();
		
		log.debug("Asetetaan FrameResolver");
		this.resolver = new FrameResolverCook(this.frameExtractor);
		log.debug("FrameResolver ok!");
		
	}
	
	public void setDocument(String filename) {
		this.frameExtractor.setDocument(filename);
	}
	public String getDocument() {
		return this.frameExtractor.getDocument();
	}
/**
 * Palauttaa lauseen numeroa vastaavan sentence-numeron
 * kytetn swing-guissa framejen esittmiseen
 * @param sentenceNumber
 * @return
 */
	public Sentence getSentence(int sentenceNumber) {
		return this.sentences.get(sentenceNumber);
	}
	
	public String getDocumentAsText() {
		return this.frameExtractor.getDocumentAsText();
	}
/**
 * Parsii oletusarvoilla dokumentista esiin
 * ihmisten nimet, paikat, verbit ja yso-ksitteet
 *
 */
	
	public void preParse() {
		frameExtractor.addRegex("[1-2][0-9][0-9][0-9]", "http://yso.fi/YSO#vuosi");
		frameExtractor.addRegex("\\d(\\d)?\\.\\d(\\d)?\\.\\d\\d\\d\\d","http://yso.fi/YSO#pivmrt");
		//tokenIndex on pois plt!
		frameExtractor.addRegexConcepts(FrameExtractor.NUMERALNODES, false, false);
		frameExtractor.addNames();
		frameExtractor.addPlaces(false);
		frameExtractor.addVerbs();
//		fe.addYso(); korvataan seuraavilla

		frameExtractor.addYso();
		this.parseDependencies(); //huom: sentencen solmutyyppi sislt lytyneet
		 						  //ksitteet: jos sentencen tietoja kytetn,
		                          //niit pitisi mys yllpit... 
							      //haetaan ja muutetaan vain rakennetta.
	}
	
	public ArrayList<Frame> provaParse() {
		
		ArrayList <Frame> frames = new ArrayList<Frame>();
		
		String expression = "/A/S/V";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			nodes = (NodeList)xpath.evaluate(expression, this.frameExtractor.getXMLDoc(), XPathConstants.NODESET);
 			log.debug("Iteroidaan verbisolmut");
 			for(int i = 0; i < nodes.getLength(); i++){
 				log.debug("Solmu "+i);
 				Node item = nodes.item(i);
           		int sentNumba = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getNodeValue());
           		int tokenNumba = Integer.parseInt(item.getAttributes().getNamedItem("no").getNodeValue());
          		           		
//         		System.out.println("DOMDEBUG:\n"+DomDebugger.printNode(item.getParentNode()));
           		
         		log.info(frameExtractor.getSentence(sentNumba));   		         		         		
           		Sentence sent = this.sentences.get(sentNumba);
         		log.info("Lause "+sentNumba+": "+sent.getText());
         		
         		String desc = sent.getToken(tokenNumba).getDesc();
         		log.debug("DESC: '"+desc+"'");

         		
         		int uriSeparator = desc.indexOf('|');
         		String uri;
         		while(uriSeparator != -1) {
         			int separator = desc.indexOf('|');
         			uri = desc.substring(0, separator-1);
         			log.debug("uri on '"+uri+"'");
         			
         			Verb v = this.frameExtractor.getVerb(uri);
               		if(v==null) {
               			log.error("V NULL");
               			break;
               		}
               		//this.resolver.getFrames(sent, v, tokenNumba);
               		Frame f = this.resolver.getFrame(sent, v, tokenNumba);
               		
               		//jos f hyvksytn... (=not discarded)
               		if(!f.discarded()) { 
               		
               			f.setSentence(sentNumba);
//	               		f.setStart
               		
               			//TODO: muuta myhemmin tokenkohtaiseksi!
               		
               			//alla selvitetn verbitokenin sisarten
               			//kautta lauseen alku- ja loppusijainti
               			NodeList nl = item.getParentNode().getChildNodes();
               			int nodecount = 0;
               			while(nl.item(nodecount).getNodeType()==Node.TEXT_NODE) {
               				nodecount++;
               			}
               			Node coordinate = nl.item(nodecount).getLastChild();
               			int start = Integer.parseInt(coordinate.getFirstChild().getTextContent());
               		
               			coordinate = nl.item(nl.getLength()-1).getLastChild();
               			int end = Integer.parseInt(coordinate.getLastChild().getTextContent());

               			f.setStart(start);
               			f.setEnd(end);
               			//-----
               		
               			frames.add(f);
               		
               			log.debug(f.toString(sent));
         			}
         			desc = desc.substring(separator+2);
         			log.debug("loput: '"+desc+"'");         			
         			
         			uriSeparator = desc.indexOf('|');
         		}
    		
           		
           		
 			}
 			
    	} catch (XPathExpressionException e) {
 				e.printStackTrace();
 				return null;
 	 	}
    	return frames;
	}
	
	public void parseTest() {
		String expression = "/A/S/V";
		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			nodes = (NodeList)xpath.evaluate(expression, this.frameExtractor.getXMLDoc(), XPathConstants.NODESET);

 			for(int i = 0; i < nodes.getLength(); i++){
           		Node item = nodes.item(i);
           		int sentNumba = Integer.parseInt(item.getParentNode().getAttributes().getNamedItem("no").getNodeValue());
           		int tokenNumba = Integer.parseInt(item.getAttributes().getNamedItem("no").getNodeValue());

         		System.out.println(frameExtractor.getSentence(sentNumba));   		         		         		
           		Sentence sent = this.sentences.get(sentNumba);
         		System.out.println("Lause "+sentNumba+": "+sent.getText());
         		
         		String desc = sent.getToken(tokenNumba).getDesc();
         		//infosolmun lopussa '|'
         		//TODO: useiden tsmvien verbien ksittely
         		desc  = desc.substring(0,desc.length()-3);

         		Verb v = this.frameExtractor.getVerb(desc);
           		if(v==null) {
           			log.error("V NULL");
           			break;
           		}
           		if(v.hasAgent()) {
           			HashSet<Token> refs = this.getRefOfType(sent, tokenNumba, "subj:");
              		if(!refs.isEmpty()) {
             			for(Token t:refs) {
             				
             				if(t.getNodeType().matches("NA|PN")) {
//             	         		System.out.println(fe.getSentence(sentNumba));
//             	         		System.out.println("Lause "+sentNumba+": "+sent.getText());
             					System.out.println("Agentti+Toiminta: '" + t.getText()+" "+item.getChildNodes().item(0).getTextContent()+"'");

             	           		HashSet<Integer> allRefs = sent.getRefs(tokenNumba);
             	           		if(!allRefs.isEmpty()) {
             	           			for(Integer ii: allRefs) {
             	           				System.out.println("Ref "+i+":"+sent.getToken(ii).getText());
             	           			}
             	           		}
             	           		
             	           		if(v.hasAbsolutePlace()) {
             	           			HashSet<Token> locRefs = this.getRefOfType(sent, tokenNumba, "loc:");
             	           			if(!refs.isEmpty()) {
             	           				for(Token tt:locRefs) {
             	           					if(tt.getNodeType().matches("PL")) {
             	           						System.out.println("Paikka          : '" + tt.getText()+"'");
             	           					}
             	           				}
             	           			}
             	           		}
             	           		
             				}
               			}
              			
              		}/* else {
              			System.out.println("--> Syntaktista agenttia ei lydy!");
              		}*/
           		}          		
           		
/*           		
           		HashSet<Integer> allRefs = sent.getRefs(tokenNumba);
           		if(!allRefs.isEmpty()) {
           			for(Integer ii: allRefs) {
           				System.out.println("Ref "+i+":"+sent.getToken(ii).getText());
           			}
           		}
 */
  			}
 			
    	} catch (XPathExpressionException e) {
 				e.printStackTrace();
 				return;
 	 	}
	}

/**
 * Luo lauseelle oliorakenteen kehysten pttely varten
 *
 */
	
//	TODO: dynaamisesti vai staattisesti, onko tarvetta
//	tallettaa kaikki lauseet parsinnassa taulukkoon?
	public void parseDependencies() { 
		
		int count = frameExtractor.sentenceCount(),
	    i = 1;
		
		while(i < count) {
			log.debug("---lause "+i+" ---------");
			log.debug(this.frameExtractor.getSentence(i));
			Sentence s = this.addSentence(i);
			this.sentences.put(i, s);
//			koo.getFrames(s);
//			System.out.println(koo.poka.getConceptsOfSentence(i)+"\n");
			i++;
		}
		
	}
	
	
	private Sentence addSentence(int sentence) {
		String expression = "/A/S["+sentence+"]";
       	XPath xpath = XPathFactory.newInstance().newXPath();
    	try {
 			Node n = (Node)xpath.evaluate(expression, this.frameExtractor.getXMLDoc(), XPathConstants.NODE);
 			return new Sentence(n);
    	} catch (XPathExpressionException e) {
			e.printStackTrace();
			return null;
 	 	}
	}
	
	
	/**
	 * Palauttaa parametrina annettavaan tokeniin viittaavat tokenit
	 * joiden dependenssityyppi on 'type' (esim 'obj:', 'subj:',  'main:')   
	 * HashSettin
	 * 
	 * @param s Sentence, ko. lause
	 * @param referent Token, johon viitataan
	 * @param type dependenssityyppi, jota etsitn
	 * @return 
	 */
	
	public HashSet<Token> getRefOfType(Sentence s, int referent, String type) {
		HashSet <Token> result = new HashSet <Token> ();
		if(s.hasRefs(referent)) { 
			HashSet <Integer> referrers = s.getRefs(referent);
			Token t;
			for(int i: referrers) {
				t = s.getToken(i);
				if(t.getDType().equals(type)) {
					result.add(t);
				}
			}
		}
		return result;
	}
	
	public static void main(String [] args) throws IOException {
		Logger hep = Logger.getLogger("fi.seco.semweb.service.poka.name");
		hep.setLevel(Level.OFF);
		Logger de = Logger.getLogger("fi.seco.semweb.service.poka.DocumentExtractor");
		de.setLevel(Level.OFF);
		Logger frame = Logger.getLogger("fi.seco.semweb.service.poka.frame");
		frame.setLevel(Level.INFO);
		Logger util = Logger.getLogger("fi.seco.semweb.service.poka.util");
		frame.setLevel(Level.OFF);
		
		//HUOM!!! EI SAA KYTT: MUUTOKSET SUORAAN ISOON MALLIIN!!!
//		new FrameOntologyBuilder(); //rakennetaan ontologiat uudestaan

//Tiedoston enkoodaus lytyy XFDGConnectista.
//TODO: voisi tuoda tnne asti.
		FrameFinderCook ff = new FrameFinderCook();
		ff.setDocument("/home/alm/termo/FRAME/Asumistesti.txt");
//		ff.setDocument("/home/alm/aaparser/text/LarinParaske.html");
//		ff.setDocument("/home/alm/aaparser/text/AckteAinoHTML.txt");
//		ff.setDocument("/home/alm/aaparser/text/LonkkaEliasHTML.txt");
//		ff.setDocument("/home/alm/aaparser/text/AleksisKivi.html");
//		ff.setDocument("/home/alm/aaparser/text/Gallenkallela.html");
		
		ff.preParse();

//		System.out.println(ff.getDocument());
		ff.provaParse();
//		ff.parseTest();
		
	}

	
	
}
