package fi.seco.semweb.service.poka.frame;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fi.seco.semweb.service.poka.Configuration;

import ws.prova.Communicator;
import ws.prova.kernel.ProvaResultSet;

public class FrameResolver {
	private static final Log log = LogFactory.getLog(FrameResolver.class);
	
	static final String kAgent = "prova";
	static final String kPort = null;
	
//	private final String RULEBASE= "frameres.prova";
	private final String RULEBASE= "frameresolver.prova";
	private Communicator comm;
	
	public FrameResolver() {
		
		comm = null;
		
		
		try {
			comm = new Communicator(kAgent, kPort, 
				new BufferedReader(
						new InputStreamReader(
								FrameResolver.class.getResource(RULEBASE).
										openStream())),
				0, Communicator.SYNC);
		} catch(IOException e) {
			log.error("Virhe prova-alustuksessa.");
			e.printStackTrace();			
			System.exit(0);
		}
		
	}

	
	//uusi 
	public Frame getFrame(Sentence sentence, Verb verb, int tokenNumber) {
		log.info("Parsitaan frame, verbi on: '"+verb.getUri()+"'");
		Frame fr = new Frame(verb.getUri());
		
		//_0 sentence _1 tokenNumba _2 verbi _3 frame
		String input = ":- eval(frame(_0, _1, _2, _3)).";
		this.queryNeuNeu(input, sentence, tokenNumber, verb, fr);
		return fr;
	}
	
/**
 * Prova-pttelijn kaipaamat tiedot. Alustavasti: iteroidaan
 * javan puolella lpi verbi-otus, jokaista framen elementti
 * kohti kynnistetn kyseisen entiteetin etsint prova-puolella.
 * Esim asuminen --> verb.hasAgent? --> yes --> prova.getAgent()
 * @param sentence
 * @param verb
 * @param tokenNumber
 */	
	
	public void getFrames(Sentence sentence, Verb verb, int tokenNumber) {
		log.info("Parsitaan frame, verbi on: '"+verb.getUri()+"'");
		String input = "";
				
		//avain --> totuusarvo -mppys.
		//avaimet mritelty julkisina Verb-luokassa
		Set <Integer> keys = verb.getVerbs().keySet();

		for(Integer i: keys) {
			switch(i) {
			
			case Configuration.AGENT:
				log.info("--> Agent");
				input = ":- solve(agent(_0, _1, X)).";	
				break;
			
			case Configuration.PATIENT:
				log.info("--> Patient");
				input = ":- solve(patient(_0, _1, X)).";
				break;
			
			case Configuration.PLACE:
				log.info("--> Place");
				input = ":- solve(place(_0, _1, X, Y)).";
				break;
				
			case Configuration.TIME:
				log.info("--> Time");
				input = ":- solve(time(_0, _1, X)).";
				break;
				
			case Configuration.MATERIAL:
				log.info("--> Material");
				input = ":- solve(material(_0, _1, X)).";
				break;
				
			case Configuration.INSTRUMENT:
				log.info("--> Instrument");
				input = ":- solve(instrument(_0, _1, X)).";
				break;
/*
   TODO: poistetaan kokonaan			
			case Verb.RELATIVEPLACE:
				log.info("--> Relative place");
	 			input = ":- solve(relative_place(_0, _1, X)).";
	 			break;
	 			
			case Verb.ABSOLUTEPLACE:
				log.info("--> Location");
				input = ":- solve(absolute_place(_0, _1, X)).";
				break;
*/
/* TODO:poistetaan
			case Verb.ABSOLUTE_TIME:
				log.info("--> Absolute time");
				input = ":- solve(absolute_time(_0, _1, X)).";
				break;
*/				
			case Configuration.HUMANAGENT:
				log.info("--> Human agent");
				input = ":- solve(human_agent(_0, _1, X)).";
				break;
			
			case Configuration.ANIMALAGENT:
				log.info("--> Animal agent");
				input = ":- solve(animal_agent(_0, _1, X)).";
				break;
			
			case Configuration.EFFECTOR:
				log.info("--> Effector");
				input = ":- solve(effector(_0, _1, X)).";
				break;
			
			case Configuration.RESOURCE:
				log.info("--> Resource");
				input = ":- solve(resource(_0, _1, X)).";
				break;
				
			case Configuration.GOAL:
				log.info("--> Goal");
				input = ":- solve(goal(_0, _1, X)).";
				break;
				
			case Configuration.DESTINATION:
				log.info("--> Destination");
				input = ":- solve(destination(_0, _1, X)).";
				break;

/* TODO: poistetaan	
			case Verb.RELATIVE_TIME:
				log.info("--> Relative time");
				input = ":- solve(resource(_0, _1, X)).";
				break;
*/			
			}	
			this.query(input, sentence, tokenNumber);
		}
		
		//kaikilla on paikka (24-7-06)
		//ei sittenkn: "min palasin helsinkiin" (25-7-06)
/*
		input = ":- solve(place(_0, _1, X, Y)).";
		this.query(input, sentence, tokenNumber);
*/		
		//kaikilla on aika (24-7-06)???
/*		
		log.info("--> Time");
		input = ":- solve(time(_0, _1, X, Y)).";
		this.query(input, sentence, tokenNumber);
*/
			
	}
	
	private void query(String queryInput, Sentence sentence, int tokenNumber) {
		
		String input = queryInput;
		
		int key = 0;
		
		try {
			List l = comm.consultSync(input,
				         Integer.toString(++key),
				         new Object[]{sentence, tokenNumber});
			for (Iterator rsit = l.iterator(); rsit.hasNext();) {
				ProvaResultSet rs = (ProvaResultSet) rsit.next();
				if (rs.getException() != null) {
					System.out.println("Test fails with " + rs.getException());
					continue;
				} else if (rs.numberOfResults() == 0) {
					// No solutions for this resultset
					continue;
				}

				rs.first();
				while(!rs.isAfterLast()) {
					Object a = rs.getResult(Object.class, "X");
					Object b = rs.getResult(Object.class, "Y");
//					Object c = rs.getResult(Object.class, "Z");
					System.out.println("result= '" + a+"'");
					System.out.println("result= '" + b+"'");
					rs.next();
				}
							
			}
		
		} catch(Exception e) {
			e.printStackTrace();
		}
		
	}
	
	
	private void queryNeuNeu(String queryInput, Sentence sentence, int tokenNumber, Verb verb, Frame frame) {
		
		String input = queryInput;
		
		int key = 0;
		
		try {
			comm.consultSync(input,
			         Integer.toString(++key),
			         new Object[]{sentence, tokenNumber, verb, frame});
				
		} catch(Exception e) {
			e.printStackTrace();
		}
		
	}
	
	
}
