/**
 * Nimien indeksoija, pit kirjaa lydetyist nimist ja
 * pyrkii pttelemn onko uusi, lydetty nimi sama edellisen
 * kanssa
 */

package fi.seco.semweb.service.poka.name;

import java.util.TreeMap;

import java.util.HashMap;
import java.util.HashSet;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Node;

import fi.seco.semweb.service.poka.frame.FrameExtractor;
import fi.seco.semweb.service.poka.util.InstanceFrequency;
import fi.seco.semweb.service.poka.util.Location;
import fi.seco.semweb.service.poka.util.PokaResource;
import fi.seco.semweb.service.poka.util.TernarySearchTrie;
import fi.seco.semweb.service.poka.util.debug.DomDebugger;


//similaritymetrics from simmetrics open source library,
//for more information, see
//http://www.dcs.shef.ac.uk/~sam/stringmetrics.html
import similaritymetrics.JaroWinkler;

/**
 * Data structure for indexing and searching names
 * for NameGrouper
 *
 * @author Olli Alm
 * @version 1.0
 */
public class NameIndex {
	private static final Log log = LogFactory.getLog(NameIndex.class);

/**
 * Threshold-value for Jaro-Winkler-similarity
 * -algorithm. If comparison of two Strings
 * returns greater value than this, then
 * Strings refers to same name...
 */
	public final double JWSIMILAR = 0.89;
	
/**
 * Implementation of JaroWinkler String-similarity
 * algorithm from SimMetrics java library by
 * Sam Chapman (GNU General Public License).
 * For more information, see
 * http://www.nist.gov/dads/HTML/jaroWinkler.html
 */
	private JaroWinkler jw = new JaroWinkler();
	
/**
 * Similaritystructure for first names: a mapping
 * from lemmatized first names to Integer-hashSet.
 * HashSet contains ID's of all names which share 
 * the same first name (=KEY)
 */
	private HashMap <String, HashSet <Integer> > firstSimilar;
/**
 * Similaritystructure for surnames: a mapping
 * from lemmatized surnames to Integer-hashSet.
 * HashSet contains ID's of all names which share 
 * the same surname (=KEY)
 */
	private HashMap <String,HashSet <Integer> > surSimilar;
/**
 * This tree structure maps from name ID's to
 * Name-instances.
 */
	private TreeMap <Integer, Name> names;
/**
 * Nimi ID-laskuri, annetaan uudelle nimelle
 * aina seuraava numero
 */
	private int counter = 0;
		
	private TernarySearchTrie nameTrie;
	
	
/**
 * UUSI KONSTRUKTORI, VANHA POISTETAAN KU PELITT
 * --> nimet listn lennosta
 * 
 */
	public NameIndex() {

		this.names = new TreeMap<Integer, Name>();

		this.nameTrie = new TernarySearchTrie(); // trie: mppys sukunimest ID-arvoon
		
//		this.addNames(nameID);
		
		firstSimilar = new HashMap <String, HashSet<Integer>>();
		surSimilar = new HashMap <String, HashSet<Integer>>();

//		this.groupSimilar(nameID);
	}	
	
/**
 * returns names in ascending ID-order
 *
 * @return names in default-representation
 */	
	public String toString() {
		String result = "";
		Iterator <Name> i = names.values().iterator();
		Name temp;
		while(i.hasNext()) {
			temp = i.next();
			result += temp.toString() + ": "+  temp.getFrequency()+"\n";
		}
		return result;
	}

//nimien palautus, palauttaa oletusnimen (nimiolio voi sislt useita
//saman identiteetin omaavan henkiln nimi (esim. kaksi lhes
//vastaavaa etunime
	public ArrayList<String> getNames() {
		ArrayList <String> result = new ArrayList<String>();
		Iterator <Name> i = names.values().iterator();

		while(i.hasNext()) {
			result.add(i.next().toString());
		}
		return result;
	}
	
//nimi-olioiden palautus
	public ArrayList<Name> getNameObjects() {
		
		ArrayList <Name> result = new ArrayList<Name>();
				
		for(Name n: names.values()) {
			result.add(n);
		}
		
		Collections.sort(result);
		
		return result;
	}
	

/**
 * gets the surname of the current
 * nameID as a String.
 *
 * @param nameid the identification number of the name-entity
 * @return surname
 */
	public String getSur(int nameid) {
		return names.get(nameid).getSur();
	}
/**
 * gets the firstname of the current
 * nameID as a String.
 *
 * @param nameid the identification number of the name-entity
 * @return firstname
 */	
	public String getFirst(int nameid) {
		return names.get(nameid).getFirst();
	}
/**
 * return both the firstnames and the surname
 * 
 */
	public String getName(int id) {
		return names.get(id).toString();
	}
	
/**
 * returns true if there exist several
 * name-entities with same lemmatized first name
 *
 * @param lemma the lemmatized form of the first name
 * @return true if there are more than one entities with same first name
 * @see #getFID(String)
 */
	public boolean severalFirst(String name) {
		if(firstSimilar.get(name).size() > 1) {
			return true;
		}
		return false;
	}

/**
 * returns true if first name's
 * similarityset contains current,
 * lemmatized form of the first name.
 *
 * @param firstname the lemmatized form of the first name
 * @return true if first name exist
 * @see #containS(String)
 */
	public boolean containF(String firstname) {
		return firstSimilar.containsKey(firstname);
	}
/**
 * returns name ID if trie contains current name, otherwise false
 *
 * @param surname the lemmatized form of the surname
 * @return name ID if surname exist
 * @see #containF(String)
 */
	public int nameInTrie(String name) {
		Object result;
		if((result = this.nameTrie.get(name)) == null) {	
			return -1;		
		} else {
			return Integer.parseInt(result.toString());	
		}
	}

	
	public List<String> triePrefixMatch(String to) {
		return this.nameTrie.matchPrefix(to);		
	}

	
/**
 * Lis nimen nimiIndeksiin. jaetaan mys numero tlt?
 * Hmm.
 * 1. Lydetn 'Aino Ackte', tallennetaan ID:ll 1
 * 2. Lydetn 'Aino'
 * 3. Kysytn nameIndexilt contains? 
 * 4. heitetn addName jos.
 * 
 */
	public int addName(Vector <String> firstnames, String surname) {

		String first = "";
		for( String s : firstnames ) {
//			log.debug("looppi ok");
			first += s + " ";
		}
//		log.debug("first on '"+first+"', firstnames: '"+firstnames+"'");
		first = first.substring(0,first.length()-1);

//		System.out.println("\n-----------------------------");
//		System.out.println("-"+first+" "+surname+"-");
//		System.out.println("-----------------------------");
		Name name;
		int index;
		
		if((index = this.nameInTrie(surname+" "+first)) != -1) { //TSMLLEEN sama nimi ei tehd mitn
			log.debug("Sama nimi lytyi: '"+names.get(index)+"'");
			this.names.get(index).addName(firstnames,surname, false);
			name = names.get(index);  		
			return index;
		} else { // lis uusi nimi
			if(first.matches("\\p{javaLowerCase}\\.")) { // P.
				log.debug("Etunimi tsm lyhennenimeen (P.)");
				if((index = this.getSimilarAbbr(surname,first)) !=-1) {
					log.debug("ABBR: " +surname+" "+first+" tsm olemassaolevaan!");
					this.names.get(index).addName(firstnames,surname, false);
					return index;
				}		
			} else if((index = this.isJWSimilar(surname, firstnames.get(0))) != -1) {
				
				//14-7-06: poistettu vaihtoehtoisten nimien lisys
				//triehen, sill yksittisten nimien samastus
				//ei toimi oikein muuten: TODO: ONKO HUONO RATKAISU?
				
				//this.nameTrie.put(surname+" "+first, index);
				//log.debug("\t--> listtiin nimelle '"+names.get(index)+"' toinen muoto ("+first+" "+surname+")");

				log.debug("\t--> '"+names.get(index) +"' samastettiin nimen '" + first+" "+surname+"' kanssa.");
				this.names.get(index).addName(firstnames,surname, false);
				return index;				
			}
			
			this.counter++;
			log.debug("\t--> Listn uusi nimi: '"+first+" "+surname+"'");
			name = new Name(firstnames, surname, counter); //counter listty 16-06-2006
			this.names.put(counter, name); //treeMap
			this.nameTrie.put(surname+" "+first, counter);
			
			this.addFirstToSimilar(counter, firstnames.get(0));
			return counter;
		}
	
	}
	
	private boolean addFirstToSimilar(int id, String fname) {
		HashSet <Integer> sameNames;
		if(!fname.contains(".")) {
			if(firstSimilar.containsKey(fname))	{ //...ota joukko ksittelyyn...
				sameNames = this.firstSimilar.get(fname);	
			} else { // jos ei, luo uusi...
				sameNames = new HashSet <Integer>();
			}
			sameNames.add(id);
			firstSimilar.put(fname, sameNames);
//			System.out.println("\t--> listtiin nimelle '" + fname+"' ID '"+id+"' etunimijoukkoon.");
			return true;
		}
		return false;
	}
	
	public int addIndividual(String firstOrSurName) {
		log.debug("\tEtsitn etu/sukuvastinetta sanalle  '"+firstOrSurName+"'");
				
		if(this.containF(firstOrSurName)) {
			log.debug("\t--> Oli etunimi...");
			if(this.severalFirst(firstOrSurName)) {
				return -1; //useita, ei voida samastaa
			} else {
				int nameID = this.getSingleFirst(firstOrSurName);
				//lis etunimi laskuriin (listty 25.1.)
				this.names.get(nameID).countIndividual();
				return nameID;
			}
		} else {
			log.debug("\t--> Sukunimen puolella (=ei etunimi)...");
			String shortForm = null;
			int size;
			boolean estimated = false;
			if((size = firstOrSurName.length()) > 2) {
				shortForm = firstOrSurName.substring(0,size-2);				
				estimated = true;
			}
			List <String> l = this.nameTrie.matchPrefix(firstOrSurName);			

// listty JW thn 16.10.06 (=ehdon tiukennus)	
			if(l.size() == 1) { //tsmllinen, listassa yksi vastine
				
				//jos Jaro-Winkler > 0.89, on nimi
				if((jw.getSimilarity(firstOrSurName, l.get(0)))
						>this.JWSIMILAR) {
				
					log.debug("\t--> Oli tsmllinen sukunimi.");
					int nameID = Integer.parseInt(nameTrie.get(l.get(0)).toString());
//					lis sukunimi laskuriin (listty 25.1.)
					this.names.get(nameID).countIndividual();
					return nameID;
				} else {
					return -1;
				}
			} 
// 25.1. Likimrinen sukunimen mtsys pois plt!
// ei toimi: 2 aaltoa, molemmilla samanlaisuus 1.0!
/*			
			else if(estimated) { //likimrinen mtsys
				l = this.nameTrie.matchPrefix(shortForm);

//				System.out.println("\t--> Arvioidaan eptsmllinen sukunimi:");
				double temp;
				double threshold = this.JWSIMILAR;
				boolean nameFound = false; 
				String nameToCompare = null;
				int indexFound = -1;
				for(int i=0; i < l.size(); i++) {
					nameToCompare = l.get(i);
					if(nameToCompare.length() > firstOrSurName.length()) {
						nameToCompare = nameToCompare.substring(0,firstOrSurName.length());
					}
					if((temp =jw.getSimilarity(nameToCompare,firstOrSurName)) > threshold) {
						threshold = temp;
						nameFound = true;
						indexFound = Integer.parseInt(this.nameTrie.get(l.get(i)).toString());
					}
//					System.out.println("\t--> Samanlaisuus: '"+nameToCompare+"' <---> '"+firstOrSurName+"', "+temp);
				}
				if(nameFound) {
//					lis sukunimi laskuriin (listty 25.1.)
					this.names.get(indexFound).countIndividual();
					return indexFound;
				} else {
					return -1;
				}
								
			}
*/
			else {
//				System.out.println("Ei sukunimi, lydettyjen nimien mr: " +l.size());
				log.debug("\t--> ei tsmllist sukunimivastinetta");
				log.debug("\t--> lista sislt seuraavat: "+l.toString());
				return -1;			
								
			}
		}
	}

/**
 * Kytetn kun tiedetn (addIndividual) ett lytyy yksiksitteinen
 * nimivastaavuus etunimelle
 * 
 * @param firstName haettava etunimi
 * @return nime vastaava ID
 */
	private int getSingleFirst(String firstName) {
		try {
			return this.firstSimilar.get(firstName).iterator().next();
		} catch(NoSuchElementException e) {
			return -1;
		}
		
	}

	private int getSimilarAbbr(String sur, String first) {
		first = first.substring(0,1);
		String fullname = sur+" "+first; // . pois perst
		log.debug("Testataan (abbr) lyhenne: '" +fullname+"'");
		List <String> l = this.triePrefixMatch(fullname);
		if(l.size() == 1) {
			log.debug("Yksi vastine lytyy");
			return Integer.parseInt(
					this.nameTrie.get(
							l.get(0).toString()).toString());
			
		} else {
			if(l.size()==0) {
				log.debug("Ei yhtn vastinetta");
				if(sur.length() < 3) { //liian lyhyt sukunimi arvioitavaksi
					return -1;
				}
				String halfOfSur = sur.substring(0, 3);
				l = this.triePrefixMatch(halfOfSur);
				int currentIndex, 
				    result = -1;
				double threshold = JWSIMILAR, //0.89
				    	   increased;
				
				Name temp;
				String firstName, surName;
				
				for(String s: l) {
		    			currentIndex = Integer.parseInt(this.nameTrie.get(s).toString()); 
		    			temp = this.names.get(currentIndex);
		    			
		    			firstName = temp.getFirst().substring(0,1);
			    		if(!first.equals(firstName)) {
			    			continue;
			    		}
			    		
			    		surName = temp.getSur();   		
			    		
//			    		System.out.println("\t--> Verrataan sukunimi: "+sur+" <--> " +surName);
//			    		System.out.println("\t    Samanlaisuus       : "+jw.getSimilarity(sur,surName));

			    		increased = jw.getSimilarity(sur,surName);
			    					
//			    		System.out.println("Sukunimisamanlaisuus: "+ increased);
			    		if(increased  > threshold) { //valitaan samanlaisin
			    			result = currentIndex;
			    			threshold = increased;
			    		}			    		
				}
				if(result != -1) {
//					System.out.println("\t--> Samastetaan nimi nimen " + names.get(result) + " kanssa!");
				} else {
//					System.out.println("\t--> Vastinetta ei lytynyt.");
				}
				return result;
			}
			
			return -1;
		}

	}

	
	private int isJWSimilar(String sur, String first) {   
//		System.out.println("Testataan JW-samanlaisuus");
//		String halfOfSur = sur.substring(0, sur.length()/2+1);

		String halfOfSur;
		if(sur.length() > 2) {
			halfOfSur = sur.substring(0, 3);
		} else {
			return -1;
		}
		List <String> l = this.triePrefixMatch(halfOfSur);
	    double threshold = JWSIMILAR; //0.89
	    double increased,
     		   firstSimilarity;
	    int result = -1,
	     	currentIndex,
	     	meanDivider = 2; //jos verrataan pelkk etunime, ei jaeta
   
	    String firstName, surName;

	    Name temp;
	    for(String s: l) {

	    		currentIndex = Integer.parseInt(this.nameTrie.get(s).toString()); 
	    		temp = this.names.get(currentIndex);
	    	
	    		firstName = temp.getFirst();

	    		if(firstName.matches("\\p{javaLowerCase}\\.")) { //jos Triess P. (kun etunimi on pitk)
//	    			System.out.println("\t--> Puun etunimi muotoa 'P.' ("+firstName+")");
	    			if(firstName.substring(0,1).equals(first.substring(0,1))) {
	    				meanDivider = 1; 
	    				firstSimilarity = 0;
	    			} else { // hyltn (jos etukirjaimet eivt tsm)...
//	    				System.out.println("Etukirjaimet eivt tsm, hyltn.");
	    				continue; 
	    			}
	    		} else {
//	    			System.out.println("\t--> Verrataan etunimi: "+first+" <--> " +firstName);    		
    				meanDivider = 2; 
    				firstSimilarity = jw.getSimilarity(first,firstName);
//  				System.out.println("\t    Samanlaisuus      : "+firstSimilarity);
	    		}
	    		surName = temp.getSur();   		
	    		    		
//	    		System.out.println("\t--> Verrataan sukunimi: "+sur+" <--> " +surName);
//	    		System.out.println("\t    Samanlaisuus       : "+jw.getSimilarity(sur,surName));

	    		increased = (firstSimilarity +
	    					jw.getSimilarity(sur,surName))/meanDivider;
	    					
//	    		System.out.println("Samanlaisuuden keskiarvo: "+ increased);
	    		if(increased  > threshold) { //valitaan samanlaisin
	    			result = currentIndex;
	    			threshold = increased;
	    		}
	    		
	    }
		if(result == -1) {
//			System.out.println("\t--> Vastinetta ei lytynyt.");
		}
	    
	    return result;
	}

	/**
	 * Olemassaolotesti PokaNameExtractorille, tarkistaa,
	 * lytyyk triest nimi, jotka alkavat annetulla
	 * sukunimell, jos lytyy, tiedetn, ett
	 * oletettu nimi loppuu kyseiseen sanaan, esim:
	 * 'Alvar Aalto Sti' --> 'Alvar Aalto'
	 * @param surname
	 * @return
	 */
	
	public boolean containsSurname(String surname) {
		if(this.triePrefixMatch(surname).size() == 0) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Uutta 12.9.06: listn nimioliolle tiedot
	 * esiintymien sijainnista dokumentin sisll
	 * 
	 * TODO: pit mietti poistetaanko nimioliot
	 * (tai nm sijaintitiedot) *AINA*
	 * kun seuraava dokumentti vaihdetaan: jos ei,
	 * eri dokumenttien nimitiedot voivat menn sekaisin
	 * 
	 * @param nameID
	 * @param nameToken
	 */
	public void addLocation(int nameID, Node nameToken) {
		
		Name temp = this.names.get(nameID);
		if(temp == null) {
			return;
		}
		
		//coordinaattisolmun indeksi
		int lastChild = nameToken.getChildNodes().getLength()-1;
		
		//tulostetaan node
//		System.out.println(DomDebugger.printNode(nameToken));

		//parsitaan alkuindeksi
		int start = Integer.valueOf(nameToken.getChildNodes().item(lastChild).
						getChildNodes().item(0).getTextContent());
		//parsitaan loppuindeksi
		
		int end = Integer.valueOf(nameToken.getChildNodes().item(lastChild).
						getChildNodes().item(1).getTextContent());
				
		temp.addLocation(start, end);
//		this.names.put(nameID, temp);
	}
	
	public ArrayList<Location> getLocation(int nameID) {
		return this.names.get(nameID).getLocations();
	}
	
	
/**
 * Returns String-representation
 * of first name and surname similarity
 * groups.
 * 
 * @return first name and surname similarity sets as a String
 * @see #addNames
 */
	public String getGroups() {
		String firstSim = "",
               surSim = "";	
		
		int k;
		HashSet <Integer> temp;
		Iterator <HashSet<Integer>> i = firstSimilar.values().iterator();
		while(i.hasNext()) {
			temp = i.next();
			if(temp.size() > 1) {
				Iterator <Integer> j = temp.iterator();
				while(j.hasNext()) {
					k = j.next();
					firstSim += this.getFirst(k) + " " +
                              this.getSur(k) + "\n";
				}
				firstSim += "-------------\n";
			}
		}
		i = surSimilar.values().iterator();

		while(i.hasNext()) {
			temp = i.next();
			if(temp.size() > 1) {
				Iterator <Integer> j = temp.iterator();
				while(j.hasNext()) {
					k = j.next();
					surSim += this.getFirst(k) + " " +
                              this.getSur(k) + "\n";
				}
				surSim += "-------------\n";
			}
		}


		return  "---------\n" +
				"Etunimet:\n" + 
				"---------\n" +
			    firstSim+"\n"+
			    "----------\n" +
				"Sukunimet:\n" + 
				"----------\n" +
			    surSim;
	}


//-------------------
// INNER CLASS NAME
//-------------------	
/**
 * Each identified name is saved to a Name-object.
 * Name-instances are associated in TreeMap-structure
 * of NameIndex.
 *
 * <pre>
 * TO DO: Comparable-mreen voi luultavasti poistaa!
 * </pre>
 */ 
	public class Name implements Comparable<Name> {
				
/**
 * Name can have multiple forms of current surname, for example Acht and
 * Ackt. Each surname is stored in this linked list.
 * Primary surname is the shortest form of surname which 
 * is added last.
 */
		private LinkedList<String> surNames = new LinkedList <String>();
/**
 * Name can have multiple first names, for example Martti Olavi Ahtisaari
 * Each first name is stored in this linked list.
 */
		private LinkedList<String> firstNames = new LinkedList <String>();

		private int nameID; //nimen id-numero aineistossa
		private int nameFrequency; //yhden dokumentin sisll
		
		//sijaintitiedot, nimien sijainti tekstiss
		private ArrayList<Location> location;
		
		//regex pienell kirjoitettaville sukunimien aluille:
		//FIXME: lis nm sukunimiparsijaan! (16.10.06)
		private String lowercaseSurnames = "af |von ";
		
/**
 * New name-object is created if NameIndex doesn't have name
 * with current ID. So, constructor is used to add a new 
 * instance of name at the first time. Later, when ID exist,
 * addName-method is used to add name to a current ID.
 * 
 * @param id identification number of the name
 * @param name the actual name 
 * @see #addName(String,boolean)
 */
		private Name(Vector<String> firstnames, String surname, int counter) {
			this.addName(firstnames, surname, true);
			this.location = new ArrayList <Location>();
			
			this.nameFrequency = 1;
			this.nameID = counter;
		}

/**
 * When Name-instance with current ID exist, this 
 * method is used to update possible new form of
 * the name. (A. Aalto --> Aino Aalto)
 *
 * @param name String-rep of name
 * @param isFirst constructor uses true, when adding name for the first time, 
 *  later false is used. 
 */		
		private void addName(Vector <String> firstnamesNeu, String surname, boolean isFirst) {
					
			if(isFirst) {

				for(String s:firstnamesNeu) {
					firstNames.add(s);
				}		
				surNames.add(surname);

			} else {
				
				this.nameFrequency++;
				
				String temp = "", fn;

				int nameI = 0;
				
				for(String s: firstnamesNeu) {

					temp = s;
					
					//jos listtv sana, temp, on pidempi 
                    //tee catch-osuus (=lis sana suoraan)
					try {
				    		fn = firstNames.get(nameI);
						if(!temp.equals(fn)	) {
//							System.out.println("sanat '" +fn+"' ja '" +temp+ "' erit.");
							if(fn.contains(".")) {
//								System.out.println("'"+fn +
//									 "' sis. pisteen, korvataan sanalla '" +
//                                     temp + "'.");
								firstNames.remove(nameI);
								firstNames.add(nameI, temp);
							}
						}
					} catch(IndexOutOfBoundsException e) {
//						System.out.println("(CATCH)-----> TOkenisssa enemmn tavaraa kuin ETUnimess.");
						firstNames.add(nameI, temp);		
					}	
					nameI++;
				}

				temp = surname;

				// verrataan sukunimet,
                // ensisijainen sukunimi on lyhin muoto!
				if(!temp.equals(surNames.getFirst())) {
					if(temp.length() < surNames.getFirst().length()) {
						surNames.addFirst(temp);
					} else {
						surNames.add(temp);
					}
				}
//				System.out.println("Nimess jlkeen:\n" + this.toString());
			}
			
		}		

		
		/**
		 * Listn yksittisen nimen esiintym frekvenssiin
		 * (listty 25.1.)
		 */
		public void countIndividual() {
			this.nameFrequency++;
		}
		
		public int getFrequency() {
			return this.nameFrequency;
		}
		public int getID() {
			return this.nameID;
		}
/**
 * Returns surname as a String
 * 
 * @return String representation of primary surname
 */
		public String getSur() {
			return this.upperCase(surNames.getFirst());
		}

/**
 * Returns first name as a String
 * 
 * @return String representation of first name
 */
		public String getFirst() {
			return firstNames.getFirst();
		}	
/**
 * Palauta kaikki etunimet
 */
		public String getFirsts() {
			String result = "";
			for(String s:firstNames) {
				result += this.upperCase(s)+" ";
			}
			return result;
		}
		
		private String upperCase(String text) {
			if (text == null || text.length() == 0) {
				return text;
			}
			
			text = text.replaceAll("#", ""); //listty 14.6.06
						
			text = text.substring(0, 1).toUpperCase()+
						text.substring(1, text.length());

//TODO: useita tavuviivoja (=Gallen-Kallela-Korhonen)!
//Voipio-juvas muotoon Voipio-Juvas
			int hyphenIndex = text.indexOf('-');
			if(hyphenIndex != -1 &&
			   text.length() > hyphenIndex+2) {
//				log.debug("HYPH: "+text);
				text = text.substring(0,hyphenIndex+1)+
					   text.substring(hyphenIndex+1, hyphenIndex+2).
					   						toUpperCase()+
					   text.substring(hyphenIndex+2, text.length());
//				log.debug("ISOKIRJAIN: "+text.substring(hyphenIndex,hyphenIndex+1));
//				log.debug("--> TEXT: "+text);
			}
			return text;
		}
		
		public int compareTo(Name another) throws ClassCastException {
		    if (!(another instanceof Name))
		      throw new ClassCastException("Name: Name expected: ");
		    int anotherFreq = ((Name)another).getFrequency();  
		    return this.getFrequency() - anotherFreq;    
		}
		
		
		public String toString() {
			String result = ""; 
			Iterator<String> i = this.firstNames.iterator();
			while(i.hasNext()) {
				result += this.upperCase(i.next())+ " ";
			}

			i = this.surNames.iterator();          
			result += this.upperCase(i.next());
	
			return result;
		}

		public void addLocation(int start, int end) {
			this.location.add(new Location(start, end));		
		}

		public ArrayList<Location> getLocations() {
			return this.location;
		}
	}
	
	public static void main(String [] args) {
		NameIndex mmm = new NameIndex();
		Vector <String> vee = new Vector <String> ();
		vee.add("aino");

		String surname = "ackt";

		System.out.println("\t--> numero: " + mmm.addName(vee, surname));
		System.out.println("\t--> numero: " + mmm.addName(vee, surname));

		
		vee.set(0,"a.");
		System.out.println("\t--> numero: " + mmm.addName(vee, surname));
		

		surname = "gustafsson";
		System.out.println("\t--> numero: " + mmm.addName(vee, surname));
		
		vee = new Vector <String> ();
		vee.add("aino");
		surname = "acktta";
		System.out.println("\t--> numero: " + mmm.addName(vee, surname));
		
//		System.out.println(mmm.getGroups());
		
		
	}

	public void clear() {
		this.names.clear();
		nameTrie = new TernarySearchTrie();
		firstSimilar.clear();
		surSimilar.clear();
	}
	
}