package fi.seco.semweb.service.poka.opas;
/**
 * 
 * @author alm
 *
 * Ker talteen olennaiset tiedot ksitteest:
 * lemma: lemmatisoitu muoto ilman risuaitaa
 * text:  tekstiss esiintyv alkuperinen muoto
 * uriMap / key  : uri, pitk muoto
 * uriMap / value: HashSet, ylluokan labelit
 */

import java.util.List;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.TreeMap;
/**
 * Luokka esitt yksittist ksitteeseen
 * tsmnnytt tekstiesiintym (=token) 
 * PokaDokkarissa. Sanaa kohden voi olla n kappaletta
 * tsmnneit ksitteit, joista tallennetaan URI,
 * label ja ylksiteinformaatio. Ylksiteinfo
 * voi olla esim ylksitteen label tai uri
 *
 * @author alm
 *
 */

public class ConceptMatch implements Serializable {

	//HUOM: typer toisteisuutta. Jokaiselle 
	//conceptMatchille on omat urijoukkonsa.
	// jos 10 samaa sanaa, jokaiselle toisteisesti
	//mtsvt labelit ja urit 
	
	/**
	 * Occurence esitt tietty sanaa vastanneen 
	 * tietyn (yhden) urin tiedot. 
	 * Counter listty tnne: counter on ksitekohtainen, so.
	 * kahdella ksitteell, joilla on sama label, voi olla 
	 * eri esiintymmr, esim:
	 * #koira_1: label 1: 'koirat'
	 * #koira_2: label 1: 'koirat' label 2: 'rakit
	 * --> #koira_2:lla voi olla enemmn osumia kuin #koira_1:ll
	 * 
	 */
	public class Occurence implements Serializable {
		/**
		 * 
		 */
		private static final long serialVersionUID = -2688583462045778567L;

		private String label; // vain yksi label

		private ArrayList<String> upperClassInfo;//monta ylluokkaa?
		
		private int count;
		
		private Occurence() {
			this.count = -1; //oletus
		}
		
		private void setCount(int count) {
			this.count = count;
		}
		public int getCount() {
			return this.count;
		}
				
		private void setLabel(String label) {
			if(label == null) {
				label = "";
			}
			this.label = label;
		}
		public String getLabel() {
			return this.label;
		}
		private void setUpper(ArrayList <String> upperClassInfo) {
			this.upperClassInfo = upperClassInfo;
		}
		public ArrayList<String> getUpper() {
			if(this.upperClassInfo==null) {
				return new ArrayList<String>();
			}
			return this.upperClassInfo;
		}		
		
	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 8063191818205866924L;

	//huom, vain yksi count: jos useita ureja joilla sama 
	//label, niin kaikilla ureilla sama esiintymmr 
	private int count;
	
	private String text;
	
	private TreeMap<String, Occurence> uriToInfo;
	
	public ConceptMatch() {
		this.uriToInfo = new TreeMap<String, Occurence>();
	}

	public void setCount(int count) {
		this.count = count;
	}
	public int getCount() {
		return this.count;
	}
	
	public void setText(String text) {
		this.text = text;
	}
	public String getText() {
		return this.text;
	}
	
	public String getLabel(String uri) {
		return this.uriToInfo.get(uri).getLabel();
	}
	public List<String> getUpper(String uri) {
		return this.uriToInfo.get(uri).getUpper();
	}
	
	/**
	 * Gets an arbitary label from the occurrences. 
	 * (this is used for proper nouns with Opas)
	 * @return
	 */
	public String getFirstLabel() {
		if (this.uriToInfo == null || this.uriToInfo.size() == 0) {
			return null;
		}		
		Occurence occurence = uriToInfo.get(uriToInfo.firstKey());
		return occurence.getLabel();
	}
	
/**
 * Lis kaiken tiedon kerralla
 * TODO: tyhm nimi: NameMatch ly urin kohdalle nimen ID:n
 * 
 * @param uri 
 * @param label
 * @param upperInfo
 */
	
	public void addUri(String uri, String label, ArrayList <String> upperInfo, int count) {
		Occurence uriInfo = new Occurence();
		uriInfo.setLabel(label);
		uriInfo.setUpper(upperInfo);
		uriInfo.setCount(count);
		if (this.count < count) {
			this.count = count;
		}		
		this.uriToInfo.put(uri, uriInfo);
	}
	
	public TreeMap<String, Occurence> getUris() {
		return this.uriToInfo;
	}
	
	
}
