/**
 * Opasta varten rtlity eristin: tarkentaa
 * GeneralExtractoria listen siihen instanssitoiminnallisuudet.
 * Mainissa esimerkki conceptMatchin kytst.
 */
package fi.seco.semweb.service.poka.opas;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import fi.seco.semweb.service.poka.Configuration;
import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.name.NameIndex.Name;
import fi.seco.semweb.service.poka.opas.ConceptMatch.Occurence;
import fi.seco.semweb.service.poka.util.FoundResources;
//import fi.seco.semweb.service.poka.util.PokaInstance;
import fi.seco.semweb.service.poka.util.PokaResource;
import fi.seco.semweb.service.poka.util.TermoQuery;
import fi.seco.semweb.service.poka.util.TrieInstanceIndex;
import fi.seco.semweb.service.poka.util.ContextInterface;
import fi.seco.semweb.service.poka.util.TrieInterface;
import fi.seco.semweb.service.poka.util.YsoQuery;
import fi.seco.semweb.service.poka.util.Location;

public class OpasExtractor extends GeneralExtractor {
	private static final Log log = LogFactory.getLog(OpasExtractor.class);
	
	private static final int YSOKB = 1;
	private static final int PLACEKB = 2;
	private static final int VERBKB = 3;
	private static final int REGEXKB = 4;
	private static final int INSTKB = 5;
	
	private final String INSTNODE = "IN";
		
	private TrieInstanceIndex instanceIndex;
//	private PokaInstance instancesFound;
	private FoundResources instancesFound;
	//private String opasTermFile = "/home/alm/termo/OpasTerms.rdf";
	private String encoding = "ISO-8859-1";	
	private String termNS = "http://www.seco.hut.fi/ns/2005/10/alm#";
	
	
	public OpasExtractor() {
		super();
	
		this.instanceIndex = new TrieInstanceIndex(Configuration.getOpasTermFile(), encoding, termNS);
//		this.instancesFound = new PokaInstance();
		this.instancesFound = new FoundResources();
	}
	
	public void addInstances() {
		this.addConcepts(INSTNODE, 
						 OpasExtractor.CONCEPTNODES,
						 true,
						 false,
						 instanceIndex,
						 null,
						 instancesFound,
						 true);
						 
	}
	
	public void saveInstances() throws IOException {
		this.instanceIndex.writeFile(Configuration.getOpasTermFile());
	}
	
	public ArrayList<ConceptMatch> getPlaces() {
		String [] place = {PLACE};
		return this.getConcepts(place, PLACEKB);	
	}
	
	public ArrayList<ConceptMatch> getConcepts() {
		String [] concept = {CONCEPT};		
		return this.getConcepts(concept, YSOKB);	
	}
	
	public ArrayList<ConceptMatch> getRegex() {
		String [] regex = {REGEX};
		return this.getConcepts(regex, REGEXKB);	
	}
	//TODO: helvetin huono nimi
	public ArrayList<ConceptMatch> getVerbNodes() {
		String [] verb = {VERB};
		return this.getConcepts(verb, VERBKB);	
	}
	
	public ArrayList<ConceptMatch> getInstNodes() {
		String [] inst =  {INSTNODE};
		return this.getConcepts(inst, INSTKB);
	}
/**
 * Seuraavat metodit palauttavat sisnkrityt resurssi-indeksit...
 * (=mit luokkia on lydetty)
 * Esimerkiksi osumalaskuri lytyy tlt, samoin sijaintitiedot
 * 
 * @param uri
 * @return
 */
	public FoundResources getYsoResources() {
		return this.ysoInstances;
	}
	public FoundResources getPlaceResources() {
		return this.placeInstances;
	}
	public FoundResources getOpasResources() {
		return this.instancesFound;
	}
	public FoundResources getVerbResources() {
		return this.verbInstances;
	}
	public FoundResources getRegexInstances() {
		return this.regexInstances;
	}

//-------------------------------------------
	
	
	public void clearPokaInstance() {
		super.clearPokaInstance();
		this.instancesFound.clear();
	}

//info nodesta parsitaan urit esiin
	private HashSet<String> getInstUris(String nodeInfo) {
		
		HashSet <String> result = new HashSet<String>();
		int sep = nodeInfo.indexOf('|'); //first occurence of '|'
		
		if(sep==-1) {
			log.error("VIRHE: <I>-solmun pitisi sislt '|' -merkki!");
			return result;
		}
		do {		
			String temp = nodeInfo.substring(0,sep).trim();
			result.add(temp);
			nodeInfo = nodeInfo.substring(sep+1);
			sep = nodeInfo.indexOf('|');
		} while(sep != -1);
		
		return result;
	}

//TODO: rajapintaluokat kyttn TermoQueryn alaluokille!
//TODO: rajapintaluokat kyttn TrieIndex-luokille!
	
	//palauttaa ksitesolmuista kuvaukset Lemmalta Urijoukolle HashMappina
	private ArrayList<ConceptMatch> getConcepts(String [] nodeNames, int KB) {
		
		TrieInterface index = null;
		FoundResources inst = null;
		ContextInterface query = null;
		boolean lemmaUpperCase = false;
		boolean urisReferInstances = false;
		
		switch(KB) {
		
		case (YSOKB):
			log.debug("Asetetaan YSO-kanta");
			index  = this.termExist;
			inst   = this.ysoInstances;
			query  = this.yso;
			break;
		case (PLACEKB):
			log.debug("Asetetaan Paikka-kanta");
			index  = this.placeExist;
			inst   = this.placeInstances;
			query  = this.places;
			lemmaUpperCase = true;
			break;
		case (VERBKB):
			log.debug("Asetetaan Verbi-kanta");
			index  = this.verbExist;
			inst   = this.verbInstances;
			query  = this.yso; //TODO: tulevaisuudessa verbien omat ylluokat
			break;
		case (REGEXKB):
			log.debug("Asetetaan Regex-kanta");
			index  = this.termExist;
			inst   = this.regexInstances;
			query  = this.yso;
			break;
		case (INSTKB):
			log.debug("Asetetaan Opas-instanssikanta");
			index = this.instanceIndex;
			inst  = this.instancesFound;
			query = this.yso;
			urisReferInstances = true;
			lemmaUpperCase = true;
		}
		
		ArrayList <ConceptMatch> result = new ArrayList<ConceptMatch>(); 

		String expression = "";
		log.debug("Kydn lpi seuraavat solmut: ");
		for(String s: nodeNames) {
			log.debug("<"+s+">");
			expression += "/A/S/"+s+"|";
		}
		expression = expression.substring(0,expression.length()-1);		

		NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();	

		String text = "", 
		       label = "",
		       form;
		
		boolean firstPart = false;
		HashSet <String> uris;
				
		try {
			nodes = (NodeList) xpath.evaluate(expression, doc, XPathConstants.NODESET);
			log.debug("lpikydn dokumentti");
			for(int i = 0; i < nodes.getLength(); i++){
				ConceptMatch cm = new ConceptMatch();
				Node item = nodes.item(i);

				if(firstPart) {
					text  += " " +item.getChildNodes().item(0).getTextContent();
//					label += " " +item.getChildNodes().item(2).getTextContent();

					log.debug("ksite jatkuu: "+text);					
				} else {
					text  = item.getChildNodes().item(0).getTextContent();
//					label = item.getChildNodes().item(2).getTextContent();
					
					log.debug("tsmv sana: "+text);
/*					
					if(lemmaUpperCase) {
						label = label.substring(0,1).toUpperCase()+
								label.substring(1);
					}
*/				
				}
				//jos ksitteen info-solmussa numero, ohitetaan (=monisanainen ksite)
				String info = item.getChildNodes().item(1).getTextContent();
				if(info.matches("\\d")) {
					firstPart = true;
					continue;
				} else {
					firstPart = false;
					uris = this.getInstUris(info);
				}
/*
				//huom, luotetaan, ett tll sama jrjestys.
				if(!uris.isEmpty()) { 
					log.debug("Ksitteit lytyy, listn osumat.");
					for(String s: uris) {
						log.debug("uri  : " + s);
						log.debug("hitit: " + inst.getHits(s));
						cm.setCount(inst.getHits(s));
					}					
				}
*/
/*				
				if(urisReferInstances) {
					HashSet <String> classUris = new HashSet<String>();
					for(String s:uris) {
						classUris.add(index.getType(s));
					}
					uris = classUris;
				}
				log.debug("URIS: "+uris);
*/				
				
				cm.setText(text);
							
				//looppaa urit lpi ja ota kontekstitieto
				for(String s: uris) {
					String resUri;
					log.debug("URI  : "+s);
					if(urisReferInstances) {
						resUri = index.getType(s);
					} else {
						resUri = s;
					}
					log.debug("LABEL: "+query.getLabel(resUri));
					ArrayList <String> upperC = query.getContextInformation(resUri);
					cm.addUri(s, query.getLabel(resUri), upperC, inst.getHits(s));
				}						
				result.add(cm);				
			}
		} catch (XPathExpressionException e) {
			System.out.println("PokaDom / getConcepts: " + e);
		}
		return result;
	}
	
	public static void main(String [] args) {
		
		Logger hephep = Logger.getLogger("fi.seco.semweb.service.poka");
		hephep.setLevel(Level.DEBUG);
		
		OpasExtractor hep = new OpasExtractor();

		hep.setInputString("Miten nuorena kuollut esseisti ja kriitikko Tatu Vaaskivi "+
				   "hankki hmmstyttvn laajat tietonsa kirjallisuudesta, taiteesta, "+ 
"historiasta, kulttuurihistoriasta, psykoanalyysist? Mik mahtoi olla "+ 
"syyn hnen varhaiseen poismenoonsa (vuonna 1942?). Onko hnest "+ 
"kirjoitettu elmkertaa, tai mist saisi listietoa hnen elmstn? "+ 
"Teoksessaan \"Tie Roomaan\" hn antaa ymmrt, ett Roomassa oleskelun "+ 
"aikana vuonna 1939 hnell oli mukana vaimonsa, jonka nime ei "+ 
"kuitenkaan mainitse. Oliko kyseess Elina Vaara? Kyselen nit "+ 
"kiinnostuksesta hnen elmntytn kohtaan.");

		
		System.out.println(hep.getDocument());		
		
//		hep.setInputString("apinoita ja gorilloita kvelivt torilla, modernia taidetta!");
//		hep.setDocument("/home/alm/termo/penkki/apina.txt");

//		hep.setDocument("/home/alm/aaparser/text/mannerheim.txt");
		hep.addInstances();
		hep.addNames();
		hep.addPlaces(OpasExtractor.CONCEPTNODES,true,false);
		hep.addRegex("[1-2][8-9|0][0-9][0-9]", "http://yso.fi/YSO#vuosi");
		hep.addRegex("\\d(\\d)?\\.\\d(\\d)?\\.\\d\\d\\d\\d","http://yso.fi/YSO#pivmrt");
		hep.addRegexConcepts(OpasExtractor.NUMERALNODES, false);
//		hep.addVerbs(false, false);
		hep.addYsoConcepts(OpasExtractor.CONCEPTNODES, true, false);

	

		Collection<ConceptMatch> list = hep.getConcepts();
		list.addAll(hep.getPlaces());
		list.addAll(hep.getInstNodes());
		
		Collection<Name> names = hep.getNames();
		for (Name n : names) {
			NameMatch nm = new NameMatch();
			nm.addUri(""+n.getID(), n.toString(), null, n.getFrequency());//TODO:purkat pois
			list.add(nm);
		}
		
		for(ConceptMatch cm: list) {
			System.out.println("--------------------");
			System.out.println("Text: "+cm.getText());
					
			TreeMap <String, Occurence> piu = cm.getUris();
			
			Set <String> keys = piu.keySet();
			for(String s: keys) {
				Occurence o = piu.get(s);
				System.out.println("key  : "+s);
				System.out.println("count: "+o.getCount());
				System.out.println("label: "+piu.get(s).getLabel());
				for(String upper:piu.get(s).getUpper()) {
					System.out.println("upper: "+upper);
				}
			}
		}

		
/* ESIMERKKI URI--> termiesiintymmppysten iteroinnista		
		FoundResources temp = hep.getPlaceResources();
		Set<String> uusi = temp.getUris();
		System.out.println("-----------\n-----------");
		for(String s: uusi) {
			System.out.println("URI : "+s);
			System.out.println("FREQ: "+temp.getHits(s));
			PokaResource r = temp.getInstance(s);
			System.out.println("Sijainnit: ");
			ArrayList<Location> aa = r.getLocations();
			Iterator<Location>ii = aa.iterator();
			while(ii.hasNext()) {
				Location l = ii.next();
				System.out.println("("+l.getStart()+", "+l.getEnd()+")");
			}
			System.out.println("----------");
 		}
		
*/			
		
	}

	
}
