package fi.seco.semweb.service.poka.tokenIndexer;

/**
 * Luokan avulla saadaan fdg:n tokeneille selvitetty alkuperiset
 * sijainnit tekstidokumentissa. Tll hetkell toimii tekstille,
 * laajennos html:n puolelle suht helppo tehd, joskin tiettyj ongelmia
 * ilmenee fdg:st johtuen. Ei viel valjastettu kyttn, otetaan
 * kyttn kun joku tt kaipaa. Osoittaminen HTML-dokumenttiin 
 * toteutusta (ja suunnitelman tarkennuksia) vailla.
 * 
 * HUOM, TODO: tyylikkmmksi, tll hetkell kaksi
 * eri tavalla toimivaa indeksointitoimintoa, molemmille
 * omat konstruktorit. 
 * --> purkkaa, josta ei ota helposti selv...
 *  * 
 * 
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.StringTokenizer;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import net.axyana.qizxopen.xquery.XQResultSequence;
import net.axyana.qizxopen.xquery.XQueryException;

import fi.seco.semweb.service.poka.GeneralExtractor;
import fi.seco.semweb.service.poka.util.XFDGConnect;

import org.htmlparser.Node;
import org.htmlparser.Parser;
import org.htmlparser.beans.StringBean;
import org.htmlparser.nodes.RemarkNode;
import org.htmlparser.nodes.TagNode;
import org.htmlparser.nodes.TextNode;
import org.htmlparser.parserapplications.StringExtractor;
import org.htmlparser.util.NodeIterator;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;
import org.htmlparser.util.Translate;
import org.htmlparser.visitors.TextExtractingVisitor;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class TokenIndexer {
	private static final Log log = LogFactory.getLog(TokenIndexer.class);
	public class Token {
			
		private int start;
		private int end;
		private boolean imNull = false;
		private boolean noMatch = false;
		private boolean continues = false;
		
		//debuggausta varten, poistetaan
		private String text;
		
		private Token(int start, int end) {
			this.start = start;
			this.end = end;
			
		}
		public int getStart() {
			return this.start;
		}
		public int getEnd() {
			return this.end;
		}
		public String toString() {
			return "("+getStart()+", "+getEnd()+")";
		}
		public void setNull() {
			this.imNull = true;
		}
		public boolean isNull() {
			return this.imNull;
		}
		public void setTokenContinues() {
			this.continues = true;
		}
		/**
		 *
		 *tt ei kytet viel, ehkei
		 *koskaan:
		 * tarkoitus oli, ett continues merkkaa jatkuuko
		 * sana / token html-tagin yli, jos nin, niin
		 * voitaisiin merkata spannilla molemmat tokenin
		 * puolikkaat toisistaan erillisin. Toisaalta,
		 * tilanne esiintyy melko harvoin, joten ei toteuteta
		 * viel: toteutus olisi vaatinut poka-dokumentin
		 * rakenteen muuttamista location-tagien osalta:
		 * yksittisen tokenin olisi pitnyt pysty viittaamaan
		 * kahteen tai useampaan lhes perkkiseen sijaintiin.
		 */
		
		
		public boolean continues() {
			return this.continues;
		}
		
		public void setNoMatch() {
			this.noMatch = true;
		}
		public boolean isNoMatch() {
			return this.noMatch;
		}

		//DEBUG
		public void setStr(String text) {
			this.text = text;
		}
		//DEBUG
		public String getStr() {
			return this.text;
		}
		
	} 
	
	/**
	 * lukuarvo, jonka verran pit vhent dekoodattuun html-tagiin
	 * kohdistettua osoitinta (enkoodatusta html-tagista, jotta kohdistin
	 * osuu dekoodatussa oikeaan paikkaan. Dekoodatussa versiossa
	 * entityjen muuttaminen merkeiksi lyhent dekoodatun pituutta. 
	 */
	private int htmlEntitySubtraction;

	//html:n korjausmerkit, pariton muutetaan seuraavaksi parilliseksi dokumentissa
	//korjausmerkit korjaavat HTML:n sisllst tietyt merkit toisilla
	public final Character[] CHARCORRECTION = {'','-'}; 

	//ohitettavat html-tagit, nopeuttaa ksittely
	public final String[] BYPASSTAGS = {"META", "STYLE","SCRIPT"};
	
	private HashSet <String> tagset;

//	public final String[] byPassTags;
	
	private StringTokenizer tokenizer;
	private StringBuffer inputText;
	private String encoding = "ISO-8859-1";
	private int textLen;
	
	private String htmldoc;
	
	private org.w3c.dom.Document pokaDocument;
	private org.w3c.dom.NodeList pokaNodes;
	private int pokaNodeCounter;
	
	private org.w3c.dom.Node textNode;	
	private String pokaTokenText;
	private Parser htmlparser;
	private TextExtractingVisitor textVisitor;
	
	private ArrayList <Token> tokens = new ArrayList<Token>();
	private XQResultSequence fdgSequence;
	
	private boolean tokenFound = true;
/**
 * teksti-indekserin konstruktori
 * @param urlOrInput
 * @param isURI
 */	
	public TokenIndexer(String urlOrInput, boolean isURI) {
		String temp;
		if(isURI) {
			temp = this.readSource(urlOrInput).toString();
		} else {
			temp = new StringBuffer(urlOrInput).toString();
		}
		this.textLen = temp.length();

		//vaihdetaan CHARCORRECTIONIN sisltmt kummat
		//merkit normaaleihin (=enkoodausongelman ratkaisu)
		for(int i=0; i< this.CHARCORRECTION.length; i+=2) {
			Character s = this.CHARCORRECTION[i];
			temp = (temp.replace(s, this.CHARCORRECTION[i+1]));
		}
		this.inputText = new StringBuffer(temp);
	}
	
/**
 * HTML-parseri-konstruktori
 * 23.10.: qizx-xquery-objektin sijasta sisn otetaan pokadokkari
 * @param url documentin url tai sislt
 * @param isURL jos true, url on url, muuten sislt
 * @param pokadoc dokumentista luotu poka-Dom-puu
 */
	public TokenIndexer(String url, boolean isURL, Document pokadoc) 
									throws IndexingFailedException {

		this.htmlEntitySubtraction = 0;
		
		this.pokaDocument = pokadoc;
		this.pokaNodes = this.getTextNodes(pokadoc);
		this.textNode = null;
		

		//ohitettavat solmut, so. solmut,
		//joissa ei ole html-sislt: meta ja style
		this.tagset = new HashSet <String>();
		for(String s: this.BYPASSTAGS) {
			this.tagset.add(s);
		}
			
		String temp;
		
		if(isURL) {
			temp = this.readSource(url).toString();
		} else {
			temp = new StringBuffer(url).toString();
		}
		this.textLen = temp.length();		
	

		
		//merkit normaaleihin (=enkoodausongelman ratkaisu)
		//UTF-8 merkit korvataan vastaavilla ISO-solmuilla
		for(int i=0; i< this.CHARCORRECTION.length; i+=2) {
			Character s = this.CHARCORRECTION[i];
			temp = (temp.replace(s, this.CHARCORRECTION[i+1]));
		}

		this.htmldoc = temp;
		
		this.inputText = new StringBuffer(this.htmldoc);
//		log.debug("\n---INPUTEXT---\n"+temp);
		
		try {
			this.htmlparser = new Parser();
			htmlparser.setInputHTML(this.htmldoc);
			
			StringBean sbean = new StringBean();
			sbean.setLinks(true);
			
			NodeIterator ni = htmlparser.elements();
			while(ni.hasMoreNodes()) {
				this.parseNodes(ni.nextNode());
			}
		} catch(ParserException e) {
			System.err.println("PARSEREXP!");
			System.exit(0); //TODO: hoitele jrkevsti
		}
		
		
	}
	
	public org.w3c.dom.Document getDoc() {
		return this.pokaDocument;
	}
	
	public String getParsedHTML() throws ParserException{
		this.htmlparser = new Parser();
		this.htmlparser.setInputHTML(htmldoc);

		NodeIterator ni = this.htmlparser.elements();
		if(ni.hasMoreNodes()) {

			return ni.nextNode().toHtml();
		}
		return null;
	}
	
	private org.w3c.dom.NodeList getTextNodes(Document pokadoc) 
						throws IndexingFailedException {
		
		this.pokaNodeCounter = 0;
		log.debug("Muodostetaan nodelist");
		String expression = "/A/S/*/T"; // /analysis/sentence/<ALL>/text
		org.w3c.dom.NodeList nodes;
		XPath xpath = XPathFactory.newInstance().newXPath();

		try {
			nodes = (org.w3c.dom.NodeList) xpath.evaluate(expression, pokadoc, XPathConstants.NODESET);
			log.debug("NodeList ok.");
			return nodes;
		} catch(XPathExpressionException e) {
			throw new IndexingFailedException("Virheellinen DOM-kysely.");
		}
		
		
	}
/**
 * HTML-dokumentin solmujen parsija:
 * 
 * @param HTML-node
 * @throws ParserException
 */	
	private void parseNodes(Node n) throws ParserException{

		
			//jos HTML:n tekstinode, tsmytetn se
			//pokadokkarin sisltn
			if(n instanceof TextNode) {
				TextNode tn = (TextNode)n;
				    		
//				log.debug("ALKAA: "+tn.getStartPosition());
				String htmlText = tn.getText();
				log.debug("Htmltext: '"+htmlText+"'");
				
				this.parseHTML(htmlText, tn.getStartPosition(),
      											 tn.getEndPosition());
				
			}
			if (n instanceof RemarkNode) {
		         // downcast to RemarkNode
		         RemarkNode remark = (RemarkNode)n;
		         //log.debug("Remarknode: '"+remark.toPlainTextString()+"'");
		         // do whatever processing you want with the comment
		     } else if (n instanceof TagNode) {
		         // downcast to TagNode
		         TagNode tag = (TagNode)n;
//		         log.debug("TAG: '"+tag.getTagName()+"'");
		         
		         
		         //HUOM: getTagName palauttaa tagin nimen aina UPPERCASE-muodossa
		         if(this.tagset.contains(tag.getTagName())) {
		        	 log.debug("--> OHITETAAN TAGI");
		        	 return;
		         }
		         
		         // do whatever processing you want with the tag itself
		         // ...
		         // process recursively (nodes within nodes) via getChildren()
		         NodeList nl = tag.getChildren ();
		         if (null != nl)
		             for (NodeIterator i = nl.elements(); i.hasMoreNodes(); )
		                  parseNodes(i.nextNode());
		     }
	}
	
	public String getText() {
		return this.inputText.toString();
	}
/**
 * HTML-parseri, sisn tekstisolmu ja pointerin sijainti
 * vastaantulevat tekstisolmun sanat tsmytetn 
 * fdg:n sanoihin
 * @param nodeText html:n 'tekstisolmun' sislt
 * @param startAt tagin alkupaikan numero html-dokumentissa
 * @param endAt tagin loppupaikan numero html-dokumentissa
 */
	
	public void parseHTML(String nodeText, int startAt, int endAt) {
		
		//iteroidaan html-dokkarin tekstitagin sislt lpi 
	
		int relativeIterator = 0; // suhteellinen iteraattori html-tagin sisll
		int pointer = startAt; //pointteri tagin sisll
	
		this.htmlEntitySubtraction = 0;

		//while HTML-solmussa merkkej jljell
		while(pointer < endAt) {
			log.debug(pointer+" / "+endAt);
			
//			try {
				
				//jos HTML-tekstist lytyi vastine Pokan tokenille
				if(this.tokenFound) {
					
					log.debug("Otetaan seuraava fdg-node");
					if((textNode = this.pokaNodes.item(pokaNodeCounter))==null) {
						
//						log.error("VIRHE: fdg:n syte loppui (kesken?), lopetetaan!");
//						System.exit(0);
						log.info("Teksti lpikyty, palataan");
						return;
					}
					pokaTokenText = textNode.getTextContent();
					log.debug("PokaText: '"+pokaTokenText+"'");
					this.tokenFound = false;
					pokaNodeCounter++;
				}
				
				//tm pois: token voi (kyllkin harvoin) jatkua
				//html-tagin yli: 
				//teksti 'Kerberos-lehti'
				//html: '<p>...<i>Kerberos</i>-lehti...</p>
/*				if(nodeText.length() < pokaTokenText.length()) {					
					log.debug("htmltagi lyhempi, otetaan seuraava");
					return;
				}
*/				
				log.debug("pokaToken : '"+pokaTokenText+"'");
				log.debug("htmlToken: '"+nodeText+"'");

				//etsi HTML-tokenista pokan tokenia
				Token t = this.parseHTMLToken(pokaTokenText, nodeText, startAt, relativeIterator);
			

//				otetaan seuraava html ja seuraava fdg.
//              fdg tulee jljess tai on samassa kuin seuraavan html:n
//				alku, osa html:n tageista j merkkaamatta, mutta
//				seuraavasta tagista toimii jlleen oikein

				//tokenia ei lydy
				if(t.isNoMatch()) { 
					log.info("entity, otetaan seuraava tagi + fdg");
					this.tokenFound=true;
					return;
				}
				
				if(t.isNull()) {					
					log.debug("null, html-tagi lopussa, otetaan seuraava html");
					this.tokenFound = false; //oli false!
					pointer = t.getEnd(); 
					log.debug("pointer: "+pointer);
					//return;//otetaan seuraava html-tagi: robusti
					continue;
				} else {
					log.debug("token lytyi!");
					this.tokenFound = true;
				}
				log.debug("getEnd: "+t.getEnd()+", startAt: "+startAt);
				relativeIterator = t.getEnd()-startAt;
				log.debug("relativeIterator htmlTokenin tunnistuksen jlkeen: "+relativeIterator);
				
				String textToken = this.inputText.substring(t.getStart(), t.getEnd());
				log.info(". token:   poka: '"+pokaTokenText+"'");
				log.info(". token:   text: '"+textToken+"' ("+t.getStart()+", "+t.getEnd()+")");
				if(!pokaTokenText.equals(textToken)) {
			    	log.info("\t-->EI TSM");
			    	log.info("\nINPUTTEXT: '"+this.inputText.substring( t.getStart()-20 , t.getEnd()+20 )+"'");
			    }
//				this.tokens.add(t);
//			    pointer = t.getEnd();
//			    log.debug("POINTER: "+pointer+" (kierroksen loppu)");

				Element coordinate;
	        	Element start;
	        	Element end;
	    		Element info;

	    		//luo solmu C
	    		coordinate = (Element) pokaDocument.createElement("CR"); //COORDINATE
	    		start = (Element) pokaDocument.createElement("start");
	    		end =   (Element) pokaDocument.createElement("end");

	    		//TODO: String vai intti, tyypitetty xml-arvo?
	    		start.setTextContent(""+t.getStart());
	    		end.setTextContent(""+t.getEnd());
	    		
	    		coordinate.appendChild(start);
	    		coordinate.appendChild(end);
	    		
	    		//koordinaatin lisys tekstinoden isn
	    		//lapseksi (iltathdeksi)

	    		textNode.getParentNode().appendChild(coordinate);
				
				pointer = t.getEnd();
				log.debug("POINTER: "+pointer+" (kierroksen loppu)");
				
		}
		
		
	}
	
/**
 * Etsii fdg:n tokenille vastineen tekstist. 
 * @param xqres
 * 
 */
	
	public ArrayList<Token> parse(XQResultSequence xqres) {
		
			ArrayList <Token> results = new ArrayList<Token>();
			int pointer = 0;
			int counter = 1;
			log.debug("loop");
			while(pointer < this.textLen) {

				try {
					xqres.next();
					//fdg:n token
					String qString = xqres.getItem().getString();
					
					Token t = this.parseToken(qString, pointer);
					if(t==null) {
						break;
					}
					String textToken = this.inputText.substring(t.getStart(),t.getEnd());
					
					log.debug(counter+". token:    fdg: '"+qString+"'");
					log.debug(counter+". token:   text: '"+textToken+"' ("+t.getStart()+", "+t.getEnd()+")");
					if(!qString.equals(textToken)) {
				    	log.debug("\t-->EI TSM");
				    	log.debug("'"+this.inputText.substring( t.getStart()-20 , t.getEnd()+20 )+"'");
				    }
					results.add(t);
				    pointer = t.getEnd();
				    log.debug("POINTER: "+pointer+" (kierroksen loppu)");
				} catch(XQueryException e) {
					log.debug("---> ITEROINTIVIRHE!");
				}		
				counter++;
			}
			return results;
	}

	
	
	
	
	
	/**
	 * Etsii fdg:n tokenille vastineen tekstist. 
	 * (Tm metodi siis tekstimuotoisen dokumentin 
	 *  indeksointiin!)
	 * 
	 * @param pokadoc Poka-dokumenttimuodon mukainen pokadokkari
	 * @return dom-dokumentti, jossa indeksit, tuunattu pokadoc
	 *  
	 * --> ei lydkn fdg:t inputiksi, vaan suoraan
	 * pokadokkari.  Oletus on, ett FDG-muoto on jo ajettu,
	 * joten tll saadaan tsmys suoraan pokan rakenteeseen,
	 * hieman yksinkertaisempaa, tosin tehottomampaa?
	 * 
	 */
		public org.w3c.dom.Document parseWithDom(Document pokadoc)
										throws IndexingFailedException {
			
			int pointer = 0;
			int counter = 1;
		
			String expression = "/A/S/*/T"; // /analysis/sentence/<ALL>/text
			org.w3c.dom.NodeList nodes;
			XPath xpath = XPathFactory.newInstance().newXPath();
			int count = 0;
			try {
				nodes = (org.w3c.dom.NodeList) xpath.evaluate(expression, pokadoc, XPathConstants.NODESET);

				log.debug("loop, tokeneita: "+nodes.getLength());
				
				int nodeCounter = 0;
				
				while(pointer < this.textLen) {
					log.debug(pointer +" / "+this.textLen);
					
					//pokan token
					org.w3c.dom.Node item = nodes.item(nodeCounter);


					nodeCounter++;
					if(item == null) {
						log.debug("Item null, LOPETETAAN");
						break;
					}
					log.debug("Item: "+item.getTextContent());
					
					String qString = item.getTextContent();
					Token t = this.parseToken(qString, pointer);
				
					if(t==null) {
						log.debug("Token null, LOPETETAAN");
						break;
					}
					log.debug("Token: "+t.getStr());
					
					String textToken = this.inputText.substring(t.getStart(),t.getEnd());
						
					log.debug(counter+". token:   poka: '"+qString+"'");
					log.debug(counter+". token:   text: '"+textToken+"' ("+t.getStart()+", "+t.getEnd()+")");
					if(!qString.equals(textToken)) {
					    log.debug("\t-->EI TSM");
					   	throw new IndexingFailedException("'"+this.inputText.substring( t.getStart()-20 , t.getEnd()+20 )+"'");					   	
					}
					
					Element coordinate;
		        	Element start;
		        	Element end;
		    		Element info;

		    		//luo solmu C
		    		coordinate = (Element) pokadoc.createElement("CR"); //COORDINATE
		    		start = (Element) pokadoc.createElement("start");
		    		end =   (Element) pokadoc.createElement("end");

		    		//TODO: String vai intti, tyypitetty xml-arvo?
		    		start.setTextContent(""+t.getStart());
		    		end.setTextContent(""+t.getEnd());
		    		
		    		coordinate.appendChild(start);
		    		coordinate.appendChild(end);
		    		
		    		//itemin (tokenin) numeron lisys, tokenin lapseksi,
		    		//eli tekstisolmun isn lapseksi 
		    		item.getParentNode().appendChild(coordinate);
					
					pointer = t.getEnd();
					log.debug("POINTER: "+pointer+" (kierroksen loppu)");
					
					counter++;
				}
				
			} catch (XPathExpressionException e) {
		    	System.out.println(e);
			}
			return pokadoc;
		}	

	
	
	
	
	
	public Token parseToken(String fdgTokenText, int pointer) {
		log.debug("Parse token");
//		int begin = pointer;
//		boolean letters=false;
//		boolean upperCase = false;
		
		int fdgCount = 0;
		int fdgTokenLength = fdgTokenText.length();
		Character fdgChar = fdgTokenText.charAt(fdgCount);
		
		
		while(pointer<this.textLen) {
			Character textChar = this.inputText.charAt(pointer);
//			log.debug("Tekstikirjain: "+pointer+": '"+textChar+"'");
			
			int endPoint; //tokenin loppu jos lytyy
			
			if(fdgChar.equals(textChar)) {
				log.debug("tokenlen: " +fdgTokenLength);
				Token t = new Token(pointer, pointer+fdgTokenLength);
				t.setStr(inputText.substring(pointer, pointer+fdgTokenLength));
				return t; //DEBUGGAUSTA
//				return new Token(pointer, pointer+fdgTokenLength);
			} //TODO: html-parserin lisys.
			
			pointer++;
		}

		log.debug("LOPPU");
		return null;
	}
/**
 * Etsii vastineen yksittiselle pokaTokenille html-tagin sislt
 * @param pokaTokenText etsittv pokan tokenin tekstimuoto
 * @param htmlText html-tagin sislt, josta tokenia etsitn
 * @param pointer absoluutti osoitin html-dokumentin sisll 
 * @param relIter suhteellinen osoitin ksiteltvn tagin sisll
 * @return Token-olio, sislt sijainnin
 */	
	
	public Token parseHTMLToken(String pokaTokenText, String htmlText, int pointer, int relIter) {
		log.debug("Parse token");
//		int begin = pointer;
//		boolean letters=false;
//		boolean upperCase = false;
					
		int fdgTokenLength = pokaTokenText.length();
		Character fdgChar = pokaTokenText.charAt(0);
		
		
		int amp =  htmlText.indexOf('&', relIter);//relIter siit, miss ollaan
		log.info("relIter on:" +relIter);

		//jos token ehditn tunnistaa tekstist ennen html-entity,
		//tunnistetaan token normaalisti.
		if(pokaTokenText.length()+relIter <= amp) { 
			amp = -1;
			log.info("Etsitn tokenia ennen amp:ia"); // eli parsitaan normaalisti edelleen
		}
			
		if(amp != -1) { // html-tagi sislt entiteetin!
			log.info("Entiteettiin tsmttv pokaToken: '" + pokaTokenText+"'");
			
			//html-tagin sislt, josta entiteetit on dekoodattu nkyviin
			//Huom: dekoodattu versio on lyhyempi, jos se sislt entityj;
			//relIter osoittaa tagin sisll liian pitklle jos samasta
			//tagista on jo lydetty entiteettej
			StringBuffer decoded = new StringBuffer(Translate.decode(htmlText));
			log.info("DECODED: '"+decoded+"'");			
			
			//puolipisteen indeksi alkuperisess html-tagin sisllss
			int semi = htmlText.indexOf(';', relIter);
			log.info("puolipisteen sijainti on "+semi);
			
			
			if(semi == -1) {
				log.error("VIRHE: ei puolipistett, entity puutteellinen, palataan normaaliin tunnistukseen!");
				//System.exit(0);

			//TODO: pit korjata. et-merkin ja puolipisteen sidonta pareiksi, jotta
		    // varmasti tiedetn, ett puolipiste on juuri kyseisen entityn loppu
			// Jos puolipiste puuttuu, fdg ksittelee &-merkin tokeniksi, kun
		    // taas htmlparserin ja esim. firefoxin renderij hyvksyy puuttuvan
			// puolipisteen. HUOM! DIVVILL EI LUULTAVASTI SAA RIKKOA ENTITY!
			// --> FDG:n tokenin indeksiksi tytyy laittaa poikkeuksetta
			// & -merkki + loput, puolipiste ei vlttmtn
				
			// toimii suht ok, tll hetkell, kunhan ei lyd
			//sisn validoimatonta tavaraa, voisi tietysti
			// hylt kaiken, mik ei mene jtidyn lpi?
			} else {
				int start = -1;
				int decodedIterator = relIter-this.htmlEntitySubtraction; //iteraattori dekoodatulle html-tekstille,
											   //relIter on alkuperiselle, enkoodatulle
				
				while(relIter < htmlText.length()) { //tyspitk html-tagin sislt
					log.info("loop");
					log.info("relIter: "+relIter);
					log.info("alussa ampin indeksi (&): "+amp);
					log.info("alussa semin indeksi (;):"+semi);
					
//					fdgChar = fdgTokenText.charAt(decodedIterator);
					
					//MIKSI TM? FIXME 25.10.06					
					if(start != -1) { //jos tokenin alku on lytynyt (edellisell kierroksella)
						log.debug("alku lytyi");
/*						if(amp == -1) { // ei lis entityj (TODO: heikko testi
							log.debug("ei uusia entityj");
							return new Token(pointer+start, pointer+start+fdgTokenLength);
						} else {
							log.debug("oli lis entityj...");
							while(amp != -1 && semi != -1) {
								log.debug("loopataan");
								log.debug("amp: "+amp);
								log.debug("semi:"+semi);
								log.debug("fdglen: "+fdgTokenLength);
								if(fdgTokenLength > amp) {
									fdgTokenLength += semi - amp;
									amp =  htmlText.indexOf('&', semi+1);
									semi = htmlText.indexOf(';', semi+1);	
								} else {
									return new Token(pointer+start, pointer+start+fdgTokenLength);		
								}
								
							}
							//varalta en tm tl..
							return new Token(pointer+start, pointer+start+fdgTokenLength);
						}*/
						return new Token(pointer+start, pointer+start+fdgTokenLength);
					} else {			
						log.debug("--> ELSE");
						if(relIter == amp) { //entityn ksittely
							log.debug("iteraattori ampissa (&)");
							//entity vastaa alkukirjainta
							log.debug("decodedIterator: "+decodedIterator);
							
							//FIXME UUSI, (9.3.2007) heitt string array indexoutofboundsia...
							
							log.debug("decoded, charAt: "+decoded.charAt(decodedIterator));

							
							
							if(((Character)decoded.charAt(decodedIterator)).equals(fdgChar)) {
								log.debug("entity vastaa alkukirjainta ("+fdgChar+")");
								log.debug("Decoded: "+(Character)decoded.charAt(decodedIterator));
								start = relIter; //mritetn tokenin alku (1)
								fdgTokenLength += semi-amp; //kasvatetaan pituus oikeaksi (2)
								this.htmlEntitySubtraction += fdgTokenLength-1;
								log.debug("asetettiin htmlEntitySubtraction: " +this.htmlEntitySubtraction);
								
								relIter = semi; //ohitetaan entity (3)
								//kokeilu, pistetty plle
								return new Token(pointer+start, pointer+start+fdgTokenLength);
							} else {
								log.debug("entity EI vastaa alkukirjainta ("+fdgChar+")");
								log.debug("Decoded: "+(Character)decoded.charAt(decodedIterator));
								relIter = semi; // ohitetaan entity
							}
						} else { //ei-entity-kirjain
							log.debug("perus");
							if(((Character)htmlText.charAt(relIter)).equals(fdgChar)) {
								start = relIter;
								fdgTokenLength += semi-amp; //kasvatetaan pituus oikeaksi
//								return new Token(pointer+start, pointer+start+fdgTokenLength);
							}

						}
					}
					decodedIterator++;
					relIter++;
					amp =  htmlText.indexOf('&', semi+1);
					semi = htmlText.indexOf(';', semi+1);
					
				}

		
			}
		}
		
		//perustunnistus ilman &-merkki
		while(relIter< htmlText.length()) {
			Character textChar = htmlText.charAt(relIter);
			log.debug("kirjain: "+relIter+" / "+htmlText.length()+": '"+textChar+"'");
						
			if(fdgChar.equals(textChar)) {
				//jos html-token on alamittainen...
				if(fdgTokenLength > htmlText.length()-relIter) {
					String subPartOfToken = pokaTokenText.substring(0, htmlText.length()-relIter);
					//ja jos alkuosat tsmvt
					if(subPartOfToken.equals(htmlText.substring(relIter,htmlText.length()))) {
						//palauta jmindeksi tokenin sislt.
						Token t = new Token(pointer+relIter, pointer+relIter+htmlText.length());
						//token jatkuu html-entityn yli
						t.setTokenContinues();
						return t;
					}
					
				}
				log.debug("Palautetaan token, pituus: "+fdgTokenLength);
				
				return new Token(pointer+relIter , pointer+relIter+fdgTokenLength);
			} //TODO: html-parserin lisys.
			
			relIter++;
		}
	

		log.debug("--> EI TOKEN");
		Token t = new Token(pointer+relIter, pointer+relIter+fdgTokenLength);
		t.setNull();
		return t;
	}
	
	
//--------------------------
	private boolean isPeriod(Character ch) {
		if(ch.equals('.')) {
			return true;
		}
		return false;
	}
	
//	--------------------------

	
	/**
	 * Method that reads textfile from the file or url.
	 *
	 * @param fileUrl name of the file or URL
	 * @return lines of the text file as a String
	 */
	public StringBuffer readSource(String fileUrl) {
		String filename;
		filename = null;
		String temp = null;	
		URL myUrl;
		File myFile;
		try {
			filename = fileUrl;
		} catch(Exception e) {
			System.out.println("Invalid or missing file name!");
			//System.exit(0);
		}

		StringBuffer textIn = new StringBuffer();
			
		try {
			BufferedReader in;
			myFile = new File(filename);
			if(myFile.exists()) {
				in = new BufferedReader(
						new InputStreamReader(
							new FileInputStream(myFile), encoding));
						textIn = new StringBuffer((int)myFile.length());
			} else {
				myUrl = new URL(filename);
				in = new BufferedReader(
						new InputStreamReader(
							myUrl.openStream()));
			}

			while((temp = in.readLine()) != null) {
//				textIn.append(temp);
				textIn.append(temp + "\n");
			}
			System.out.println("...valmis!");
			in.close();
		} catch(ArrayIndexOutOfBoundsException e) {
				System.out.println("Array index out of bounds:: " + e);
		} catch(MalformedURLException e) {
			System.out.println("URL error: " + e);
			System.exit(0);
		} catch(Exception e) {
			System.out.println("Error: " + e);
			System.exit(0);
		}
			
		return textIn;
	}
	
	public static void main (String [] args) 
							throws IOException,
							IndexingFailedException {

		Logger hep = Logger.getLogger("fi.seco.semweb.service");
		hep.setLevel(Level.INFO);
		
		String docurl = "http://haku.kansallisbiografia.fi/FMPro?-db=kbnet.fp5&-format=kb%2fkbartikkeli.htm&-lay=www&-sortfield=lajittelukentt%e4&zz=zet5rr342uu5556v&-max=2147483647&-recid=32952&-find=";
//		String docurl = "/home/alm/termo/penkki/Ackte.html";
		
		//dokumentti fdg:lle
//		XFDGConnect xf = new XFDGConnect("/home/alm/aaparser/text/BrykPetri.txt", true);
//		XFDGConnect xf = new XFDGConnect("miten ky (1884AAAaaa", false);

//		XFDGConnect xf = new XFDGConnect(docurl, true);
//		QizxImpl qi = new QizxImpl(xf.getFDG());
//	    XQResultSequence xqres = qi.sequenceQuery("for $i in input()/analysis/sentence/token/text return data($i)");


		GeneralExtractor ge = new GeneralExtractor();
		ge.setDocument(docurl);
		TokenIndexer ttex = new TokenIndexer(docurl, true, ge.getXMLDoc());
	    
	    //dokumentti tokeniserille
//		TextTokenizer tt = new TextTokenizer(docurl, true);
//		tt.parse(xqres); //fdg:n ja dokkarin vastaavuuksien etsint
		
//		System.out.println(xf.sourceToString(xf.getFDG()));


		
	}
 	
}
