/**
 * Pitää kirjaa dokument(e)ista löytyneistä resursseista
 * --> Entinnen PokaInstance
 * TODO: lisätäänkö tänne osoittimet alkuperäistä tekstiä varten.
 */
 
package fi.seco.semweb.service.poka.util;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fi.seco.semweb.service.poka.name.NameIndex;

public class FoundResources {
	private static final Log log = LogFactory.getLog(FoundResources.class);
	
	
	//mäppäys uri --> resurssi
	private HashMap <String, PokaResource> uris;
	private int occurenceCounter;
	public FoundResources() {
		uris = new HashMap <String, PokaResource>();
		occurenceCounter = 0;
	
	}
	
	public void clear() {

		if (uris != null) {

			uris.clear();
			occurenceCounter = 0;
		}

	}
/**
 * Urin lisäys
 * @param uri uri
 * @param sent esiintymän lause //TURHA
 * @param token esiintymän token //TURHA
 * @param newOccurence jos true, kasvatetaan laskuria. False, kun lisätään monisanaisten
 *        käsitteiden muita osia (DocumentExtractor: checkLongTerm)
 */
	public void addURI(String uri, int sent, int token, boolean newOccurence) {
		if(uris.containsKey(uri)) {
			PokaResource i = uris.get(uri);
			if(newOccurence) {
				i.add();
			}
		} else { //urin ilmentymiä ei ole, luodaan uri
			PokaResource i = new PokaResource(uri);
			this.uris.put(uri, i);
		}
		if(newOccurence) {
			occurenceCounter++; //esiintymien lukumäärän laskuri
		}
	}
	
	public HashMap<String, PokaResource> getResources() {
		return this.uris;
	}
	
	/**
	 * Urin sekä käsitteen esiintymän indeksin lisäys
	 * @param uri uri
	 * @param sent esiintymän lause
	 * @param token esiintymän token
	 * @param newOccurence jos true, kasvatetaan laskuria. False, kun lisätään monisanaisten
	 *        käsitteiden muita osia (DocumentExtractor: checkLongTerm)
	 * @param start esiintymän alkuindeksi
	 * @param end esiintymän loppuindeksi
	 */
	public void addURI(String uri, int sent, int token, boolean newOccurence,
				       int start, int end) {
		if(uris.containsKey(uri)) {
			PokaResource i = uris.get(uri);
			if(newOccurence) {
				i.add();
			}
			i.addLocation(start, end);
		} else { //urin ilmentymiä ei ole, luodaan uri
			PokaResource i = new PokaResource(uri);
			i.addLocation(start, end);
			this.uris.put(uri, i);
		}
		if(newOccurence) {
			occurenceCounter++; //esiintymien lukumäärän laskuri
		}
	}
	
	
	public Set<String> getUris() {
		return this.uris.keySet();
	}

/**
 * Palauttaa järjestetyn merkkijonotaulukon löydetyistä
 * resursseista uri + count
 * @param desceding jos true, laskeva järjestys
 * @return
 */
	public ArrayList<String> getOrderedResults(boolean descending) {

		//purkka
		Iterator <String> i = uris.keySet().iterator(); 
		
		ArrayList <InstanceFrequency> al = new ArrayList <InstanceFrequency> ();
		
		String current;
		
		//palautetaan null jos ei vielä parsittu
		if(!i.hasNext()) {
			return null;
		}
		
		
		while(i.hasNext()) {
			current = i.next();
			
			al.add(new InstanceFrequency(current, uris.get(current).getCount()));		
		}	
		Collections.sort((al));
		
		String [] result = new String[uris.size()];
//		ArrayList <String> result = new ArrayList<String>(uris.size());
		
		int it;
		if(descending) {
			int size = uris.size()-1;
			for(it = 0; it <= size; it++) {
				result[it] =  al.get(size-it).getUri()+": "+ al.get(size-it).getFreq();
			}
		} else {
			it = 0;
			for(InstanceFrequency iF:al) {
				result[it] =  iF.getUri()+": "+iF.getFreq();
				it++;
			}
		}
		
		return new ArrayList<String>(Arrays.asList(result));
	}
	
	/**
	 * Palauttaa käsitteen esiintymien määrän
	 * Jos URIa ei löydy, palautetaan -1
	 * @param uri merkkijono pitkässä muodossa, esim: http://yso.fi/YSO#liput
	 * @return esiintymien lukumäärän, jos uria ei löydy, -1 palautetaan
	 */
	
	public int getHits(String uri) {
		if(uris.containsKey(uri)) {
			return this.uris.get(uri).getCount(); 
		}
		System.out.println("URIA EI OLE: "+uri);
		System.out.println(uris.keySet().toString());
		return -1;
	}
	
	public PokaResource getInstance(String uri) {
		return uris.get(uri);
	}
	
//vanha toString, joka palauttaa yhteenvedon sekä löydetyt käsitteet
//järjestettynä, järjestämiseen käytetään instanceFrequencyä.
	public String toString() {
		String result = "Käsitteitä yhteensä    : " + uris.size() + "\n"+
                        "Käsitteiden esiintymiä : " + occurenceCounter + "\n"+
                        "Esiintymiä per käsite  : " +((double)occurenceCounter/uris.size())+"\n";
	
		
		Iterator <String> i = uris.keySet().iterator(); 

		ArrayList <InstanceFrequency> al = new ArrayList <InstanceFrequency> ();
		
		String current;
		while(i.hasNext()) {
			current = i.next();
			al.add(new InstanceFrequency(current, uris.get(current).getCount()));		
		}	
		Collections.sort((al));
		for(InstanceFrequency iF:al) {
			result += iF.toString()+"\n";
		}
		return result;
	}

}