package fi.seco.semweb.service.poka.util;

/**
 * Poimii lemma-tagien sisällön fdg:n inputista. 
 * Käytetään Verboratorissa ja Termossa.
 * 
 * Syötteenä: fdg:n output
 * Ulos: ArrayList<String> siten, että tyhjä fdg-token
 * erottaa alkiot, ja String sisältää tokenien lemmat
 * välilyönneillä eroteltuina
 * 
 */

import java.io.IOException;
import java.util.ArrayList;

import org.xml.sax.helpers.*;
import org.xml.sax.*;


public class LemmaLister extends DefaultHandler {

	private final int INITCAPACITY = 20000;
	
	private ArrayList <String> result;
	private StringBuffer lemma;
	private XMLReader xr;
	private boolean lemmaStart;
	private boolean tokenStart;
	public LemmaLister() {
		super();
		try {
			xr = XMLReaderFactory.createXMLReader();
			xr.setContentHandler(this);
			xr.setErrorHandler(this);
		} catch(SAXException e) {
			System.err.println("LemmaParser / SAXEXP: "+e);
			//System.exit(0);
		}
	}
	
	public void parse(InputSource input) {
		this.result = new ArrayList <String>(INITCAPACITY);
		this.lemma = new StringBuffer();
		this.lemmaStart = false;
		this.tokenStart = false;
		
		try {
//			System.out.print("parsitaan...");
			xr.parse(input);
//			System.out.println("...ok!");
		} catch(SAXParseException e) {
			System.err.println("LemmaParser / SAXPARSE: "+e);
			//System.exit(0);
		} catch(SAXException e) {
			System.err.println("LemmaParser / SAXEXP: "+e);
			//System.exit(0);
		} catch(IOException e) {
			System.err.println("LemmaParser / IOEXP: "+ e);
			//System.exit(0);
		} 
	}
	
	public ArrayList <String> getResult() {
		return this.result;
	}
	
	 public void startElement (String uri, String name,
		      String qName, Attributes atts) {
		 
		 if(qName.equals("token")) {
			 this.tokenStart = true;
		 } else if(qName.equals("lemma")) {
			 this.lemmaStart = true;
		 }
	 }
	 
	 public void endElement (String uri, String name, String qName) {
		 if(qName.equals("lemma")) {
			 lemma.append(" "); //POISTETTU 24-7-06! MIKSI ylipäänsä tarvittu?
			 this.lemmaStart = false;	
			 this.tokenStart = false;
		 } else if(qName.equals("token")) {
			 if(tokenStart) { // tekstitagi tyhjä ('_')
				 this.result.add(this.lemma.toString().trim());
				 this.lemma = new StringBuffer();
				 this.tokenStart = false;		 
			 }
			 this.lemmaStart = false;
		 }
	 }
	 
	 public void characters (char ch[], int start, int length) {
		 if(lemmaStart) {
			 for (int i = start; i < start + length; i++) {
				
	    			switch (ch[i]) {
	    			
	    			case '&':
	    				lemma.append("&amp;");
	    				break;
	    			case '>':
	    				lemma.append("&gt;");
	    				break;
	    			case '<':
	    				lemma.append("&lt;");
	    				break;
	    			case '\'':
	    				lemma.append("'"); 
	    				break;
	    			case '\\':	
	    				lemma.append("\\\\");
	 				break;
	    			case '"':
	    				lemma.append("\"");
			        break;    
	    			case '\n':
	    				lemma.append("\\n");
	    				break;
	    				
	    			case '\r':
	    				lemma.append("\\r");
	    				break;

	    			case '\t':
	    				lemma.append("\\t");
	    				break;

	    			default:
	    				lemma.append(ch[i]);
	    				break;
	    			}
	    		}
						
		 } else {
			 return;
		 }
	
	 }
	 
	   /** This method is called when warnings occur */
	    public void warning(SAXParseException exception) {
	        System.err.println("WARNING: line " + exception.getLineNumber() + ": "+
	                           exception.getMessage());
	    }

	    /** This method is called when errors occur */
	    public void error(SAXParseException exception) {
	        System.err.println("ERROR: line " + exception.getLineNumber() + ": " +
	                           exception.getMessage());
	    }

	    /** This method is called when non-recoverable errors occur. */
	    public void fatalError(SAXParseException exception) throws SAXException {
	        System.err.println("FATAL: line " + exception.getLineNumber() + ": " +
	                           exception.getMessage());
	        throw(exception);
	    }
	 
}
