/**
 * Sanojen lemmatisoija, käytetään Nimien eristimessä
 * TODO: poistetaan joskus, LemmaParser tekee saman 
 * homman tyylikkäämmin.
 * 
 */

package fi.seco.semweb.service.poka.util;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.BufferedReader;

import java.net.*;

import net.axyana.qizxopen.xquery.*;
import net.axyana.qizxopen.dm.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.InputSource;

import fi.seco.semweb.service.poka.name.NameExtractorFrame;

public class Lemmatizer {

	private static final Log log = LogFactory.getLog(Lemmatizer.class);
	
/**
 * constant value for host name
 */
  private final String HOSTNAME = "nipo.seco.hut.fi";

  private String filename = null;
  //private InputSource in;
  private Socket fdgSocket;

  private PrintWriter out;
  
  private String xmlOutput;


  private BufferedReader bf;

  private XQueryDataSource ds;
  private XQueryBasicEngine be;
  private XQueryConnection c;


/**
 * Port for Connexor's FDG-parser
 */
  private final int FDGPORT = 10102;

	public Lemmatizer() {

		try {
			be = new XQueryBasicEngine();
			be.startup();
			ds = be.getDataSource("");
			c = ds.getConnection();
		} catch(Exception e) {
			System.err.println("konstrukori: " +e);
			//System.exit(0);
		}

	}

/*
		VANHA konstruktorin sisältö

 		//yhteys FDG:hen
		System.out.println("<FDG> Ota yhteys...");
		InputSource in = this.getLemmas(aksessorista saatu String)
		System.out.println("<FDG> ...ok!");

		//input Qizx-kyselyitä varten
		System.out.println("<FDG> Aseta Input...");
		this.setInput(in);
		System.out.println("<FDG> ...ok!");
*/

	private boolean close() {
		try {
			out.flush();
        	fdgSocket.shutdownOutput();
		} catch(Exception e) {
			System.out.println("Error: " + e);
  		}
		return true;
	}

	private InputSource getLemma(String terms) {
		
//-----------------------------------
//inputstream ottaa tavaraa vastaan serveriltä
//outputstream lähettää
		InputSource in = null;
		try {
            fdgSocket = new Socket(HOSTNAME, FDGPORT);
			//fdg-ulostulo ohjataan kirjoittajalle (PW)
            out = new PrintWriter(fdgSocket.getOutputStream(), true);
            //System.out.print("Vastaanotetaan fdg...");
			in = new InputSource(new InputStreamReader(fdgSocket.getInputStream()));
		} catch(UnknownHostException he) {
            System.err.println("Unknown host: "+ HOSTNAME);

        } catch(IOException ioe) {
            System.err.println("IO Exception occurred: "+ ioe);
        } catch(Exception e) {
            System.err.println("Exception occurred: "+e);
        }
	
		try {
			//System.out.println("kirjoitetaan termi outputstreamiin...");
			out.write(terms);
			out.flush();
        	fdgSocket.shutdownOutput();
		} catch(Exception e) {
			System.out.println("Error: " + e);
  		}
		return in;
	}

	private void setInput(InputSource is) {
		try {
			c.setInputDocument(is);
				
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch( XQueryException e ) {
			System.err.println("Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
	}
/*
	private void setInput(String input) {
		try {

			pro = 
			new XQueryProcessor("/home/alm/termo",
 									"/home/alm/termo");
			

							
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch( XQueryException e ) {
			System.err.println("Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
	}
*/
/**
 * Homma etenee seuraavasti:
 * 1. iteroi solmuja
 * 2. jos 
 *         -käsite moniosainen JA
 *         -1. osa monikko
 *         --> palauta kaikki perusmuotoon
 *
 * 3. jos
 *         -viimeinen käsite -na tai imperfekti (TAI MUUTA)
 *          älä lemmaa!
 *
 * 4. muissa tapauksissa palauta vain viimeinen osuus perusmuotoon
 *
 */

	public String lemma(String terms) {

		//korvataan kaikki '_'-merkit terms-merkkijonosta:
        //fdg käyttäytyy hieman kummallisesti alaviivojen suhteen 
        //ja generoi jokaisesta alaviivasta oman tokenin 
        //joka sisältää tyhjän tekstitagin (<text />)
//		terms = terms.replaceAll("_"," ");


		InputSource resultSet = this.getLemma(terms);
		this.setInput(resultSet);
	
//		String result = this.query("for $i in input()/analysis/sentence/token return <sana> {data($i/text)} /  {data($i/lemma)} / M: {data($i/tags[1]/morpho)} / S: {data($i/tags[1]/syntax)} </sana>");
		String result = this.query("for $i in input()/analysis/sentence/token/lemma return data($i)");
//		String result = this.query("input()/analysis/sentence");
		return result;
	}

/**
 * Tarkistaa, onko <sukunimi>, <etunimi> -muodosta
 * muotoon <etunimi> <sukunimi> lemmatisoituna
 * sellainen, että etunimellä on etumääre (A)-syntaktisena
 * määreenä
 * @param terms
 * @return true, jos syntax on A>
 */
	public boolean preExist(String terms) {
		InputSource resultSet = this.getLemma(terms);
		this.setInput(resultSet);
		log.debug("Lemmatizer / preExist: Aloitetaan kysely...");
		return this.query("data(/analysis/sentence[1]/token[1]/tags[1]/syntax)").trim().matches("(\\&amp;)?A&gt;");
	}
	
	public String query(String query) {
		
		StringWriter sw = new StringWriter();
		//System.out.print("Suoritetaan kyselyä...");
			 

		XQueryExpression q;
		XMLSerialOptions ops = new XMLSerialOptions();
		ops.setOption("omit-xml-declaration", "yes");
		ops.setOption("indent", "yes");
		String result = null;

		try {
			q = c.createExpression();
			result = q.executeQuery( query, ops );
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch (XQEvalException ee) {
    		System.err.println("XQEval: " + ee);
		} catch( XQueryException e ) {
			System.err.println("Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
		//System.out.println("...valmis!");
		return result;
	}



  /** Main method to run from the command line.    */
  public static void main (String[] args) 
							throws Exception {

	try {
		Lemmatizer app = new Lemmatizer();	
		String result = "";
		int i=0;
		while(i<args.length) {
			result += args[i] + " ";
			i++;
		}
		result = app.lemma(result);
		System.out.println(result);
//		System.out.println("NÄYTETÄÄN VAIN 1. MORPHO/SYNTAKSI\n"+result);
	} catch (Exception e) {
      System.out.println("Lemmaa seuraavasti: 'java Lemmaa <teksti>'");
	  System.exit(0);
    }
    
  }	
  
}

