/**
 * Qizx / XQuery-toteutuksen perusluokka, käytetään
 * nykyään enää HTML-muuntimessa
 */


package fi.seco.semweb.service.poka.util;
//package name;

/**
 * <pre>
 * Xquery implementation for querying Fdg-parsed
 * xml-files. 
 *
 * Constructor takes filename in, reads the file, 
 * passes it to Connexor Fdg-parser, takes the
 * xml-input and gives it open source
 * xQuery-implementation called Qizx.
 * (http://www.xfra.net/qizxopen/root.html)
 *
 * query-method can be used to get query-outputs
 * of the current instance (textfile)
 * </pre>
 * @author Olli Alm
 * @version 1.0
 */
import org.xml.sax.InputSource;
import net.axyana.qizxopen.xquery.*;
import net.axyana.qizxopen.dm.*;
import net.axyana.qizxopen.xquery.XQueryException;
//import net.axyana.qizxopen.xquery.XQuery;
import java.io.*;
import java.net.*;

public class PokaQuery {

	private String xmltext;
/**
 * Qizx XqQueryProcessor
 */
	


	private XQueryDataSource ds;
    private XQueryBasicEngine be;
	private XQueryConnection c;

//	private XQueryProcessor pro;
	PrintWriter out;
    BufferedReader in;

	private boolean initOk = false;

//---------------------------------------------
// CONSTRUCTOR
//---------------------------------------------

/**
 * Constructor for querying FDG XML-documents
 *
 * @param filename name of the file OR complete document as a String
 * @param isFileName if this parameter is true, then given filename is 
 *  handled as a filename, otherwise filename is handled as a complete String document
 */
	public PokaQuery(String filename, boolean isFileName) {
		
		try {
			be = new XQueryBasicEngine();
			be.startup();
			ds = be.getDataSource("");
			c = ds.getConnection();
		} catch(Exception e) {
			System.err.println("konstrukori: " +e);
			//System.exit(0);
		}

		if(isFileName) {
			this.xmltext = readSource(filename);
		} else { //if not, given String will be casted to InputSource-instance
			this.xmltext = filename;
		}	
		initOk = true;
		this.setInput();
	}


	public PokaQuery() {
		
		try {
			be = new XQueryBasicEngine();
			be.startup();
			ds = be.getDataSource("");
			c = ds.getConnection();
		} catch(Exception e) {
			System.err.println("konstrukori: " +e);
			//System.exit(0);
		}

	}

/**
 * Method that reads textfile from the file or url.
 *
 * @param fileUrl name of the file or URL
 * @return lines of the text file as a String
 */
	public String readSource(String fileUrl) {
		//String filename = null;
		String temp = null, textIn = "";
		URL myUrl;
		File myFile;

//		System.out.print("Luetaan tiedostoa...");
		try {
			BufferedReader in;
			myFile = new File(fileUrl);
			if(myFile.exists()) {
				in = new BufferedReader(
						new InputStreamReader(
					  		new FileInputStream(myFile)));
			} else {
				myUrl = new URL(fileUrl);
				in = new BufferedReader(
						new InputStreamReader(
							myUrl.openStream()));
			}
			while((temp = in.readLine()) != null) {
				  textIn += temp;
				  //System.out.println("loopissa, rivi:"+temp);
			}
//			System.out.println("...valmis!");
			in.close();
		} catch(ArrayIndexOutOfBoundsException e) {
				System.out.println("Array index out of bounds:: " + e);
		} catch(MalformedURLException e) {
				System.out.println("URL error: " + e);
				//System.exit(0);
		} catch(Exception e) {
			System.out.println("Error: " + e);
			//System.exit(0);
		}
		if(!initOk) {
			this.xmltext = textIn;
			initOk = true;
		}	
		return textIn;
	}

/**
 * This method sets the default input (the document)
 * for the current query. This method is used by 
 * the constructor
 *
 */	
	public void setInput() {
		if(!initOk) {
			System.out.println("Document not read!");
			//System.exit(0);	
		}
		try {
			c.setInputDocument(new InputSource
								(new StringReader(xmltext)));
					
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch( XQueryException e ) {
			System.err.println("(setInput)Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
	}



	/**
	 *	This method executes the Qizx-xQuery query;
     *   
     *  @param query Xquery as a String
     *  @return Xquery results
	 */
	public String query(String query) {
		
		XQueryExpression q;
		XMLSerialOptions ops = new XMLSerialOptions();
		ops.setOption("omit-xml-declaration", "yes");
		ops.setOption("indent", "yes");
		String result = null;		

		try {
//			q = pro.compileQuery(query, "<source>", log);
			q = c.createExpression();
			

			result = q.executeQuery( query, ops );
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch (XQEvalException ee) {
			System.err.println(ee);
		} catch( XQueryException e ) {
			System.err.println("Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
		//System.out.println("...valmis!");
		//result = result.substring(0, result.length()-1);
		return result;
	}

   /**
	*	Executes the Qizx-xQuery query littlebit
    *   different than query-method. This method
    *   returns results of the query as a Qizx
    *   Value-object which can be iterated 
    *   several times
	*
    *  @param query Xquery as a String
    *  @return Xquery results as a Value, CAN BE ITERATED
    */
	public XQResultSequence valueQuery(String query) {
		
//		XQValue v = null;
		XQResultSequence v = null;
//		XQuery q;
		XQueryExpression q;
		
		try {
//			q = pro.compileQuery(query, "<source>", log);
			q = c.createExpression();
			//XMLSerializer serial = new XMLSerializer();
			//serial.setOutput( sw );

			
            //serial.setOption("method", "XML");
			//serial.setOption("indent", "yes");
			//LISÄTTY 15.9.05 --> EI XML-OTSAKETTA VASTAUKSIIN!
			//serial.setOption("omit-xml-declaration","yes");

			v = q.executeQuery(query);
		} catch( IndexOutOfBoundsException e ) {
			System.err.println("Args missing: " + e);
		} catch (XQEvalException ee) {
    		System.err.println(ee);
		} catch( XQueryException e ) {
			System.err.println("Query compile error: " + e);
		} catch ( Exception e) {
			System.err.println("Something happened: " + e);
		} 
		//System.out.println("...valmis!");
		return v;
	}

//---------------------------------------------
// MAIN
//---------------------------------------------
	
	public static void main(String [] args)
						throws Exception {
		
		if(args.length != 1) {
			System.out.println("Usage: XqueryTest <filename>");
			System.exit(0);
		}

		PokaQuery my = new PokaQuery(args[0], true);
		
		String myFirstQuery = "/ANALYSIS";
		System.out.println(my.query(myFirstQuery));

	}
}
