/**
 * Jena-kyselyiden hieman sotkuinen toteutus
 * Luokan ilmentymiä ovat Termistö, YSO ja Verbiontologia,
 * luokan metodit ovat kaikille yhteiset, vaikkeivat toimi
 * ristiin... 
 */

package fi.seco.semweb.service.poka.util;

import java.io.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;


import com.hp.hpl.jena.rdf.model.*;

import com.hp.hpl.jena.rdql.*;
import com.hp.hpl.jena.util.FileManager;

import fi.seco.semweb.service.poka.Termo;

//import org.apache.log4j.*;

public abstract class TermoQuery {

//lemmas generated more complicated way (see: FDGConnection)
	public static final String COMPLEMMAS = "/home/alm/termo/compLemmas.rdf";

//lemmat ilman kielimäärettä
	public static final String LEMMAS = "/home/alm/termo/lemmaNoLang.rdf";
//uuden YSOn mukaiset lemmat
//	public static final String NEULEMMAS = "/home/alm/termo/Terms-all.owl";

	public static final String DLEMMAS = "/home/alm/termo/D-Terms.owl";
	
	public static final String ONTOLOGY = "/home/alm/ontocvs/ontologies/YSO/YSO-Fyysinen_objekti.owl";

	public static final String RDFNS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	
//	public static final String NEUONTOLOGY = "/home/alm/ontocvs/ontologies/YSO/YSO-All.owl";	

//	public static final String VERBS = "/home/alm/termo/Verbs.rdf";
	
	public static final String LANG = "fi";		

		
	private Property rdftype;
	protected Model m;

/**
 * Konstruktori, kutsuu readFileä, lukee tiedoston,
 * luo siitä mallin
 */ 

	public TermoQuery(String filename, String coding) {

		try {
			InputStream in = this.readFile(filename);
			m = ModelFactory.createDefaultModel();
			m.read(new InputStreamReader(in, coding), "");
			rdftype = m.getProperty(RDFNS, "type");
		} catch(IOException e) {
			System.out.println("Error reading file '"+
  				filename+"'.");
			//System.exit(0);
		}

	}
/**
 * --> konstruktori jolle syötetään suoraan malli
 * 
 *  Mahdollistaa erityyppisten kyselyrajapintojen
 *  toteutuksen yhdelle  mallille, esim
 *  mallille, joka sisältää paikat, yson ja verbit
 *  voidaan kiinnittää kolme eri kyselyrajapintaa
 *  (YsoQuery, VerbQuery, PlaceQuery) 
 * 
 */
	
	public TermoQuery(Model model) {
		this.m = model;
	}

/**
 * Jos TermoQuerylle luetaan malli useista tiedostoista, 
 * tällä metodilla voidaan lisätä alkuperäiseen malliin
 * tavaraa muista tiedostoista (alkujaan: YSO-fyysinen + YSO-UO)
 *
 */
	public boolean addOntology(String filename) {
		Model mAttach;
		try {
			InputStream in = this.readFile(filename);
			mAttach = ModelFactory.createDefaultModel();
			mAttach.read(new InputStreamReader(in, "ISO-8859-1"), "");
			this.m.add(mAttach);				

			return true;
		} catch(IOException e) {
			System.out.println("Error reading file '"+
  				filename+"'.");
			//System.exit(0);

			return false;
		}
	}

/**
 * Tiedoston lukija
 */

	private InputStream readFile(String filename)
						throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
        if (in == null) {
    		throw new IllegalArgumentException(
            	"File '" + filename + "' not found");
		}
		return in;
	}

/**
 * Palauttaa mallin
 */
	public Model getModel() {
		return this.m;
	}
	
/**
 * Palauttaa urin tyypin Stringinä
 * EI KÄYTÖSSÄ: otetaan käyttöön jos instanssidatalle
 * tehdään malli eikä instanssien tyyppejä tallenneta
 * terminologiaan
 * 
 * @param uri resurssin uri
 * @return null jos ongelmia, muuten tyyppi stringinä
 */	
	public String getType(String uri) {
		Resource r = m.getResource(uri);
		
		Statement st = r.getProperty(rdftype);
		if(st==null) {
//			System.out.println("--> null");
			return null;
		}
//TODO: tarkista toimiiko oikein
		return st.getString();
	}

/**
 * palauttaa StatementIteratorin, iteroi kaikki statementit
 */
	public StmtIterator getAll() {
		return m.listStatements();
	}	

	public abstract String getUpper(String uri);
	
	public static void main (String [] args) 
						throws IOException {

		
		System.out.println("Luetaan paikat...");
		PlaceQuery paikat = new PlaceQuery("/home/alm/ontocvs/ontologies/YSO/Paikat.owl", "UTF-8");
		System.out.println("...ok!");
		System.out.println("Ruukki on: "+paikat.getUpper("http://www.yso.fi/kulttuuriSampo/paikat#ruukki"));
		
		System.out.print("Luetaan YSO...");
		YsoQuery yso = new YsoQuery(ONTOLOGY, "ISO-8859-1");
		System.out.println("...ok!");


//		yso.addOntology("/home/alm/ontocvs/ontologies/YSO/YSO-UO.owl");

// UPPER-ONTOLOGY-LUOKILLA EI OLE LABELIA, 
// allaolevan esimerkin mukaan luokan aika yläluokalla time ei esim. ole!
		System.out.println("teollisuus is-a:" + yso.getUpper("http://yso.fi/YSO#teollisuus"));


/*
		StmtIterator si = terms.getAll();
		while(si.hasNext()) {
			System.out.println(si.next());
		}
*/

	}

}


