package fi.seco.semweb.service.poka.util;


/**
 * Toteuttaa termitiedostolle trie-indeksoinnin
 */


//TODO: hajota erilleen YSOn, termien ja verbien käsittely:
//yläluokka + 3 alaluokkaa
// OntoQuery --- YSOQuery / TermoQuery / VerboQuery
//

import java.io.*;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import com.hp.hpl.jena.rdf.model.*;

import com.hp.hpl.jena.query.*;
import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.QueryException;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.rdql.*;
import com.hp.hpl.jena.util.FileManager;

import fi.seco.semweb.service.poka.Configuration;
import fi.seco.semweb.service.poka.Termo;
import fi.seco.semweb.service.poka.frame.FrameExtractorCook;

//import org.apache.log4j.*;

public class TrieIndex implements TrieInterface {
	private static final Log log = LogFactory.getLog(TrieIndex.class);
	
	protected Lemmatizer lemmatizer;
	protected TernarySearchTrie trieIndex;
	protected Model terms;
	protected Property termlabel;
	protected String encoding;
	protected String termFile;
	protected String termNameSpace;
	
/**
 * Konstruktori, joka lukee suoraan mallista.
 *
 */
	public TrieIndex(Model m, String termNameSpace) {
		
		this.terms = m;
		this.termlabel = terms.getProperty(termNameSpace, "termlabel");

		log.debug("Luetaan termit puuhun");
		this.trieIndex = this.getTerms(terms, termNameSpace);
		log.debug("ok");
	}
	
/**
 * Konstruktori, kutsuu readFileä, lukee termitiedoston,
 * luo siitä mallin
 */ 

	public TrieIndex(String termFile, String encoding,String termNameSpace) {
		log.debug("TrieIndex alkaa");
		
		this.encoding = encoding;
		this.termFile = termFile;
		this.termNameSpace = termNameSpace;
		
		log.debug("Luodaan lemmatizer");
		this.lemmatizer = new Lemmatizer();
		log.debug("Lemmatizer ok");
		
		try {
			log.debug("luetaan termitiedosto");
			InputStream in = this.readFile(termFile);
			terms = ModelFactory.createDefaultModel();
			terms.read(new InputStreamReader(in, encoding), "");
			log.debug("ok");
						
			this.termlabel = terms.getProperty(termNameSpace, "termlabel");
			
			log.debug("Luetaan termit puuhun");
			this.trieIndex = this.getTerms(terms, termNameSpace);
			log.debug("ok");
			
		} catch(IOException e) {
			log.error("Error in creating trieindex: '"+
  				termFile+"'.");
			System.exit(0);
		} catch(Exception e) {
			log.error("Muu virhe: "+e);
		}
	
	}

/**
 * Tiedoston lukija
 * --> createSearchTrie käyttää tätä
 */

	private InputStream readFile(String filename)
						throws IOException {

		InputStream in = null;
		in = FileManager.get().openNoMap( filename );
        if (in == null) {
    		throw new IllegalArgumentException(
            	"File '" + filename + "' not found");
		}
		return in;
	}
/**
 * Tiedoston kirjoittaja
 */	
	public void writeFile(String filename) 
					throws IOException {
		
		if(filename == null) { //jos null, käytä oletusta
			filename = this.termFile;
		}
		File f = new File(filename);
		FileOutputStream fos = new FileOutputStream(f);
		terms.write(fos);
		fos.flush();
		fos.close();
	}

/**
 * Indeksoi annetun jena-mallin prefiksipuuhun,
 * createSearchTrie käyttää tätä
 * @param terms malli
 * @param termNameSpace termistön nimiavaruus
 * @return puu
 */
	private TernarySearchTrie getTerms(Model terms, String termNameSpace) {

		TernarySearchTrie result = new TernarySearchTrie();
		
		String qString = 
        "SELECT ?label, ?concept " +
		"WHERE (?concept, <oma:termlabel>, ?label) " +
        "USING oma FOR <"+termNameSpace+">";

		log.debug("Kysely luodaan");
		Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
//			q = new Query(qString);
		} catch(com.hp.hpl.jena.query.QueryException e) {
			log.error("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}

//		q.setSource(terms);	
//		QueryExecution qe = new QueryEngine(q);
		
		QueryExecution qe = QueryExecutionFactory.create(q, terms);
		
		try {
			ResultSet results = qe.execSelect();
		
			//QueryResults results = qe.exec();

			//initcapacity 25000

			HashSet <String> uriBag;
			String label;
		
			log.debug("iterointi alkaa");
			for( ; results.hasNext() ;) {

				QuerySolution res = results.nextSolution() ;			
//VANHA:		ResultBinding res = (ResultBinding)i.next();
	
				label = res.get("label").toString();
				
				log.debug("Käsite: "+res.get("concept").toString());
				if(label.length()==0) {
					log.debug("Label null, skipataan");
					continue;
				}
				uriBag = (HashSet <String>)result.get(label);
				if(uriBag == null) {
					uriBag = new HashSet<String>(5);
				}
				uriBag.add(res.get("concept").toString());
				result.put(label, uriBag);
			}
		} finally { qe.close(); }

		return result;
	}
	
	public boolean createTerm(String uri, String name) {
		String lemma = this.lemmatizer.lemma(name);
		Resource r = terms.createResource(Configuration.DEFAULTFRAMENAMESPACE+uri);
		r.addProperty(termlabel, lemma);
		this.put(lemma, uri);
		return true;		
	}
	
/**
 * Palauttaa tiedon, sisältääkö puu annetun termin avaimen alkuosana (prefix) 
 * @param term hakusana
 * @return true, jos sisältää, false, jos ei
 */
	public boolean containsPrefix(String term) {
		return !((List <String>)this.trieIndex.matchPrefix(term)).isEmpty();
	}
	
	public boolean contains(String term) {
		return this.trieIndex.get(term) != null;
	}
	
	/**
	 * Palauttaa d-eristimelle listana kaikki urit.
	 * @return
	 */
	public HashSet<String> getTermURIs() {

		HashSet <String> results = new HashSet<String>();
		ResIterator r  = this.terms.listSubjects();
		while(r.hasNext()) {
			results.add(r.next().toString());
		}
		return results;
	}
	
	/**
	 * alunperin D-eristimen käyttöön tehty palikka (25.2.06)
	 * @param uri käsitteen uri
	 * @return Termien käsitteen termLabel
	 * 
	 * palauttaa vain yhden labelin, tarvitaanko?
	 * 
	 */

	public String getLabel(String uri) {
		
		String qString = 
        "SELECT ?label " +
		"WHERE (<"+uri+">, <oma:termlabel>, ?label) " +
        "USING oma FOR <http://www.seco.hut.fi/ns/2005/10/alm#>";
		
		Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
			//q = new Query(qString);
		} catch(QueryException e) {
			System.out.println("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}

		//vanha (jena 2.3)
//		q.setSource(terms);	
//		QueryExecution qe = new QueryEngine(q);

		//uusi
		QueryExecution qe = QueryExecutionFactory.create(q, terms);
		
		//QueryResults results = qe.exec();
		ResultSet results = qe.execSelect();
		
		String label = null;
		for( ; results.hasNext() ;) {
			QuerySolution res = results.nextSolution() ;	
			if(!((Literal)res.get("label")).getLanguage().equals("se")) {
				label = ((Literal)res.get("label")).getString();
			}
		}
			
		return label;
	}
	
/**
  * Hakee puusta annetulla merkkijonolla alkavat merkkijonot
  * @param term hakusana
  * @return palauttaa täsmäävät avaimet
  */
	
	public List<String> matchByPrefix(String term) {
		List <String> l = (List <String>)this.trieIndex.matchPrefix(term);
		return l;
	}
 /**
  * Epätäsmällinen haku puusta. Toimii huonosti, ei kannata käyttää.
  * 
  * @param term hakutermi
  * @param hits jotakin, pitää tarkistaa
  * @return osumat listana
  */
	
	public List<String> matchAlmost(String term, int hits) {
		List <String> l = (List <String>)this.trieIndex.matchAlmost(term, hits);
		return l;
	}
 /**
  * Palauttaa labelia vastaavan urien joukon 
  * 
  * @param label
  * @return
  */
	
	public HashSet<String> getUris(String label) {
		return (HashSet <String>)this.trieIndex.get(label);
	}
/**
 * Käsitteen lisäys triehen, käytetään
 * ainoastaan käsitteen tai labelin lisäysmetodista.
 * 
 * @param key merkkijonoavain
 * @param objUri lisättävä uri
 */
	
	protected void put(String key, String objUri) {
		HashSet<String> temp = (HashSet <String>)this.trieIndex.get(key);
		if(temp  == null) {
			temp = new HashSet<String>();
		}
		temp.add(objUri);
		this.trieIndex.put(key, temp);
	}
	
	public int size() {
		return this.trieIndex.numDataNodes();
	}
	
	protected void remove(String key, String objUri) {
		HashSet<String> temp = (HashSet <String>)this.trieIndex.get(key);
		if(temp == null) {
			return;
		}
		temp.remove(objUri);
		if(temp.isEmpty()) { //jos hashSet jää tyhjäksi, poistetaan label indeksistä
			this.trieIndex.remove(key);
		}		
	}
	
	/**
	 * Palauttaa termiä vastaavat URIt Stringinä ilman
	 * nimiavaruutta
	 * 
	 */
	public String getConceptString(String term) {
		Iterator it = this.getUris(term).iterator();
		String result = "";
		String temp;
		while(it.hasNext()) {
			temp = (String)it.next();
			result += temp.substring(temp.indexOf('#')+1)+" ";
		}
		return result.trim();
	}
	
//	-----------------------------
//	---> TERMINOLOGIAN KÄSITTELY (poista / lisää)
//	-----------------------------
	/**
	 * 
	 * 
	 */

	public boolean addLabel(String uri, String label) {

		Property p = terms.getProperty(Termo.DEFAULTNAMESPACE, Termo.TERMLABEL);
		Resource r = terms.getResource(uri);

		if(terms.containsResource(r)) {
			r.addProperty(p, terms.createLiteral(label));
			this.put(label, uri); //lisäys triehen
			return true;
		}
		System.out.println("addLabel: resource doesn't exist!");
		return false;
	}
	
	/**
	 * removeLabel, labelin poisto
	 * TODO: toimiiko, näyttää kummalliselta... (tehty joskus muinoin)
	 * 
	 */ 

	public boolean removeLabel(String uri, String label) {

		Resource r = terms.getResource(uri);
		Property p = terms.getProperty(Termo.DEFAULTNAMESPACE, Termo.TERMLABEL);
		Literal l = terms.createLiteral(label);
		Statement s = terms.createStatement(r,p,l);
		
		//jos statement on olemassa
		if(terms.contains(s)) {
			terms.remove(s); //poisto termistömallista
			this.remove(label, uri); //poisto hakupuusta
			return true;
		}
		System.out.println("removeLabel: statement doesn't exist!");
		return false;
	}


	
//-----------------------------
//-----------------------------

	public static void main (String [] args) 
						throws IOException {

		System.out.print("Luodaan indeksi...");
		TrieIndex terms = new TrieIndex(Configuration.getNeulemmas(), "ISO-8859-1", Termo.DEFAULTNAMESPACE);
		System.out.println("...ok!");
	
		List <String> l = terms.matchByPrefix("sota");
		HashSet <String> hs;

		for(String s:l) {
			System.out.println(s.toString()+":");
			hs = terms.getUris(s);
			if(hs==null) {
				continue;
			}
			for(String ss: hs) {
				System.out.println("\t"+ss);
			}
		}
		
	

		System.out.print("Luodaan paikkaindeksi...");
		TrieIndex places = new TrieIndex(Configuration.getPlaceLemmas(), "ISO-8859-1", Termo.DEFAULTNAMESPACE);
		System.out.println("...ok!");
	
		l = places.matchByPrefix("sii");
		
		for(String s:l) {
			System.out.println(s.toString()+":");
			hs = places.getUris(s);
			if(hs==null) {
				continue;
			}
			for(String ss: hs) {
				System.out.println("\t"+ss);
			}
		}

		places.createTerm("yykaakokoeqp", "kuulakärkikynä");
		System.out.println(places.matchByPrefix("kuu"));
//		places.writeFile(null);


	}
	//ei toteuteta luokkatyyppisille muuttujille
	//tarvitaan opasExtractoriin, jotta saadaan
	//käytettyä oikeaa indeksiä (TrieIndex ja TrieInstanceIndex
	//implementoivat TrieInterfacen
	public String getType(String uri) {
		// TODO Auto-generated method stub
		return null;
	}

}

