package fi.seco.semweb.service.poka.util;
/**
 * YSOn kyselyrajapinta
 */

import java.util.ArrayList;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QueryFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;

public class YsoQuery extends TermoQuery
							implements ContextInterface {

	private static final Log log = LogFactory.getLog(YsoQuery.class);

	private final String YSONS = "http://yso.fi/YSO#";
	private final String YSOLABEL = "prefLabel";
	private Property label;

	public YsoQuery(String filename, String encoding) {
		super(filename, encoding);
		this.label = this.m.getProperty(YSONS,YSOLABEL);
	}

	public YsoQuery(Model model) {
		super(model);
		this.label = this.m.getProperty(YSONS,YSOLABEL);
	}

	@Override
	public String getUpper(String uri) {
		String qString =
        "SELECT ?y " +
        "WHERE (<"+uri+">,<rdfs:subClassOf>,?x) " +
        "      (?x,<yso:prefLabel>,?y) " +
        "USING rdfs FOR <http://www.w3.org/2000/01/rdf-schema#> " +
        "       yso FOR <http://yso.fi/YSO#>";

		com.hp.hpl.jena.query.Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
		} catch(com.hp.hpl.jena.query.QueryException e) {
			log.error("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}

		com.hp.hpl.jena.query.QueryExecution qe = QueryExecutionFactory.create(q, m);
		String result = "";

		try {

			ResultSet results = qe.execSelect();

			for( ; results.hasNext() ; ) {
				QuerySolution res = results.nextSolution();
				if(!((Literal)res.get("y")).getLanguage().equals("se")) result += ((Literal)res.get("y")).getString() +" | ";
			}

		} finally { qe.close(); }

		if(result.length() >=3) result = result.substring(0,result.length()-3);
		return result;
	}

	/**
	 * palauta YSOsta yläluokat listana
	 *
	 */
	public ArrayList<String> getContextInformation(String uri) {
		String qString =
        "SELECT ?y " +
        "WHERE (<"+uri+">,<rdfs:subClassOf>,?x) " +
        "      (?x,<yso:prefLabel>,?y) " +
        "USING rdfs FOR <http://www.w3.org/2000/01/rdf-schema#> " +
        "       yso FOR <http://yso.fi/YSO#>";

		com.hp.hpl.jena.query.Query q = null;
		try {
			q = QueryFactory.create(qString, com.hp.hpl.jena.query.Syntax.syntaxRDQL);
		} catch(com.hp.hpl.jena.query.QueryException e) {
			log.error("Virheellinen kyselymerkkijonon sisältö:\n"+qString+"\nVirhe:"+e);
		}

		com.hp.hpl.jena.query.QueryExecution qe = QueryExecutionFactory.create(q, m);

		ArrayList <String> result = new ArrayList<String>();

		try {

			ResultSet results = qe.execSelect();

			for( ; results.hasNext() ; ) {
				QuerySolution res = results.nextSolution();
				if(!((Literal)res.get("y")).getLanguage().equals("se")) result.add(((Literal)res.get("y")).getString());
			}

		} finally { qe.close(); }

		return result;
	}

	public String getLabel(String uri) {

		Resource r = this.m.getResource(uri);

		if(r.hasProperty(label)) {
			StmtIterator st = r.listProperties(label);
			while(st.hasNext()) {
				Statement s = (Statement) st.next();
				if(!s.getLanguage().equals("se")) return s.getLiteral().getLexicalForm();
			}
		}
		log.warn("Urille (tai ominaisuudelle) ei löydy vastinetta ontologiasta");
		return ""; //jottei kaatuile...
	}

}
