package fi.seco.semweb.service.resourcelinking;

import java.util.HashMap;
import java.util.Map;

import com.hp.hpl.jena.rdf.model.Resource;

import fi.seco.semweb.model.WeightedResourceObject;

/**
 * A wrapper for a resource linker that eliminates duplicates in the result set.
 * When the same Resource appears several times in the result set (wrapped in different
 * WeightedResourceObjects), only the one with the maximum weight value is returned.
 * @author oisuomin
 *
 */

public class MaxWeightResourceLinker implements IResourceLinker {

	private IResourceLinker linker;

	/**
	 * Create a new MaxWeightResourceLinker wrapping another resource linker
	 * @param linker the resource linker to wrap
	 */
	
	public MaxWeightResourceLinker(IResourceLinker linker) {
		this.linker = linker;
	}

	public Iterable<WeightedResourceObject> getLinkedResources(Resource resource) {
		Map<Resource,WeightedResourceObject> result = new HashMap<Resource,WeightedResourceObject>();

		for (WeightedResourceObject cat : linker.getLinkedResources(resource)) {
			if (result.containsKey(cat.getResource())) {
				if (cat.getWeight() > result.get(cat.getResource()).getWeight()) {
					result.put(cat.getResource(), cat);
				}
			} else {
				result.put(cat.getResource(), cat);
			}
		}
		
		return result.values();
	}

}
