package fi.seco.semweb.service.resourcelinking;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import com.hp.hpl.jena.rdf.model.Resource;

import fi.seco.semweb.model.WeightedResourceObject;

/**
 * Merges the results of several resource linkers, producing a union of the results.
 * Duplicates are not eliminated.
 * @author oisuomin
 */

public class UnionResourceLinker implements IResourceLinker {

	private List<IResourceLinker> linkers;

	/**
	 * Create a new UnionResourceLinker, specifying a list of subordinate resource linkers.
	 * @param linkers list of subordinate resource linkers
	 */

	public UnionResourceLinker(List<IResourceLinker> linkers) {
		this.linkers = linkers;
	}
	
	/**
	 * Convenience method for creating a UnionResourceLinker for two resource linkers.
	 * @param first first subordinate resource linker
	 * @param second second subordinate resource linker
	 */
	public UnionResourceLinker(IResourceLinker first, IResourceLinker second) {
		linkers = new ArrayList<IResourceLinker>(2);
		linkers.add(first);
		linkers.add(second);
	}

	/**
	 * Convenience method for creating a UnionResourceLinker for three resource linkers.
	 * @param first first subordinate resource linker
	 * @param second second subordinate resource linker
	 * @param third third subordinate resource linker
	 */
	public UnionResourceLinker(IResourceLinker first, IResourceLinker second, IResourceLinker third) {
		linkers = new ArrayList<IResourceLinker>(3);
		linkers.add(first);
		linkers.add(second);
		linkers.add(third);
	}
	
	/**
	 * Convenience method for creating a UnionResourceLinker for four resource linkers.
	 * @param first first subordinate resource linker
	 * @param second second subordinate resource linker
	 * @param third third subordinate resource linker
	 * @param fourth fourth subordinate resource linker
	 */
	public UnionResourceLinker(IResourceLinker first, IResourceLinker second, IResourceLinker third, IResourceLinker fourth) {
		linkers = new ArrayList<IResourceLinker>(4);
		linkers.add(first);
		linkers.add(second);
		linkers.add(third);
		linkers.add(fourth);
	}
	
	public Iterable<WeightedResourceObject> getLinkedResources(Resource resource) {
		List<WeightedResourceObject> result = new LinkedList<WeightedResourceObject>();
		
		for (IResourceLinker source : linkers) {
			for (WeightedResourceObject obj : source.getLinkedResources(resource))
			result.add(obj);
		}
		
		return result;
	}

}
