package fi.seco.semweb.service.search;

import java.util.Map;

import fi.seco.semweb.service.indexing.item.ItemIndexService;
import fi.seco.semweb.service.search.query.IQueryPart;
import fi.seco.semweb.service.search.resultproducer.IResultProducer;
import fi.seco.semweb.util.map.primitive.IntObjectHashMap;
import gnu.trove.TIntArrayList;

public class ResultSet implements IQueryPart {
	private IntObjectHashMap<Map<Object,Object>> info = new IntObjectHashMap<Map<Object,Object>>();
	private Object cacheKey;
	private int[] ids;
	private ItemIndexService index;

	public ResultSet(ItemIndexService index, IResultProducer s, Object cacheKey) {
		this.index=index;
		this.cacheKey=cacheKey;
		TIntArrayList tmp = new TIntArrayList();
		s.calculate();
		while (s.next()) {
			int id = s.getID();
			tmp.add(id);
			info.put(id,s.getInfo());
		}
		ids=tmp.toNativeArray();
	}

	public int size() {
		return ids.length;
	}

	public Map<Object,Object> getInfo(int id) {
		return info.get(id);
	}

	public int[] ids() {
		return ids;
	}

	public Iterable<String> uris() {
		return index.mapFromIdsToUris(ids);
	}

	public Object getCacheKey() {
		return cacheKey;
	}

	public IResultProducer getResultProducer() {
		return new IResultProducer() {
			private int pointer=-1;
			public int getID() {
				return 0;
			}
			public Map<Object, Object> getInfo() {
				return info.get(ids[pointer]);
			}

			public boolean next() {
				return ++pointer<ids.length;
			}

			public boolean skipTo(int id) {
				while (ids[pointer]<id) if (++pointer>=ids.length) return false;
				return true;
			}
			public void calculate() {
			}
		};
	}
}
