package fi.seco.semweb.service.search.resultproducer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class IntersectionResultProducer implements IResultProducer {
	
	private int curId=-1;
	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(IntersectionResultProducer.class);
	
	private List<IResultProducer> resultProducers;
	
	public IntersectionResultProducer(List<IResultProducer> resultProducers) {
		setResultProducers(resultProducers);
	}

	public IntersectionResultProducer() {
		resultProducers = new ArrayList<IResultProducer>();
	}
	
	public void setResultProducers(List<IResultProducer> resultProducers) {
		this.resultProducers=resultProducers;
	}
	
	public void addResultProducer(IResultProducer resultProducer) {
		resultProducers.add(resultProducer);
	}
	
	public void addResultProducers(List<IResultProducer> resultProducers) {
		this.resultProducers.addAll(resultProducers);
	}
	
	public int getID() {
		return curId;
	}

	public Map<Object, Object> getInfo() {
		Map<Object, Object> info=new HashMap<Object, Object>();
		for (IResultProducer resultProducer : resultProducers) {
			if (resultProducer.getInfo()!=null) info.putAll(resultProducer.getInfo());
		}
		return info.isEmpty() ? null : info;
	}

	public boolean next() {
		if (!resultProducers.get(0).next()) return false;
		curId=resultProducers.get(0).getID();
		boolean isEqual;
		do {
			isEqual=true;
			for (IResultProducer resultProducer : resultProducers) {
				if (!resultProducer.skipTo(curId)) return false;
				int tmp = resultProducer.getID();
				if (tmp>curId) {
					curId=tmp;
					isEqual=false;
				} else if (tmp<curId) isEqual=false;
			}
		} while (!isEqual);
		return true;
	}

	public boolean skipTo(int id) {
		if (curId>=id) return true;
		if (!resultProducers.get(0).skipTo(id)) return false;
		curId=resultProducers.get(0).getID();
		boolean isEqual;
		do {
			isEqual=true;
			for (IResultProducer resultProducer : resultProducers) {
				if (!resultProducer.skipTo(curId)) return false;
				int tmp = resultProducer.getID();
				if (tmp>curId) {
					curId=tmp;
					isEqual=false;
				} else if (tmp<curId) isEqual=false;
			}
		} while (!isEqual);
		return true;
	}

	public void calculate() {
		for (IResultProducer resultProducer : resultProducers) resultProducer.calculate();
	}

}
