package fi.seco.semweb.service.search.resultproducer;

import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;

import fi.seco.semweb.service.indexing.item.ItemIndexService;
import fi.seco.semweb.service.search.query.ILuceneSelectorState;
import fi.seco.semweb.util.lucene.NonExceptingIndexReader;
import fi.seco.semweb.util.lucene.NonExceptingIndexSearcher;
import fi.seco.semweb.util.lucene.NonExceptingScorer;

public class LuceneQueryResultProducer implements IResultProducer {

	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(LuceneQueryResultProducer.class);

	private BooleanQuery bq = new BooleanQuery();
	private NonExceptingScorer scorer;
	private ItemIndexService index;

	public LuceneQueryResultProducer(ItemIndexService index,Collection<ILuceneSelectorState> luceneSelectors) {
		this.index=index;
		for (ILuceneSelectorState is : luceneSelectors) bq.add(is.getQuery(),BooleanClause.Occur.MUST);
	}
	public int getID() {
		return scorer.doc();
	}

	public Map<Object, Object> getInfo() {
		Map<Object,Object> ret = new HashMap<Object,Object>();
		NonExceptingIndexReader reader = index.getLuceneReader();
		ret.put("luceneDocument", reader.document(scorer.doc()));
		ret.put("luceneScore", scorer.score());
		return ret;
	}

	public void calculate() {
		try {
			log.debug("Querying Lucene with "+bq.toString());
			NonExceptingIndexSearcher tmp = index.getLuceneSearcher();
			scorer = new NonExceptingScorer(bq.weight(tmp.getSearcher()).scorer(tmp.getIndexReader()));
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public boolean next() {
		return scorer.next();
	}

	public boolean skipTo(int id) {
		if (scorer.doc()==id) return true;
		return scorer.skipTo(id);
	}

}
