package fi.seco.semweb.service.search.resultproducer;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fi.seco.semweb.service.search.ResultSet;
import fi.seco.semweb.service.search.ItemSearchController;
import fi.seco.semweb.service.search.query.ISQLSelectorState;
import fi.seco.semweb.util.map.primitive.ObjectIntHashMap;
import fi.seco.semweb.util.sql.NonExceptingResultSet;
import gnu.trove.TObjectIntIterator;

public class SQLQueryResultProducer implements IFilteringResultProducer {

	private NonExceptingResultSet rs;
	private Set<Object> cacheKey;
	
	private static final Log log = LogFactory.getLog(SQLQueryResultProducer.class);	
	private ItemSearchController control;
	private Collection<ISQLSelectorState> states;
	private ObjectIntHashMap<String> columns = new ObjectIntHashMap<String>(); 

	private ResultSet rsToFilter;

	public SQLQueryResultProducer(ItemSearchController control, Collection<ISQLSelectorState> states) {
		this.control=control;
		this.states=states;
	}	

	public void calculate() {
		SQLInfo sqlInfo = buildWhatWhere(states);
		String query;
		String filterTable=null;
		if (rsToFilter!=null) {
			if (rsToFilter.getCacheKey()!=null) filterTable=control.getResultAsSQLTempTable(rsToFilter);
			else filterTable = control.createTempTable(rsToFilter.ids());
			query = "SELECT id"+sqlInfo.getWhat()+" FROM "+sqlInfo.getFrom()+", "+filterTable+" WHERE "+sqlInfo.getFirstFrom()+".id="+filterTable+".id AND "+sqlInfo.getWhere();
		} else query="SELECT id"+sqlInfo.getWhat()+" FROM "+sqlInfo.getFrom()+" WHERE "+sqlInfo.getWhere();
		log.debug("Querying SQL with: "+query);
		rs = control.getSqlConn().createStatement().executeQuery(query);
		if (rsToFilter!=null && rsToFilter.getCacheKey()==null) control.removeTempTable(filterTable);
	}	
	
	public int getID() {
		return rs.getInt(1);
	}

	public Map<Object, Object> getInfo() {
		int id = rs.getInt(1);
		Map<Object,Object> map = null;
		if (rsToFilter!=null) map=rsToFilter.getInfo(id);
		if (map==null) map = new HashMap<Object,Object>();
		TObjectIntIterator t = columns.iterator();
		while (t.hasNext()) {
			t.advance();
			map.put(t.key(),rs.getObject(t.value()));
		}
		return map;
	}

	public boolean next() {
		return rs.next();
	}

	public boolean skipTo(int id) {
		if (!rs.isBeforeFirst() && rs.getInt(1)>=id) return true;
		while (rs.next()) if (rs.getInt(1)>=id) return true;
		return false;
	}

	public Object getCacheKey() {
		return cacheKey;
	}

	public void initializeToFilter(ResultSet rsToFilter) {
		this.rsToFilter=rsToFilter;
	}

	private static class SQLInfo {
		private String firstFrom, what, where, from;
		
		public SQLInfo(String firstFrom, String what, String where, String from) {
			this.firstFrom=firstFrom; this.what=what; this.where=where; this.from=from;
		}
		
		public String getFirstFrom() {
			return firstFrom;
		}
		public String getFrom() {
			return from;
		}

		public String getWhat() {
			return what;
		}

		public String getWhere() {
			return where;
		}
	}
	
	private static SQLInfo buildWhatWhere(Collection<ISQLSelectorState> sqlSelectors) {
		String firstFrom;
		StringBuilder what = new StringBuilder();
		StringBuilder where = new StringBuilder();
		StringBuilder whereId = new StringBuilder();
		StringBuilder from = new StringBuilder();
		StringBuilder groupBy = new StringBuilder();
		Iterator<ISQLSelectorState> i = sqlSelectors.iterator();
		boolean containsAggregate = false;
			ISQLSelectorState s = i.next();
			firstFrom=s.getFrom();
			String whatTmp = s.getWhat();
			String aggregateWhatTmp = s.getAggregateWhat();
			if (whatTmp!=null) {
				what.append(", ");
				what.append(whatTmp);
				groupBy.append(", ");
				groupBy.append(whatTmp);
			}
			if (aggregateWhatTmp!=null) {
				containsAggregate=true;
				what.append(", ");
				what.append(aggregateWhatTmp);
			}
			where.append(s.getWhere());
			from.append(s.getFrom());
			while(i.hasNext()) {
				where.append(" AND ");
				from.append(", ");
				s = i.next();
				whatTmp = s.getWhat();
				aggregateWhatTmp = s.getAggregateWhat();
				if (whatTmp!=null) {
					what.append(", ");
					what.append(whatTmp);
					groupBy.append(", ");
					groupBy.append(whatTmp);
				}
				if (aggregateWhatTmp!=null) {
					containsAggregate=true;
					what.append(", ");
					what.append(aggregateWhatTmp);
				}
				where.append(s.getWhere());
				whereId.append(firstFrom);
				whereId.append(".id=");
				whereId.append(s.getFrom());
				whereId.append(".id");
				whereId.append(" AND ");
				from.append(s.getFrom());
			}
		if (containsAggregate) {
			where.append(" GROUP BY id");
			where.append(groupBy);
		}
		whereId.append(where);
		return new SQLInfo(firstFrom,what.toString(),whereId.toString(),from.toString());
	}

}
