package fi.seco.semweb.service.view.categorytree;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import fi.seco.semweb.service.indexing.tree.model.IndexedNode;
import fi.seco.semweb.service.search.query.ISQLSelectorState;


public class CategoryTreeSelectorState implements ISQLSelectorState {

	private Set<IndexedNode<?>> selectedCategories=new HashSet<IndexedNode<?>>();
	private String itemTable;

	public CategoryTreeSelectorState(CategoryTreeIndexService<?> index) {
		itemTable=index.itemTable;
	}

	public CategoryTreeSelectorState(CategoryTreeIndexService<?> index, IndexedNode<?> selected) {
		this(index);
		selectedCategories.add(selected);
	}

	public CategoryTreeSelectorState(String prefix) {
		itemTable=prefix+"_itemcategory";
	}

	public String getWhere() {
		StringBuilder sb = new StringBuilder();
		sb.append('(');
		Iterator<IndexedNode<?>> ci = selectedCategories.iterator();
		while (ci.hasNext()) {
			IndexedNode<?> category = ci.next();
			sb.append('(');
			sb.append(itemTable);
			sb.append(".cat>=");
			sb.append(category.getNodeId());
			sb.append(" AND ");
			sb.append(itemTable);
			sb.append(".cat<=");
			sb.append(category.getNodeid_end());
			sb.append(')');
			if (ci.hasNext()) sb.append(" OR ");
		}
		sb.append(')');
		return sb.toString();
	}

	public String getWhat() {
		return null;
	}

	public String[] getExtractColumnNames() {
		return new String[] {itemTable+"_degree"};
	}

	public String getAggregateWhat() {
		StringBuilder sb = new StringBuilder();
		sb.append("MAX(");
		sb.append(itemTable);
		sb.append(".degree) as ");
		sb.append(itemTable);
		sb.append("_degree");
		return sb.toString();
	}

	public String getFrom() {
		return itemTable;
	}

	public String getCacheKey() {
		StringBuilder sb = new StringBuilder();
		sb.append(itemTable);
		sb.append(':');
		for (IndexedNode<?> c : selectedCategories) {
			sb.append(c.getNodeId());
			sb.append(';');
		}
		return sb.toString();
	}

	public Set<IndexedNode<?>> getCategories() {
		return selectedCategories;
	}

	public CategoryTreeSelectorState addCategory(IndexedNode<?> category) {
		this.selectedCategories.add(category);
		return this;
	}

}
