package fi.seco.semweb.service.view.keyword;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Field;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.StmtIterator;

import fi.seco.semweb.service.indexing.item.IItemLuceneIndexer;
import fi.seco.semweb.util.JenaHelper;


public class KeywordIndexer implements IItemLuceneIndexer {
	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(KeywordIndexer.class);

	private String field;
	private Set<Property> properties=new HashSet<Property>();
	private Model model;
	
	public KeywordIndexer() {}; // Don't use this, it's just needed for HiveMind
	
	public KeywordIndexer(String field, Model model) {
		this.field=field;
		this.model=model;
	}
	
	public KeywordIndexer(String field, Property property, Model model) {
		this(field,model);
		this.properties.add(property);
	}
	
	public KeywordIndexer(String field, String property, Model model) {
		this(field,JenaHelper.p(model,property),model);
	}
	
	public void setProperties(String property) {
		properties.add(JenaHelper.p(model,property));
	}
	
	public Collection<Field> getIndexFields(String resourceUri, int id) {
		Collection<Field> fields = new ArrayList<Field>();
		for (Property property : properties)
			for (StmtIterator si = model.createResource(resourceUri).listProperties(property);si.hasNext();) {
				String content = si.nextStatement().getString();
				log.debug("Found terms "+content+" for property "+model.shortForm(property.getURI()));
				fields.add(new Field(field,content,Field.Store.YES,Field.Index.TOKENIZED));
			}
		return fields;
	}

}
