package fi.seco.semweb.util.collection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import fi.seco.semweb.util.iterator.ALazyInitializingIterator;
import fi.seco.semweb.util.iterator.BackedIterator;
import fi.seco.semweb.util.iterator.IIterableIterator;

public class IteratorList<E> implements List<E> {

	private List<E> col;
	private Iterator<? extends E> iter;
	private E notAdded;

	@SuppressWarnings("unused")
	private static final Log log = LogFactory.getLog(IteratorList.class);

	public IteratorList(IIterableIterator<? extends E> iter)  {
		this(iter,new ArrayList<E>());
	}

	public IteratorList(Iterator<? extends E> iter) {
		this(iter,new ArrayList<E>());
	}

	public IteratorList(Iterable<? extends E> iterable) {
		this(iterable.iterator(),new ArrayList<E>());
	}

	@SuppressWarnings("unchecked")
	public IteratorList(Iterator<? extends E> iter, List<E> col) {
		if (iter instanceof BackedIterator && ((BackedIterator) iter).getBackingCollection() instanceof List)
			this.col=(List<E>) ((BackedIterator) iter).getBackingCollection();
		else {
			this.iter=iter;
			this.col=col;
		}
	}

	public void collapse() {
		if (notAdded!=null) col.add(notAdded);
		for (;iter.hasNext();) col.add(iter.next());
		iter=null;
	}

	public boolean add(E o) {
		if (!(iter==null)) collapse();
		return col.add(o);
	}

	public boolean addAll(Collection<? extends E> c) {
		if (!(iter==null)) collapse();
		return col.addAll(c);
	}

	public void clear() {
		iter=null;
		col.clear();
	}

	public boolean contains(Object o) {
		if (!(iter==null)) collapse();
		return col.contains(o);
	}

	@SuppressWarnings("unchecked")
	public boolean containsAll(Collection c) {
		if (!(iter==null)) collapse();
		return col.containsAll(c);
	}

	@Override
	public boolean equals(Object o) {
		if (!(iter==null)) collapse();
		return col.equals(o);
	}

	@Override
	public int hashCode() {
		if (!(iter==null)) collapse();
		return col.hashCode();
	}

	public boolean isEmpty() {
		if (!(iter==null)) return !iter.hasNext() || col.isEmpty();
		return col.isEmpty();
	}

	public Iterator<E> iterator() {
		if (!(iter==null) && col.isEmpty()) return new ALazyInitializingIterator<E>() {
			@Override
			public Iterator<? extends E> initialize() {
				if (iter==null) return col.iterator();
				return new Iterator<E>() {

					public boolean hasNext() {
						boolean hasNext = IteratorList.this.iter.hasNext();
						if (!hasNext) {
							if (notAdded!=null) col.add(notAdded);
							IteratorList.this.iter=null;
						}
						return hasNext;
					}

					public E next() {
						if (notAdded!=null) col.add(notAdded);
						notAdded = IteratorList.this.iter.next();
						return notAdded;
					}

					public void remove() {
						notAdded=null;
					}

				};
			}

		};
		else if (!(iter==null)) collapse();
		return col.iterator();
	}

	public boolean remove(Object o) {
		if (!(iter==null)) collapse();
		return col.remove(o);
	}

	@SuppressWarnings("unchecked")
	public boolean removeAll(Collection c) {
		if (!(iter==null)) collapse();
		return col.removeAll(c);
	}

	@SuppressWarnings("unchecked")
	public boolean retainAll(Collection c) {
		if (!(iter==null)) collapse();
		return col.retainAll(c);
	}

	public int size() {
		if (!(iter==null)) collapse();
		return col.size();
	}

	public Object[] toArray() {
		if (!(iter==null)) collapse();
		return col.toArray();
	}

	public <T> T[] toArray(T[] a) {
		if (!(iter==null)) collapse();
		return col.toArray(a);
	}

	public boolean addAll(int index, Collection<? extends E> c) {
		if (!(iter==null)) collapse();
		return col.addAll(index, c);
	}

	public E get(int index) {
		if (!(iter==null) && col.size()<index+1) collapse();
		return col.get(index);
	}

	public E set(int index, E element) {
		if (!(iter==null)) collapse();
		return col.set(index, element);
	}

	public void add(int index, E element) {
		if (!(iter==null) && col.size()<index+1) collapse();
		col.add(index,element);
	}

	public E remove(int index) {
		if (!(iter==null) && col.size()<index+1) collapse();
		return col.remove(index);
	}

	public int indexOf(Object o) {
		if (!(iter==null)) collapse();
		return col.indexOf(o);
	}

	public int lastIndexOf(Object o) {
		if (!(iter==null)) collapse();
		return col.lastIndexOf(o);
	}

	public ListIterator<E> listIterator() {
		if (!(iter==null)) collapse();
		return col.listIterator();
	}

	public ListIterator<E> listIterator(int index) {
		if (!(iter==null)) collapse();
		return col.listIterator(index);
	}

	public List<E> subList(int fromIndex, int toIndex) {
		if (!(iter==null) && col.size()<toIndex+1) collapse();
		return col.subList(fromIndex,toIndex);
	}

}
