package fi.seco.semweb.util.lucene;

import java.io.IOException;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.index.TermFreqVector;
import org.apache.lucene.index.TermPositions;
import org.apache.lucene.index.IndexReader.FieldOption;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

public final class NonExceptingIndexReader {

	private IndexReader reader;
	private static Log log = LogFactory.getLog(NonExceptingIndexReader.class);

	public NonExceptingIndexReader(IndexReader reader) {
		this.reader=reader;
	}

	public final Directory directory() {
		return reader.directory();
	}

	public final int docFreq(Term arg0) {
		try {
			return reader.docFreq(arg0);
		} catch (IOException e) {
			log.error("",e);
			return -1;
		}
	}

	public final Document document(int arg0) {
		try {
			return reader.document(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	@Override
	public final boolean equals(Object o) {
		return reader.equals(o);
	}

	@SuppressWarnings("unchecked")
	public final Collection<? extends Object> getFieldNames(FieldOption arg0) {
		return reader.getFieldNames(arg0);
	}

	public final TermFreqVector getTermFreqVector(int arg0, String arg1) {
		try {
			return reader.getTermFreqVector(arg0, arg1);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermFreqVector[] getTermFreqVectors(int arg0) {
		try {
			return reader.getTermFreqVectors(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final long getVersion() {
		return reader.getVersion();
	}

	public final boolean hasDeletions() {
		return reader.hasDeletions();
	}

	@Override
	public final int hashCode() {
		return reader.hashCode();
	}

	public final boolean hasNorms(String arg0) {
		try {
			return reader.hasNorms(arg0);
		} catch (IOException e) {
			log.error("",e);
			return false;
		}
	}

	public final boolean isCurrent() {
		try {
			return reader.isCurrent();
		} catch (IOException e) {
			log.error("",e);
			return false;
		}
	}

	public final boolean isDeleted(int arg0) {
		return reader.isDeleted(arg0);
	}

	public final int maxDoc() {
		return reader.maxDoc();
	}

	public final void norms(String arg0, byte[] arg1, int arg2) {
		try {
			reader.norms(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final byte[] norms(String arg0) {
		try {
			return reader.norms(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final int numDocs() {
		return reader.numDocs();
	}

	public final void setNorm(int arg0, String arg1, float arg2) {
		try {
			reader.setNorm(arg0, arg1, arg2);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final TermDocs termDocs() {
		try {
			return reader.termDocs();
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermDocs termDocs(Term arg0) {
		try {
			return reader.termDocs(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermPositions termPositions() {
		try {
			return reader.termPositions();
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermPositions termPositions(Term arg0) {
		try {
			return reader.termPositions(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermEnum terms() {
		try {
			return reader.terms();
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	public final TermEnum terms(Term arg0) {
		try {
			return reader.terms(arg0);
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}

	@Override
	public final String toString() {
		return reader.toString();
	}

	public final void close() {
		try {
			reader.close();
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final IndexReader getReader() {
		return reader;
	}

	public final void deleteDocument(int id) {
		try {
			reader.deleteDocument(id);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final static NonExceptingIndexReader open(Directory directory) {
		try {
			return new NonExceptingIndexReader(IndexReader.open(directory));
		} catch (IOException e) {
			log.error("",e);
			return null;
		}
	}


	public static void unlock(FSDirectory directory) {
		try {
			IndexReader.unlock(directory);
		} catch (IOException e) {
			log.error(e);
		}
	}
}
