package fi.seco.semweb.util.lucene;

import java.io.IOException;
import java.io.PrintStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.store.Directory;

public final class NonExceptingIndexWriter {

	private IndexWriter writer;
	private static Log log = LogFactory.getLog(NonExceptingIndexWriter.class);

	public NonExceptingIndexWriter(IndexWriter writer) {
		this.writer=writer;
	}

	public final void addDocument(Document arg0, Analyzer arg1) {
		try {
			writer.addDocument(arg0, arg1);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final void addDocument(Document arg0) {
		try {
			writer.addDocument(arg0);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final void addIndexes(Directory[] arg0) {
		try {
			writer.addIndexes(arg0);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final void addIndexes(IndexReader[] arg0) {
		try {
			writer.addIndexes(arg0);
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final void close() {
		try {
			writer.close();
		} catch (IOException e) {
			log.error("",e);
		}
	}

	public final int docCount() {
		return writer.docCount();
	}

	@Override
	public final boolean equals(Object o) {
		return writer.equals(o);
	}

	public final Analyzer getAnalyzer() {
		return writer.getAnalyzer();
	}

	/*public final long getCommitLockTimeout() {
		return writer.getCommitLockTimeout();
	}*/

	public final Directory getDirectory() {
		return writer.getDirectory();
	}

	public final PrintStream getInfoStream() {
		return writer.getInfoStream();
	}

	public final int getMaxBufferedDocs() {
		return writer.getMaxBufferedDocs();
	}

	public final int getMaxFieldLength() {
		return writer.getMaxFieldLength();
	}

	public final int getMaxMergeDocs() {
		return writer.getMaxMergeDocs();
	}

	public final int getMergeFactor() {
		return writer.getMergeFactor();
	}

	public final Similarity getSimilarity() {
		return writer.getSimilarity();
	}

	public final int getTermIndexInterval() {
		return writer.getTermIndexInterval();
	}

	public final boolean getUseCompoundFile() {
		return writer.getUseCompoundFile();
	}

	public final long getWriteLockTimeout() {
		return writer.getWriteLockTimeout();
	}

	@Override
	public final int hashCode() {
		return writer.hashCode();
	}

	public final void optimize() {
		try {
			writer.optimize();
		} catch (IOException e) {
			log.error("",e);
		}
	}

	/*public final void setCommitLockTimeout(long arg0) {
		writer.setCommitLockTimeout(arg0);
	}*/

	public final void setInfoStream(PrintStream arg0) {
		writer.setInfoStream(arg0);
	}

	public final void setMaxBufferedDocs(int arg0) {
		writer.setMaxBufferedDocs(arg0);
	}

	public final void setMaxFieldLength(int arg0) {
		writer.setMaxFieldLength(arg0);
	}

	public final void setMaxMergeDocs(int arg0) {
		writer.setMaxMergeDocs(arg0);
	}

	public final void setMergeFactor(int arg0) {
		writer.setMergeFactor(arg0);
	}

	public final void setSimilarity(Similarity arg0) {
		writer.setSimilarity(arg0);
	}

	public final void setTermIndexInterval(int arg0) {
		writer.setTermIndexInterval(arg0);
	}

	public final void setUseCompoundFile(boolean arg0) {
		writer.setUseCompoundFile(arg0);
	}

	public final void setWriteLockTimeout(long arg0) {
		writer.setWriteLockTimeout(arg0);
	}

	@Override
	public final String toString() {
		return writer.toString();
	}

	/*public final int[] truncated() {
		return writer.truncated();
	}*/

}
