package fi.seco.semweb.view.js.widget.tree;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.lucene.index.Term;
import org.apache.lucene.search.PrefixQuery;

import fi.seco.semweb.service.indexing.tree.model.IndexedNode;
import fi.seco.semweb.service.indexing.tree.model.SearchableCategoryInfo;
import fi.seco.semweb.service.search.ResultSet;
import fi.seco.semweb.service.search.query.IQueryPart;
import fi.seco.semweb.service.view.categorytree.CategoryTreeIndexService;
import fi.seco.semweb.service.view.categorytree.CategoryTreeSelectorState;
import fi.seco.semweb.util.collection.AMappingCollection;
import fi.seco.semweb.util.collection.NullFilteringCollection;
import fi.seco.semweb.view.js.vbsearch.IJSView;
import fi.seco.semweb.view.js.vbsearch.JSSearchController;
import fi.seco.semweb.view.js.widget.tree.model.JSMatch;
import fi.seco.semweb.view.js.widget.tree.model.JSMatchResponse;
import fi.seco.semweb.view.js.widget.tree.model.SearchableJSINode;
import fi.seco.semweb.view.js.widget.tree.model.SearchableJSITreeNode;
import gnu.trove.TIntHashSet;
import gnu.trove.TIntIterator;

public class JSSearchableTreeView implements IJSView {

	private CategoryTreeIndexService<SearchableCategoryInfo> index;
	private JSSearchController control;
	private String widgetId;
	private TIntHashSet sentNodes = new TIntHashSet();
	private TIntHashSet selected = new TIntHashSet();

	public JSSearchableTreeView(CategoryTreeIndexService<SearchableCategoryInfo> index, JSSearchController control, String widgetId) {
		this.index=index;
		this.control=control;
		this.widgetId=widgetId;
		control.registerView(this);
	}

	public Collection<SearchableJSINode> getChildren(int nodeId, final String prefLang) {
		final ResultSet crs = control.getCurrentResultSet();
		return new NullFilteringCollection<SearchableJSINode>(new AMappingCollection<IndexedNode<SearchableCategoryInfo>, SearchableJSINode>(index.getNodeById(nodeId).getSubNodes()) {
			@Override public SearchableJSINode map(IndexedNode<SearchableCategoryInfo> a) {
				if (sentNodes.contains(a.getNodeId())) return null;
				sentNodes.add(a.getNodeId());
				boolean isFolder = false;
				for (IndexedNode<SearchableCategoryInfo> n : a.getSubNodes()) if ((n.getInfo()).isShowEvenIfNoHits() || index.getFilteredItemCount(n, crs)>0) { isFolder=true; break; }
				return new SearchableJSINode(a,widgetId,(a.getInfo()).getLabel(prefLang),index.getFilteredItemCount(a, crs),isFolder);
			}
		});
	}

	public JSMatchResponse match(String text, String prefLang) {
		JSMatchResponse ret = new JSMatchResponse();
		ResultSet crs = control.getCurrentResultSet();
		for (IndexedNode<SearchableCategoryInfo> n : index.match(new PrefixQuery(new Term("label",text))))
			if (index.getFilteredItemCount(n, crs)>0) {
				ret.addMatch(new JSMatch(n,widgetId,text,prefLang));
				SearchableJSINode tn = null;
				while (!sentNodes.contains(n.getNodeId())) {
					boolean isFolder = false;
					for (IndexedNode<SearchableCategoryInfo> n2 : n.getSubNodes()) if ((n2.getInfo()).isShowEvenIfNoHits() || index.getFilteredItemCount(n2, crs)>0) { isFolder=true; break; }
					if (tn==null) tn = new SearchableJSINode(n,widgetId,(n.getInfo()).getLabel(prefLang),index.getFilteredItemCount(n, control.getCurrentResultSet()),isFolder);
					else {
						SearchableJSINode temp = tn;
						tn = new SearchableJSITreeNode(n,widgetId,(n.getInfo()).getLabel(prefLang),index.getFilteredItemCount(n, control.getCurrentResultSet()),isFolder);
						((SearchableJSITreeNode) tn).addChild(temp);
					}
					sentNodes.add(n.getNodeId());
					n = (IndexedNode<SearchableCategoryInfo>)n.getParentNode();
				}
				if (tn!=null) ret.addTree("tn_"+widgetId+"_"+n.getNodeId(), tn);
			}
		return ret;
	}

	public boolean deselect(int nodeId) {
		return selected.remove(nodeId);
	}

	public boolean select(int nodeId) {
		return selected.add(nodeId);
	}

	public IQueryPart getCurrentConstraints() {
		if (selected.size()==0) return null;
		CategoryTreeSelectorState st = new CategoryTreeSelectorState(index);
		TIntIterator t = selected.iterator();
		while (t.hasNext())
			 st.addCategory(index.getNodeById(t.next()));
		return st;
	}

	public String getWidgetId() {
		return widgetId;
	}

	public Object getViewUpdate() {
		Map<String,Integer> ret = new HashMap<String,Integer>();
		TIntIterator t = sentNodes.iterator();
		while (t.hasNext()) {
			int nodeId = t.next();
			ret.put("tn_"+widgetId+"_"+nodeId,index.getFilteredItemCount(index.getNodeById(nodeId), control.getCurrentResultSet()));
		}
		return ret;
	}
}
